@@ -31,13 +31,17 @@
 #include <pulse/pulseaudio.h>
 #endif
 
-typedef void (*audio_monitor_notifier) (gpointer, gdouble);
+typedef void (*audio_monitor_notifier) (gpointer user_data, gdouble monitor_l,
+					gdouble monitor_r);
+typedef void (*audio_playback_cursor_notifier) (gint64 position);
+typedef void (*audio_ready_callback) ();
+typedef void (*audio_volume_change_callback) (gdouble);
 
 #define MAX_RECORDING_TIME_S 30
 #define AUDIO_CHANNELS 2	// Audio system is always stereo
 #define AUDIO_BUF_FRAMES (preferences_get_int (PREF_KEY_AUDIO_BUFFER_LEN))
-#define AUDIO_BUF_BYTES (AUDIO_BUF_FRAMES * FRAME_SIZE (AUDIO_CHANNELS,SF_FORMAT_PCM_16))
-#define AUDIO_SEL_LEN(a) ((a)->sel_start == -1 && (a)->sel_end == -1 ? 0 : (a)->sel_end - (a)->sel_start + 1)
+#define AUDIO_BUF_BYTES (AUDIO_BUF_FRAMES * FRAME_SIZE (AUDIO_CHANNELS,sample_get_internal_format ()))
+#define AUDIO_SEL_LEN (audio.sel_start == -1 && audio.sel_end == -1 ? 0 : audio.sel_end - audio.sel_start + 1)
 
 #define RECORD_LEFT 0x1
 #define RECORD_RIGHT 0x2
@@ -72,63 +76,69 @@ struct audio
   pa_cvolume volume;
   pa_sample_spec sample_spec;
 #endif
+  gboolean float_mode;
   guint32 rate;
   struct idata sample;
   struct sample_info sample_info_src;
   gboolean loop;
   guint32 pos;
-  void (*volume_change_callback) (gpointer, gdouble);
-  void (*ready_callback) (gpointer);
-  gpointer callback_data;
+  audio_ready_callback ready_callback;
+  audio_volume_change_callback volume_change_callback;
   guint32 release_frames;
-  struct job_control control;	//Used to synchronize access to sample, frames, loop and pos members.
+  struct task_control control;	//Used to synchronize access to sample
   gchar *path;
   enum audio_status status;
   gint64 sel_start;		//Space for guint32 and -1
   gint64 sel_end;		//Space for guint32 and -1
   gboolean mono_mix;
   guint record_options;
-  void (*monitor) (void *, gdouble);
+  audio_monitor_notifier monitor_notifier;
   void *monitor_data;
+  gfloat monitor_level_l;
+  gfloat monitor_level_r;
+  audio_playback_cursor_notifier cursor_notifier;
 };
 
-void audio_start_playback (struct audio *);
+extern struct audio audio;
 
-void audio_stop_playback (struct audio *);
+gboolean audio_is_stopped ();
 
-void audio_start_recording (struct audio *, guint, audio_monitor_notifier,
-			    void *);
+void audio_start_playback (audio_playback_cursor_notifier cursor_notifier);
 
-void audio_stop_recording (struct audio *);
+void audio_stop_playback ();
 
-gboolean audio_check (struct audio *);
+void audio_start_recording (guint, audio_monitor_notifier, void *);
 
-void audio_reset_record_buffer (struct audio *, guint, audio_monitor_notifier,
-				void *);
+void audio_stop_recording ();
 
-void audio_init (struct audio *, void (*)(gpointer, gdouble),
-		 void (*)(gpointer), gpointer);
+gboolean audio_check ();
 
-gint audio_run (struct audio *);
+void audio_reset_record_buffer (guint, audio_monitor_notifier, void *);
 
-void audio_destroy (struct audio *);
+void audio_init (audio_ready_callback ready_callback,
+		 audio_volume_change_callback volume_change_callback);
 
-void audio_reset_sample (struct audio *);
+void audio_destroy ();
 
-void audio_set_volume (struct audio *, gdouble);
+void audio_reset_sample ();
 
-void audio_write_to_output (struct audio *, void *, gint);
+void audio_set_volume (gdouble);
 
-void audio_read_from_input (struct audio *, void *, gint);
+void audio_write_to_output (void *, gint);
 
-void audio_prepare (struct audio *, enum audio_status);
+void audio_read_from_input (void *, gint);
 
-void audio_delete_range (struct audio *, guint32, guint32);
-
-guint audio_detect_start (struct audio *);
+void audio_prepare (enum audio_status);
 
 const gchar *audio_name ();
 
 const gchar *audio_version ();
 
+void audio_init_and_wait ();
+
+void audio_set_play_and_wait (struct idata *sample,
+			      struct task_control *control);
+
+void audio_record_and_wait (guint32 options, struct task_control *control);
+
 #endif
