@@ -20,7 +20,7 @@
 
 #include "audio.h"
 
-void audio_finish_recording (struct audio *);
+void audio_finish_recording ();
 
 static struct rtaudio_stream_options STREAM_OPTIONS = {
   .flags = 0,
@@ -29,85 +29,90 @@ static struct rtaudio_stream_options STREAM_OPTIONS = {
 };
 
 void
-audio_stop_playback (struct audio *audio)
+audio_stop_playback ()
 {
   enum audio_status status;
 
-  g_mutex_lock (&audio->control.mutex);
-  status = audio->status;
-  g_mutex_unlock (&audio->control.mutex);
+  g_mutex_lock (&audio.control.controllable.mutex);
+  status = audio.status;
+  g_mutex_unlock (&audio.control.controllable.mutex);
 
   if (status != AUDIO_STATUS_PLAYING)
     {
       return;
     }
 
-  g_mutex_lock (&audio->control.mutex);
-  audio->status = AUDIO_STATUS_STOPPED;
-  g_mutex_unlock (&audio->control.mutex);
-
   debug_print (1, "Stopping playback...");
-  rtaudio_abort_stream (audio->playback_rtaudio);	//Stop and flush buffer
+
+  g_mutex_lock (&audio.control.controllable.mutex);
+  audio.status = AUDIO_STATUS_STOPPED;
+  if (audio.cursor_notifier)
+    {
+      audio.cursor_notifier (-1);
+    }
+  g_mutex_unlock (&audio.control.controllable.mutex);
+
+  rtaudio_abort_stream (audio.playback_rtaudio);	//Stop and flush buffer
 }
 
 void
-audio_start_playback (struct audio *audio)
+audio_start_playback (audio_playback_cursor_notifier cursor_notifier)
 {
-  audio_stop_playback (audio);
-  audio_prepare (audio, AUDIO_STATUS_PLAYING);
+  audio_stop_playback ();
+
+  g_mutex_lock (&audio.control.controllable.mutex);
+  audio.cursor_notifier = cursor_notifier;
+  g_mutex_unlock (&audio.control.controllable.mutex);
+
   debug_print (1, "Starting playback...");
-  rtaudio_start_stream (audio->playback_rtaudio);
+
+  audio_prepare (AUDIO_STATUS_PLAYING);
+
+  rtaudio_start_stream (audio.playback_rtaudio);
 }
 
 void
-audio_stop_recording (struct audio *audio)
+audio_stop_recording ()
 {
-  struct sample_info *sample_info = audio->sample.info;
   enum audio_status status;
 
-  g_mutex_lock (&audio->control.mutex);
-  status = audio->status;
-  g_mutex_unlock (&audio->control.mutex);
+  g_mutex_lock (&audio.control.controllable.mutex);
+  status = audio.status;
+  g_mutex_unlock (&audio.control.controllable.mutex);
 
   if (status != AUDIO_STATUS_RECORDING)
     {
       return;
     }
 
-  g_mutex_lock (&audio->control.mutex);
-  audio->status = AUDIO_STATUS_STOPPING_RECORD;
-  g_mutex_unlock (&audio->control.mutex);
+  g_mutex_lock (&audio.control.controllable.mutex);
+  audio.status = AUDIO_STATUS_STOPPING_RECORD;
+  g_mutex_unlock (&audio.control.controllable.mutex);
 
-  audio_finish_recording (audio);
+  audio_finish_recording ();
 
-  debug_print (1, "Stopping recording (%d frames read)...",
-	       sample_info->frames);
-  rtaudio_abort_stream (audio->record_rtaudio);	//Stop and flush buffer
+  rtaudio_abort_stream (audio.record_rtaudio);	//Stop and flush buffer
 }
 
 void
-audio_start_recording (struct audio *audio, guint options,
+audio_start_recording (guint32 options,
 		       audio_monitor_notifier monitor_notifier,
 		       void *monitor_data)
 {
-  struct sample_info *sample_info;
+  audio_stop_recording ();
+  audio_reset_record_buffer (options, monitor_notifier, monitor_data);
+  audio_prepare (AUDIO_STATUS_RECORDING);
 
-  audio_stop_recording (audio);
-  audio_reset_record_buffer (audio, options, monitor_notifier, monitor_data);
-  audio_prepare (audio, AUDIO_STATUS_RECORDING);
+  debug_print (1, "Starting recording...");
 
-  sample_info = audio->sample.info;
-  debug_print (1, "Starting recording (max %d frames)...",
-	       sample_info->frames);
-
-  rtaudio_start_stream (audio->record_rtaudio);
+  rtaudio_start_stream (audio.record_rtaudio);
 }
 
 int
 audio_record_cb (void *out, void *in, unsigned int frames, double stream_time,
 		 rtaudio_stream_status_t rtaudio_status, void *audio)
 {
-  audio_read_from_input (audio, in, frames);
+  audio_read_from_input (in, frames);
   return 0;
 }
 
@@ -116,7 +121,7 @@ audio_playback_cb (void *out, void *in, unsigned int frames,
 		   double stream_time, rtaudio_stream_status_t rtaudio_status,
 		   void *audio)
 {
-  audio_write_to_output (audio, out, frames);
+  audio_write_to_output (out, frames);
   return 0;
 }
 
@@ -127,7 +132,7 @@ audio_error_cb (rtaudio_error_t err, const char *msg)
 }
 
 void
-audio_init_int (struct audio *audio)
+audio_init_int ()
 {
   gint i, err, dev_id;
   guint buffer_frames;
@@ -136,9 +141,11 @@ audio_init_int (struct audio *audio)
     record_stream_params;
   const rtaudio_api_t *apis = rtaudio_compiled_api ();
   gint api_count = rtaudio_get_num_compiled_apis ();
+  rtaudio_format_t format =
+    audio.float_mode ? RTAUDIO_FORMAT_FLOAT32 : RTAUDIO_FORMAT_SINT16;
 
-  audio->playback_rtaudio = NULL;
-  audio->record_rtaudio = NULL;
+  audio.playback_rtaudio = NULL;
+  audio.record_rtaudio = NULL;
 
   for (i = 0; i < api_count; i++)
     {
@@ -166,21 +173,21 @@ audio_init_int (struct audio *audio)
       return;
     }
 
-  audio->playback_rtaudio = rtaudio_create (apis[i]);
-  if (rtaudio_error (audio->playback_rtaudio))
+  audio.playback_rtaudio = rtaudio_create (apis[i]);
+  if (rtaudio_error (audio.playback_rtaudio))
     {
       error_print ("Error while initilizing playback RtAudio: %s",
-		   rtaudio_error (audio->playback_rtaudio));
+		   rtaudio_error (audio.playback_rtaudio));
       return;
     }
 
-  if (!rtaudio_device_count (audio->playback_rtaudio))
+  if (!rtaudio_device_count (audio.playback_rtaudio))
     {
       error_print ("No devices found");
       goto error_playback;
     }
 
-  dev_id = rtaudio_get_default_output_device (audio->playback_rtaudio);
+  dev_id = rtaudio_get_default_output_device (audio.playback_rtaudio);
   playback_stream_params = (struct rtaudio_stream_parameters)
   {
     .device_id = dev_id,
@@ -188,44 +195,47 @@ audio_init_int (struct audio *audio)
     .first_channel = 0
   };
 
-  dev_info = rtaudio_get_device_info (audio->playback_rtaudio, dev_id);
-  audio->rate = dev_info.preferred_sample_rate;
+  dev_info = rtaudio_get_device_info (audio.playback_rtaudio, dev_id);
+  audio.rate = dev_info.preferred_sample_rate;
   buffer_frames = AUDIO_BUF_FRAMES;
-  err = rtaudio_open_stream (audio->playback_rtaudio, &playback_stream_params,
-			     NULL, RTAUDIO_FORMAT_SINT16,
-			     audio->rate, &buffer_frames,
-			     audio_playback_cb, audio, &STREAM_OPTIONS,
+  err = rtaudio_open_stream (audio.playback_rtaudio, &playback_stream_params,
+			     NULL, format, audio.rate, &buffer_frames,
+			     audio_playback_cb, NULL, &STREAM_OPTIONS,
 			     audio_error_cb);
-  if (err || !rtaudio_is_stream_open (audio->playback_rtaudio))
+  if (err || !rtaudio_is_stream_open (audio.playback_rtaudio))
     {
       error_print
 	("Error occurred while opening the playback RtAudio stream: %s",
-	 rtaudio_error (audio->playback_rtaudio));
+	 rtaudio_error (audio.playback_rtaudio));
       goto error_playback;
     }
 
   debug_print (1,
 	       "Using %s for playback with %d Hz sample rate and %d frames...",
-	       dev_info.name, audio->rate, buffer_frames);
+	       dev_info.name, audio.rate, buffer_frames);
 
-  audio->volume = 1.0;
-  audio->volume_change_callback (audio->callback_data, audio->volume);
+  audio.volume = 1.0;
 
-  audio->record_rtaudio = rtaudio_create (apis[i]);
-  if (rtaudio_error (audio->record_rtaudio))
+  if (audio.volume_change_callback)
+    {
+      audio.volume_change_callback (audio.volume);
+    }
+
+  audio.record_rtaudio = rtaudio_create (apis[i]);
+  if (rtaudio_error (audio.record_rtaudio))
     {
       error_print ("Error while initilizing recording RtAudio: %s",
-		   rtaudio_error (audio->record_rtaudio));
+		   rtaudio_error (audio.record_rtaudio));
       goto error_playback;
     }
 
-  if (!rtaudio_device_count (audio->record_rtaudio))
+  if (!rtaudio_device_count (audio.record_rtaudio))
     {
       error_print ("No devices found");
       goto error_record;
     }
 
-  dev_id = rtaudio_get_default_input_device (audio->record_rtaudio);
+  dev_id = rtaudio_get_default_input_device (audio.record_rtaudio);
   record_stream_params = (struct rtaudio_stream_parameters)
   {
     .device_id = dev_id,
@@ -233,57 +243,56 @@ audio_init_int (struct audio *audio)
     .first_channel = 0
   };
 
-  dev_info = rtaudio_get_device_info (audio->record_rtaudio, dev_id);
+  dev_info = rtaudio_get_device_info (audio.record_rtaudio, dev_id);
   buffer_frames = AUDIO_BUF_FRAMES;
-  err = rtaudio_open_stream (audio->record_rtaudio, NULL,
-			     &record_stream_params, RTAUDIO_FORMAT_SINT16,
-			     audio->rate, &buffer_frames,
-			     audio_record_cb, audio, &STREAM_OPTIONS,
-			     audio_error_cb);
-  if (err || !rtaudio_is_stream_open (audio->record_rtaudio))
+  err = rtaudio_open_stream (audio.record_rtaudio, NULL,
+			     &record_stream_params, format, audio.rate,
+			     &buffer_frames, audio_record_cb, NULL,
+			     &STREAM_OPTIONS, audio_error_cb);
+  if (err || !rtaudio_is_stream_open (audio.record_rtaudio))
     {
       error_print
 	("Error occurred while opening the recording RtAudio stream: %s",
-	 rtaudio_error (audio->record_rtaudio));
+	 rtaudio_error (audio.record_rtaudio));
       goto error_record;
     }
 
   debug_print (1,
 	       "Using %s for recording with %d Hz sample rate and %d frames...",
-	       dev_info.name, audio->rate, buffer_frames);
+	       dev_info.name, audio.rate, buffer_frames);
 
   goto end;
 
 error_record:
-  rtaudio_destroy (audio->record_rtaudio);
-  audio->record_rtaudio = NULL;
+  rtaudio_destroy (audio.record_rtaudio);
+  audio.record_rtaudio = NULL;
 error_playback:
-  rtaudio_destroy (audio->playback_rtaudio);
-  audio->playback_rtaudio = NULL;
+  rtaudio_destroy (audio.playback_rtaudio);
+  audio.playback_rtaudio = NULL;
 end:
-  audio->ready_callback (audio->callback_data);
+  audio.ready_callback ();
 }
 
 void
-audio_destroy_int (struct audio *audio)
+audio_destroy_int ()
 {
-  if (audio_check (audio))
+  if (audio_check ())
     {
-      rtaudio_destroy (audio->playback_rtaudio);
-      rtaudio_destroy (audio->record_rtaudio);
+      rtaudio_destroy (audio.playback_rtaudio);
+      rtaudio_destroy (audio.record_rtaudio);
     }
 }
 
 gboolean
-audio_check (struct audio *audio)
+audio_check ()
 {
-  return audio->playback_rtaudio != NULL;
+  return audio.playback_rtaudio != NULL;
 }
 
 void
-audio_set_volume (struct audio *audio, gdouble volume)
+audio_set_volume (gdouble volume)
 {
-  audio->volume = volume;
+  audio.volume = volume;
 }
 
 const gchar *
