@@ -18,11 +18,14 @@
  *   along with Elektroid. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <stdarg.h>
 #include "backend.h"
 #include "local.h"
 #include "sample.h"
 #include "preferences.h"
+#include "utils.h"
+
+// This can be allocated statically as the only function that uses it is synchronized.
+static guint8 tmp_buffer[BE_MAX_BUFF_SIZE];
 
 struct connector *system_connector = NULL;
 GSList *connectors = NULL;
@@ -30,8 +33,9 @@ GSList *connectors = NULL;
 // When sending a batch of SysEx messages we want the trasfer status to be controlled outside this function.
 // This is what the update parameter is for.
 
-gint backend_tx_sysex_internal (struct backend *, struct sysex_transfer *,
-				gboolean);
+gint backend_tx_sysex_int (struct backend *backend,
+			   struct sysex_transfer *sysex_transfer,
+			   struct controllable *controllable);
 
 void backend_rx_drain_int (struct backend *);
 void backend_destroy_int (struct backend *);
@@ -45,6 +49,85 @@ void backend_fill_devices_array (GArray *);
 static const guint8 BE_MIDI_IDENTITY_REQUEST[] =
   { 0xf0, 0x7e, 0x7f, 6, 1, 0xf7 };
 
+void
+sysex_transfer_set_status (struct sysex_transfer *sysex_transfer,
+			   struct controllable *controllable,
+			   enum sysex_transfer_status status)
+{
+  if (controllable)
+    {
+      g_mutex_lock (&controllable->mutex);
+      sysex_transfer->status = status;
+      g_mutex_unlock (&controllable->mutex);
+    }
+  else
+    {
+      sysex_transfer->status = status;
+    }
+}
+
+enum sysex_transfer_status
+sysex_transfer_get_status (struct sysex_transfer *sysex_transfer,
+			   struct controllable *controllable)
+{
+  enum sysex_transfer_status status;
+  g_mutex_lock (&controllable->mutex);
+  status = sysex_transfer->status;
+  g_mutex_unlock (&controllable->mutex);
+  return status;
+}
+
+static void
+sysex_transfer_init (struct sysex_transfer *sysex_transfer, gint timeout,
+		     GByteArray *data, gboolean batch)
+{
+  sysex_transfer->status = SYSEX_TRANSFER_STATUS_INIT;
+  sysex_transfer->timeout = timeout;
+  sysex_transfer->time = 0;
+  sysex_transfer->batch = batch;
+  sysex_transfer->raw = data;
+  sysex_transfer->err = 0;
+}
+
+void
+sysex_transfer_init_tx (struct sysex_transfer *sysex_transfer,
+			GByteArray *data)
+{
+  sysex_transfer_init (sysex_transfer, -1, data, FALSE);
+}
+
+void
+sysex_transfer_init_rx (struct sysex_transfer *sysex_transfer, gint timeout,
+			gboolean batch)
+{
+  sysex_transfer_init (sysex_transfer, timeout, NULL, batch);
+}
+
+void
+sysex_transfer_init_tx_and_rx (struct sysex_transfer *sysex_transfer,
+			       gint timeout, GByteArray *data)
+{
+  sysex_transfer_init (sysex_transfer, timeout, data, FALSE);
+}
+
+GByteArray *
+sysex_transfer_steal (struct sysex_transfer *sysex_transfer)
+{
+  GByteArray *raw = sysex_transfer->raw;
+  sysex_transfer->raw = NULL;
+  return raw;
+}
+
+void
+sysex_transfer_clear (struct sysex_transfer *sysex_transfer)
+{
+  GByteArray *raw = sysex_transfer_steal (sysex_transfer);
+  if (raw)
+    {
+      g_byte_array_free (raw, TRUE);
+    }
+}
+
 gdouble
 backend_get_storage_stats_percent (struct backend_storage_stats *statfs)
 {
@@ -70,7 +153,7 @@ static gint
 backend_get_fs_operations_name_comparator (gconstpointer a, gconstpointer b)
 {
   const struct fs_operations *ops = a;
-  return strcmp (GET_CLI_NAME (ops), (gchar *) b);
+  return strcmp (ops->name, (gchar *) b);
 }
 
 const struct fs_operations *
@@ -153,17 +236,13 @@ backend_midi_handshake (struct backend *backend)
   usleep (BE_REST_TIME_US);
 }
 
-gint
-backend_tx_sysex_no_status (struct backend *backend,
-			    struct sysex_transfer *transfer)
-{
-  return backend_tx_sysex_internal (backend, transfer, FALSE);
-}
+//Not synchronized
 
 gint
-backend_tx_sysex (struct backend *backend, struct sysex_transfer *transfer)
+backend_tx_sysex (struct backend *backend, struct sysex_transfer *transfer,
+		  struct controllable *controllable)
 {
-  return backend_tx_sysex_internal (backend, transfer, TRUE);
+  return backend_tx_sysex_int (backend, transfer, controllable);
 }
 
 //Synchronized
@@ -172,39 +251,51 @@ gint
 backend_tx (struct backend *backend, GByteArray *tx_msg)
 {
   struct sysex_transfer transfer;
-  transfer.raw = tx_msg;
+
+  sysex_transfer_init_tx (&transfer, tx_msg);
+
   g_mutex_lock (&backend->mutex);
-  backend_tx_sysex (backend, &transfer);
+  backend_tx_sysex (backend, &transfer, NULL);
   g_mutex_unlock (&backend->mutex);
-  free_msg (tx_msg);
+
+  sysex_transfer_clear (&transfer);
+
   return transfer.err;
 }
 
+GByteArray *
+backend_rx (struct backend *backend, gint timeout,
+	    struct controllable *controllable)
+{
+  struct sysex_transfer transfer;
+
+  sysex_transfer_init_rx (&transfer, timeout, FALSE);
+
+  g_mutex_lock (&backend->mutex);
+  backend_rx_sysex (backend, &transfer, controllable);
+  g_mutex_unlock (&backend->mutex);
+
+  return sysex_transfer_steal (&transfer);
+}
+
 //Synchronized
 
 gint
 backend_tx_and_rx_sysex_transfer (struct backend *backend,
 				  struct sysex_transfer *transfer,
-				  gboolean free)
+				  struct controllable *controllable)
 {
-  transfer->batch = FALSE;
-  transfer->err = 0;
-
   g_mutex_lock (&backend->mutex);
 
   if (transfer->raw)
     {
-      backend_tx_sysex (backend, transfer);
-      if (free)
-	{
-	  free_msg (transfer->raw);
-	  transfer->raw = NULL;
-	}
+      backend_tx_sysex (backend, transfer, controllable);
+      sysex_transfer_clear (transfer);
     }
 
   if (!transfer->err)
     {
-      backend_rx_sysex (backend, transfer);
+      backend_rx_sysex (backend, transfer, controllable);
     }
 
   g_mutex_unlock (&backend->mutex);
@@ -213,17 +304,17 @@ backend_tx_and_rx_sysex_transfer (struct backend *backend,
 }
 
 //Synchronized
-//A timeout of 0 means infinity; a negative timeout means the default timeout.
+//A timeout of 0 means infinity; a timeout of -1 means the default timeout.
 
 GByteArray *
 backend_tx_and_rx_sysex (struct backend *backend, GByteArray *tx_msg,
 			 gint timeout)
 {
   struct sysex_transfer transfer;
-  transfer.raw = tx_msg;
-  transfer.timeout = timeout < 0 ? BE_SYSEX_TIMEOUT_MS : timeout;
-  backend_tx_and_rx_sysex_transfer (backend, &transfer, TRUE);
-  return transfer.raw;
+  gint t = timeout == -1 ? BE_SYSEX_TIMEOUT_MS : timeout;
+  sysex_transfer_init_tx_and_rx (&transfer, t, tx_msg);
+  backend_tx_and_rx_sysex_transfer (backend, &transfer, NULL);
+  return sysex_transfer_steal (&transfer);
 }
 
 void
@@ -309,7 +400,7 @@ backend_send_rpn (struct backend *backend, guint8 channel,
   return err;
 }
 
-gint
+static gint
 backend_init_midi (struct backend *backend, const gchar *id)
 {
   debug_print (1, "Initializing backend (%s) to '%s'...",
@@ -366,19 +457,46 @@ backend_check (struct backend *backend)
     case BE_TYPE_MIDI:
       return backend_check_int (backend);
     case BE_TYPE_SYSTEM:
+    case BE_TYPE_NO_MIDI:
       return TRUE;
     default:
       return FALSE;
     }
 }
 
+static guint8 *
+backend_get_sysex_start (guint8 *tmp, ssize_t *size)
+{
+  guint i, total = *size;
+  guint8 *msg_start;
+
+  //Everything is skipped until a 0xf0 is found. This includes every RT MIDI message.
+  msg_start = tmp;
+  for (i = 0; i < total; i++, msg_start++, (*size)--)
+    {
+      if (*msg_start == 0xf0)
+	{
+	  break;
+	}
+    }
+
+  if (debug_level >= 4)
+    {
+      gchar *text = debug_get_hex_data (debug_level, tmp, i);
+      debug_print (4, "Skipping non SysEx data (%d): %s", i, text);
+      g_free (text);
+    }
+
+  return msg_start;
+}
+
 static ssize_t
-backend_rx_raw_loop (struct backend *backend, struct sysex_transfer *transfer)
+backend_rx_raw_loop (struct backend *backend, struct sysex_transfer *transfer,
+		     struct controllable *controllable)
 {
-  ssize_t rx_len, rx_len_msg;
   gchar *text;
-  guint8 tmp[BE_TMP_BUFF_LEN];
-  guint8 *tmp_msg, *data = backend->buffer + backend->rx_len;
+  ssize_t rx_len;
+  guint8 *msg_start;
 
   if (!backend->inputp)
     {
@@ -390,7 +508,7 @@ backend_rx_raw_loop (struct backend *backend, struct sysex_transfer *transfer)
 
   while (1)
     {
-      if (!transfer->active)
+      if (!CONTROLLABLE_IS_NULL_OR_ACTIVE (controllable))
 	{
 	  return -ECANCELED;
 	}
@@ -398,27 +516,30 @@ backend_rx_raw_loop (struct backend *backend, struct sysex_transfer *transfer)
       debug_print (6, "Checking timeout (%d ms, %d ms, %s mode)...",
 		   transfer->time, transfer->timeout,
 		   transfer->batch ? "batch" : "single");
-      if (((transfer->batch && transfer->status == RECEIVING)
+      if (((transfer->batch &&
+	    transfer->status == SYSEX_TRANSFER_STATUS_RECEIVING)
 	   || !transfer->batch) && transfer->timeout > -1
 	  && transfer->time >= transfer->timeout)
 	{
 	  debug_print (1, "Timeout (%d)", transfer->timeout);
-	  gchar *text = debug_get_hex_data (debug_level, backend->buffer,
-					    backend->rx_len);
-	  debug_print (4, "Internal buffer data (%zd): %s", backend->rx_len,
-		       text);
+	  gchar *text =
+	    debug_get_hex_data (debug_level, backend->buffer->data,
+				backend->buffer->len);
+	  debug_print (4, "Internal buffer data (%u): %s",
+		       backend->buffer->len, text);
 	  g_free (text);
 	  return -ETIMEDOUT;
 	}
 
-      rx_len = backend_rx_raw (backend, tmp, BE_TMP_BUFF_LEN);
+      rx_len = backend_rx_raw (backend, tmp_buffer, BE_MAX_BUFF_SIZE);
       if (rx_len < 0)
 	{
 	  return rx_len;
 	}
       if (rx_len == 0)
 	{
-	  if ((transfer->batch && transfer->status == RECEIVING)
+	  if ((transfer->batch &&
+	       transfer->status == SYSEX_TRANSFER_STATUS_RECEIVING)
 	      || !transfer->batch)
 	    {
 	      transfer->time += BE_POLL_TIMEOUT_MS;
@@ -426,28 +547,20 @@ backend_rx_raw_loop (struct backend *backend, struct sysex_transfer *transfer)
 	  continue;
 	}
 
-      //Everything is skipped until a 0xf0 is found. This includes every RT MIDI message.
-      tmp_msg = tmp;
-      if (!backend->rx_len && *tmp_msg != 0xf0)
+      if (debug_level >= 4)
 	{
-	  if (debug_level >= 4)
-	    {
-	      gchar *text = debug_get_hex_data (debug_level, tmp, rx_len);
-	      debug_print (4, "Skipping non SysEx data (%zd): %s", rx_len,
-			   text);
-	      g_free (text);
-	    }
+	  gchar *text = debug_get_hex_data (debug_level, tmp_buffer, rx_len);
+	  debug_print (4, "Read data (%zd): %s", rx_len, text);
+	  g_free (text);
+	}
 
-	  tmp_msg++;
-	  rx_len_msg = 1;
-	  for (gint i = 1; i < rx_len; i++, tmp_msg++, rx_len_msg++)
-	    {
-	      if (*tmp_msg == 0xf0)
-		{
-		  break;
-		}
-	    }
-	  rx_len -= rx_len_msg;
+      if (backend->buffer->len)
+	{
+	  msg_start = tmp_buffer;
+	}
+      else
+	{
+	  msg_start = backend_get_sysex_start (tmp_buffer, &rx_len);
 	}
 
       if (rx_len == 0)
@@ -458,31 +571,33 @@ backend_rx_raw_loop (struct backend *backend, struct sysex_transfer *transfer)
 
       if (rx_len > 0)
 	{
-	  memcpy (backend->buffer + backend->rx_len, tmp_msg, rx_len);
-	  backend->rx_len += rx_len;
-	  break;
-	}
-
-      if (rx_len < 0)
-	{
+	  guint8 *v = msg_start;
+	  for (guint i = 0; i < rx_len; i++, v++)
+	    {
+	      if (*v == 0xf0 || *v == 0xf7 || (*v & 0xf0) != 0xf0)
+		{
+		  g_byte_array_append (backend->buffer, v, 1);
+		}
+	    }
+	  if (debug_level >= 3)
+	    {
+	      text = debug_get_hex_data (debug_level, msg_start, rx_len);
+	      debug_print (3, "Queued data (%zu): %s", rx_len, text);
+	      g_free (text);
+	    }
 	  break;
 	}
     }
 
-  if (debug_level >= 3)
-    {
-      text = debug_get_hex_data (debug_level, data, rx_len);
-      debug_print (3, "Queued data (%zu): %s", rx_len, text);
-      g_free (text);
-    }
 
-  return rx_len;
+  return backend->buffer->len;
 }
 
 //Access to this function must be synchronized.
 
 gint
-backend_rx_sysex (struct backend *backend, struct sysex_transfer *transfer)
+backend_rx_sysex (struct backend *backend, struct sysex_transfer *transfer,
+		  struct controllable *controllable)
 {
   gint next_check, len, i;
   guint8 *b;
@@ -490,21 +605,21 @@ backend_rx_sysex (struct backend *backend, struct sysex_transfer *transfer)
 
   transfer->err = 0;
   transfer->time = 0;
-  transfer->active = TRUE;
-  transfer->status = WAITING;
-  transfer->raw = g_byte_array_sized_new (BE_INT_BUF_LEN);
+  transfer->raw = g_byte_array_sized_new (BE_INT_BUFF_SIZE);
+  sysex_transfer_set_status (transfer, controllable,
+			     SYSEX_TRANSFER_STATUS_WAITING);
 
   next_check = 0;
   while (1)
     {
-      if (backend->rx_len == next_check)
+      if (backend->buffer->len == next_check)
 	{
 	  debug_print (4, "Reading from MIDI device...");
 	  if (transfer->batch)
 	    {
 	      transfer->time = 0;
 	    }
-	  rx_len = backend_rx_raw_loop (backend, transfer);
+	  rx_len = backend_rx_raw_loop (backend, transfer, controllable);
 
 	  if (rx_len == -ENODATA || rx_len == -ETIMEDOUT
 	      || rx_len == -ECANCELED)
@@ -530,10 +645,11 @@ backend_rx_sysex (struct backend *backend, struct sysex_transfer *transfer)
 	  debug_print (4, "Reading from internal buffer...");
 	}
 
-      transfer->status = RECEIVING;
+      sysex_transfer_set_status (transfer, controllable,
+				 SYSEX_TRANSFER_STATUS_RECEIVING);
       len = -1;
-      b = backend->buffer + next_check;
-      for (; next_check < backend->rx_len; next_check++, b++)
+      b = backend->buffer->data + next_check;
+      for (; next_check < backend->buffer->len; next_check++, b++)
 	{
 	  if (*b == 0xf7)
 	    {
@@ -543,17 +659,17 @@ backend_rx_sysex (struct backend *backend, struct sysex_transfer *transfer)
 	    }
 	}
 
-      //We filter out whatever SysEx message not suitable for Elektroid.
+      //We filter out any SysEx message not suitable.
 
       if (len > 0)
 	{
 	  //Filter out everything until an 0xf0 is found.
-	  b = backend->buffer;
+	  b = backend->buffer->data;
 	  for (i = 0; i < len && *b != 0xf0; i++, b++);
 	  if (i > 0 && debug_level >= 4)
 	    {
-	      gchar *text = debug_get_hex_data (debug_level, backend->buffer,
-						i);
+	      gchar *text = debug_get_hex_data (debug_level,
+						backend->buffer->data, i);
 	      debug_print (4, "Skipping non SysEx data in buffer (%d): %s",
 			   i, text);
 	      g_free (text);
@@ -561,9 +677,8 @@ backend_rx_sysex (struct backend *backend, struct sysex_transfer *transfer)
 	  debug_print (3, "Copying %d bytes...", len - i);
 	  g_byte_array_append (transfer->raw, b, len - i);
 
-	  backend->rx_len -= len;
-	  memmove (backend->buffer, backend->buffer + next_check,
-		   backend->rx_len);
+	  g_byte_array_remove_range (backend->buffer, 0, len);
+
 	  transfer->err = 0;
 	  next_check = 0;
 
@@ -619,8 +734,10 @@ end:
 	}
 
     }
-  transfer->active = FALSE;
-  transfer->status = FINISHED;
+
+  sysex_transfer_set_status (transfer, controllable,
+			     SYSEX_TRANSFER_STATUS_FINISHED);
+
   return transfer->err;
 }
 
@@ -630,15 +747,15 @@ void
 backend_rx_drain (struct backend *backend)
 {
   struct sysex_transfer transfer;
-  transfer.timeout = 1000;
-  transfer.batch = FALSE;
+
+  sysex_transfer_init_rx (&transfer, 1000, FALSE);
 
   debug_print (2, "Draining buffers...");
-  backend->rx_len = 0;
+  backend->buffer->len = 0;
   backend_rx_drain_int (backend);
-  while (!backend_rx_sysex (backend, &transfer))
+  while (!backend_rx_sysex (backend, &transfer, NULL))
     {
-      free_msg (transfer.raw);
+      sysex_transfer_clear (&transfer);
     }
 }
 
@@ -649,20 +766,63 @@ backend_get_path_type (struct backend *backend)
     PATH_INTERNAL;
 }
 
+static gint
+be_device_compare (gconstpointer a, gconstpointer b)
+{
+  const struct backend_device *da = a;
+  const struct backend_device *db = b;
+
+  if (da->type == db->type)
+    {
+      return strcmp (da->name, db->name);
+    }
+  else
+    {
+      return da->type < db->type ? -1 : da->type > db->type ? +1 : 0;
+    }
+}
+
 GArray *
 backend_get_devices ()
 {
+  guint id;
+  GSList *c;
+  GArray *devices;
   struct backend_device *backend_device;
-  GArray *devices =
-    g_array_new (FALSE, FALSE, sizeof (struct backend_device));
 
+  devices = g_array_new (FALSE, FALSE, sizeof (struct backend_device));
+
+  //System
   backend_device = g_malloc (sizeof (struct backend_device));
   backend_device->type = BE_TYPE_SYSTEM;
   snprintf (backend_device->id, LABEL_MAX, "%s", BE_SYSTEM_ID);
   snprintf (backend_device->name, LABEL_MAX, "%s", g_get_host_name ());
   g_array_append_vals (devices, backend_device, 1);
 
+  //Actually present devices
   backend_fill_devices_array (devices);
+
+  //Devices that need a manual handshake. These may or may not be a present device.
+  id = 0;
+  c = connectors;
+  while (c)
+    {
+      struct connector *connector = c->data;
+      if (connector->options & CONNECTOR_OPTION_NO_MIDI)
+	{
+	  backend_device = g_malloc (sizeof (struct backend_device));
+	  backend_device->type = BE_TYPE_NO_MIDI;
+	  snprintf (backend_device->id, LABEL_MAX, "%s", connector->name);
+	  snprintf (backend_device->name, LABEL_MAX, "%s",
+		    connector->device_name);
+	  g_array_append_vals (devices, backend_device, 1);
+	  id++;
+	}
+      c = c->next;
+    }
+
+  g_array_sort (devices, be_device_compare);
+
   return devices;
 }
 
@@ -675,11 +835,10 @@ gint
 backend_init_connector (struct backend *backend,
 			struct backend_device *device,
 			const gchar *conn_name,
-			struct sysex_transfer *sysex_transfer)
+			struct controllable *controllable)
 {
   gint err;
   GSList *list = NULL, *iterator;
-  gboolean active = TRUE;
   GSList *c;
 
   if (device->type == BE_TYPE_SYSTEM)
@@ -689,6 +848,29 @@ backend_init_connector (struct backend *backend,
       return system_connector->handshake (backend);
     }
 
+  if (device->type == BE_TYPE_NO_MIDI)
+    {
+      for (iterator = connectors; iterator; iterator = iterator->next)
+	{
+	  const struct connector *c = iterator->data;
+	  if (!strcmp (c->name, device->id))
+	    {
+	      if (conn_name && strcmp (c->name, conn_name))
+		{
+		  error_print ("Unexpected connector");
+		  return -ENODEV;
+		}
+
+	      if (!c->handshake (backend))
+		{
+		  backend->conn_name = c->name;
+		  backend->type = BE_TYPE_NO_MIDI;
+		  return 0;
+		}
+	    }
+	}
+    }
+
   err = backend_init_midi (backend, device->id);
   if (err)
     {
@@ -722,6 +904,12 @@ backend_init_connector (struct backend *backend,
       c = c->next;
     }
 
+  if (!CONTROLLABLE_IS_NULL_OR_ACTIVE (controllable))
+    {
+      err = -ECANCELED;
+      goto end;
+    }
+
   if (!conn_name)
     {
       backend_midi_handshake (backend);
@@ -732,27 +920,20 @@ backend_init_connector (struct backend *backend,
     {
       const struct connector *c = iterator->data;
 
-      if (sysex_transfer)
-	{
-	  g_mutex_lock (&sysex_transfer->mutex);
-	  active = sysex_transfer->active;
-	  g_mutex_unlock (&sysex_transfer->mutex);
-	}
-
-      if (!active)
+      if (!CONTROLLABLE_IS_NULL_OR_ACTIVE (controllable))
 	{
 	  err = -ECANCELED;
 	  goto end;
 	}
 
-      debug_print (1, "Testing %s connector (%sstandard handshake)...",
-		   c->name, c->standard ? "" : "non ");
+      debug_print (1, "Testing %s connector (options 0x%08x)...",
+		   c->name, c->options);
 
       if (conn_name)
 	{
 	  if (!strcmp (conn_name, c->name))
 	    {
-	      if (c->standard)
+	      if (!(c->options & CONNECTOR_OPTION_NO_MIDI))
 		{
 		  backend_midi_handshake (backend);
 		}
