@@ -23,6 +23,11 @@
 #define BE_DEVICE_NAME "hw:%d,%d"
 #define BE_DEVICE_NAME_SUB "hw:%d,%d,%d"
 
+void
+sysex_transfer_set_status (struct sysex_transfer *sysex_transfer,
+			   struct controllable *controllable,
+			   enum sysex_transfer_status status);
+
 void
 backend_destroy_int (struct backend *backend)
 {
@@ -72,10 +77,8 @@ backend_init_int (struct backend *backend, const gchar *id)
   backend->inputp = NULL;
   backend->outputp = NULL;
   backend->pfds = NULL;
-  backend->rx_len = 0;
-  backend->buffer = NULL;
 
-  backend->buffer = g_malloc (sizeof (guint8) * BE_INT_BUF_LEN);
+  backend->buffer = g_byte_array_sized_new (BE_INT_BUFF_SIZE);
 
   if ((err = snd_rawmidi_open (&backend->inputp, &backend->outputp, id,
 			       SND_RAWMIDI_NONBLOCK | SND_RAWMIDI_SYNC)) < 0)
@@ -159,24 +162,23 @@ backend_tx_raw (struct backend *backend, guint8 *data, guint len)
 }
 
 gint
-backend_tx_sysex_internal (struct backend *backend,
-			   struct sysex_transfer *transfer, gboolean update)
+backend_tx_sysex_int (struct backend *backend,
+		      struct sysex_transfer *transfer,
+		      struct controllable *controllable)
 {
   ssize_t tx_len;
   guint total;
   guint len;
   guchar *b;
 
-  if (update)
-    {
-      transfer->err = 0;
-      transfer->active = TRUE;
-      transfer->status = SENDING;
-    }
+  transfer->err = 0;
+  sysex_transfer_set_status (transfer, controllable,
+			     SYSEX_TRANSFER_STATUS_SENDING);
 
   b = transfer->raw->data;
   total = 0;
-  while (total < transfer->raw->len && transfer->active)
+  while (total < transfer->raw->len &&
+	 CONTROLLABLE_IS_NULL_OR_ACTIVE (controllable))
     {
       len = transfer->raw->len - total;
       if (len > BE_MAX_TX_LEN)
@@ -194,7 +196,7 @@ backend_tx_sysex_internal (struct backend *backend,
       total += len;
     }
 
-  if (!transfer->active)
+  if (!CONTROLLABLE_IS_NULL_OR_ACTIVE (controllable))
     {
       transfer->err = -ECANCELED;
     }
@@ -207,11 +209,9 @@ backend_tx_sysex_internal (struct backend *backend,
       g_free (text);
     }
 
-  if (update)
-    {
-      transfer->active = FALSE;
-      transfer->status = FINISHED;
-    }
+  sysex_transfer_set_status (transfer, controllable,
+			     SYSEX_TRANSFER_STATUS_FINISHED);
+
   return transfer->err;
 }
 
