@@ -18,8 +18,10 @@
  *   along with Elektroid. If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <rtmidi_c.h>
 #include "backend.h"
-#include "rtmidi_c.h"
+
+#define RTMIDI_TMP_BUFF_SIZE 1024
 
 #if defined(__linux__)
 #define ELEKTROID_RTMIDI_API RTMIDI_API_LINUX_ALSA
@@ -34,6 +36,11 @@
 
 #define INPUT_OUTPUT_SEPARATOR " :: "
 
+void
+sysex_transfer_set_status (struct sysex_transfer *sysex_transfer,
+			   struct controllable *controllable,
+			   enum sysex_transfer_status status);
+
 void
 backend_destroy_int (struct backend *backend)
 {
@@ -66,6 +73,12 @@ backend_init_int (struct backend *backend, const gchar *id)
   gchar oportname[LABEL_MAX];
   gint iportnamelen, oportnamelen;
 
+#if defined(__linux__)
+  warn_print
+    ("On Linux, using RtMidi requires setting the ALSA MIDI buffer to %d",
+     BE_TMP_BUFF_SIZE);
+#endif
+
   backend->inputp = NULL;
   backend->outputp = NULL;
   backend->buffer = NULL;
@@ -114,15 +127,14 @@ backend_init_int (struct backend *backend, const gchar *id)
 	    {
 	      backend->inputp = rtmidi_in_create (ELEKTROID_RTMIDI_API,
 						  PACKAGE_NAME,
-						  BE_INT_BUF_LEN);
+						  BE_INT_BUFF_SIZE);
 	      rtmidi_in_ignore_types (backend->inputp, false, true, true);
 	      rtmidi_open_port (backend->inputp, i, PACKAGE_NAME);
 	      backend->outputp =
 		rtmidi_out_create (ELEKTROID_RTMIDI_API, PACKAGE_NAME);
 	      rtmidi_open_port (backend->outputp, j, PACKAGE_NAME);
-	      backend->rx_len = 0;
 
-	      backend->buffer = g_malloc (sizeof (guint8) * BE_INT_BUF_LEN);
+	      backend->buffer = g_byte_array_sized_new (BE_INT_BUFF_SIZE);
 	      goto cleanup_output;
 	    }
 	}
@@ -140,15 +152,13 @@ cleanup_input:
 }
 
 gint
-backend_tx_sysex_internal (struct backend *backend,
-			   struct sysex_transfer *transfer, gboolean update)
+backend_tx_sysex_int (struct backend *backend,
+		      struct sysex_transfer *transfer,
+		      struct controllable *controllable)
 {
-  if (update)
-    {
-      transfer->err = 0;
-      transfer->active = TRUE;
-      transfer->status = SENDING;
-    }
+  transfer->err = 0;
+  sysex_transfer_set_status (transfer, controllable,
+			     SYSEX_TRANSFER_STATUS_SENDING);
 
   rtmidi_out_send_message (backend->outputp, transfer->raw->data,
 			   transfer->raw->len);
@@ -162,42 +172,51 @@ backend_tx_sysex_internal (struct backend *backend,
       g_free (text);
     }
 
-  if (update)
-    {
-      transfer->active = FALSE;
-      transfer->status = FINISHED;
-    }
+  sysex_transfer_set_status (transfer, controllable,
+			     SYSEX_TRANSFER_STATUS_FINISHED);
+
   return transfer->err;
 }
 
 ssize_t
 backend_tx_raw (struct backend *backend, guint8 *data, guint len)
 {
+  GByteArray *msg;
   struct sysex_transfer transfer;
-  transfer.raw = g_byte_array_sized_new (len);
-  g_byte_array_append (transfer.raw, data, len);
-  backend_tx_sysex_internal (backend, &transfer, TRUE);
+
+  msg = g_byte_array_sized_new (len);
+  g_byte_array_append (msg, data, len);
+
+  sysex_transfer_init_tx (&transfer, msg);
+  backend_tx_sysex_int (backend, &transfer, NULL);
+  sysex_transfer_clear (&transfer);
+
   return transfer.err ? transfer.err : len;
 }
 
 void
 backend_rx_drain_int (struct backend *backend)
 {
+  guint8 tmp[RTMIDI_TMP_BUFF_SIZE];
   while (1)
     {
-      size_t len = BE_INT_BUF_LEN;
-      rtmidi_in_get_message (backend->inputp, backend->buffer, &len);
-      if (len == 0)
+      size_t size = RTMIDI_TMP_BUFF_SIZE;
+      //The size is not important here. As long as there are no more data.
+      rtmidi_in_get_message (backend->inputp, tmp, &size);
+      if (size == 0)
 	{
 	  break;
 	}
     }
 }
 
+// rtmidi_in_get_message requires enough space in the buffer to store the message.
+// As this is unknown, quite a large buffer needs to be allocated for this.
+
 ssize_t
-backend_rx_raw (struct backend *backend, guint8 *buffer, guint len)
+backend_rx_raw (struct backend *backend, guint8 *buffer, guint s)
 {
-  size_t size = len;
+  size_t size = s;
   rtmidi_in_get_message (backend->inputp, buffer, &size);
   if (!backend->inputp->ok)
     {
