@@ -47,15 +47,15 @@ item_iterator_free (struct item_iterator *iter)
 }
 
 gboolean
-item_iterator_is_dir_or_matches_extensions (struct item_iterator *iter,
-					    const gchar **extensions)
+item_iterator_is_dir_or_matches_exts (struct item_iterator *iter,
+				      const gchar **extensions)
 {
   if (iter->item.type == ITEM_TYPE_DIR)
     {
       return TRUE;
     }
 
-  return file_matches_extensions (iter->item.name, extensions);
+  return filename_matches_exts (iter->item.name, extensions);
 }
 
 gchar *
@@ -70,3 +70,21 @@ item_get_filename (struct item *item, guint32 fs_options)
 
   return strdup (item->name);
 }
+
+void
+item_set_name (struct item *item, const gchar *format, ...)
+{
+  va_list args;
+  va_start (args, format);
+  vsnprintf (item->name, ITEM_NAME_MAX, format, args);
+  va_end (args);
+}
+
+void
+item_set_object_info (struct item *item, const gchar *format, ...)
+{
+  va_list args;
+  va_start (args, format);
+  vsnprintf (item->object_info, ITEM_OBJECT_INFO_MAX, format, args);
+  va_end (args);
+}
