@@ -45,7 +45,9 @@
 #define EFACTOR_SINGLE_PRESET_MAX_LEN 256	//This is an empirical value. The maximum found value is 233 but we just add a few bytes just in case.
 #define EFACTOR_MAX_ID_TAG_LEN 16	//The longest value is "[100]" plus the \0.
 
-#define EFACTOR_TIMEOUT_TOTAL_PRESETS 30000	//20 s is not enough with RtMidi.
+#define EFACTOR_READ_DIR_TIMEOUT_MS 30000	//20 s is not enough with RtMidi.
+
+#define EFACTOR_WRITE_SLEEP_TIME_S 3
 
 #define EFACTOR_PEDAL_NAME(data) (data->type == EFACTOR_FACTOR ? EFACTOR_FACTOR_NAME_PREFIX : EFACTOR_H9_NAME_PREFIX)
 
@@ -53,7 +55,6 @@
 #define EFACTOR_ALPHABET " 0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz*+-_|}"
 #define EFACTOR_DEFAULT_CHAR '_'
 
-static const guint8 EVENTIDE_ID[] = { 0x1c };
 static const guint8 FAMILY_ID[] = { 0, 6 };	//This might not be the same value for all the Factor and H9 pedals.
 static const guint8 MODEL_ID[] = { 0x11, 0 };	//This might not be the same value for all the Factor and H9 pedals.
 
@@ -134,7 +135,7 @@ efactor_next_dentry (struct item_iterator *iter)
 
   iter->item.id = data->next + backend_data->min;
   preset_name = data->backend_data->lines[data->next * 7 + 6];
-  snprintf (iter->item.name, LABEL_MAX, "%s", preset_name);
+  item_set_name (&iter->item, "%s", preset_name);
   iter->item.type = ITEM_TYPE_FILE;
   iter->item.size = -1;
   data->next++;
@@ -165,7 +166,7 @@ efactor_read_dir (struct backend *backend, struct item_iterator *iter,
 
   tx_msg = efactor_new_op_msg (EFACTOR_OP_PRESETS_WANT);
   rx_msg = backend_tx_and_rx_sysex (backend, tx_msg,
-				    EFACTOR_TIMEOUT_TOTAL_PRESETS);
+				    EFACTOR_READ_DIR_TIMEOUT_MS);
   if (!rx_msg)
     {
       return -ETIMEDOUT;
@@ -191,7 +192,7 @@ efactor_read_dir (struct backend *backend, struct item_iterator *iter,
 
 static gint
 efactor_download (struct backend *backend, const gchar *path,
-		  struct idata *preset, struct job_control *control)
+		  struct idata *preset, struct task_control *control)
 {
   gint err = 0;
   guint id;
@@ -200,7 +201,7 @@ efactor_download (struct backend *backend, const gchar *path,
   struct efactor_data *data = backend->data;
   GByteArray *output;
 
-  job_control_reset (control, 1);
+  task_control_reset (control, 1);
 
   if (!data->lines)
     {
@@ -239,9 +240,9 @@ efactor_download (struct backend *backend, const gchar *path,
     }
   g_byte_array_append (output, (guint8 *) "\0\xf7", 2);
 
-  job_control_set_progress (control, 1.0);
+  task_control_set_progress (control, 1.0);
 
-  idata_init (preset, output, strdup (name), NULL);
+  idata_init (preset, output, strdup (name), NULL, NULL);
 
   sleep (1);
 
@@ -250,7 +251,7 @@ efactor_download (struct backend *backend, const gchar *path,
 
 static gint
 efactor_upload (struct backend *backend, const gchar *path,
-		struct idata *preset, struct job_control *control)
+		struct idata *preset, struct task_control *control)
 {
   gint err = 0, i, num;
   guint id;
@@ -313,7 +314,7 @@ efactor_upload (struct backend *backend, const gchar *path,
   err = common_data_tx (backend, tx_msg, control);
   free_msg (tx_msg);
 end:
-  sleep (1);
+  sleep (EFACTOR_WRITE_SLEEP_TIME_S);
   return err;
 }
 
@@ -322,20 +323,19 @@ efactor_rename (struct backend *backend, const gchar *src, const gchar *dst)
 {
   GByteArray *preset, *rx_msg;
   gint err, len;
-  struct job_control control;
+  struct task_control control;
   gchar **lines, **line, *sanitized;
   struct idata idata;
 
   debug_print (1, "Renaming from %s to %s...", src, dst);
 
-  //The control initialization is needed.
-  control.active = TRUE;
+  controllable_init (&control.controllable);
   control.callback = NULL;
-  g_mutex_init (&control.mutex);
+
   err = efactor_download (backend, src, &idata, &control);
   if (err)
     {
-      return err;
+      goto end;
     }
 
   preset = idata.content;
@@ -371,8 +371,10 @@ efactor_rename (struct backend *backend, const gchar *src, const gchar *dst)
       free_msg (rx_msg);
     }
 
-  sleep (1);
+  sleep (EFACTOR_WRITE_SLEEP_TIME_S);
 
+end:
+  controllable_clear (&control.controllable);
   return err;
 }
 
@@ -400,7 +402,8 @@ static const struct fs_operations FS_EFACTOR_OPERATIONS = {
     FS_OPTION_SHOW_SLOT_COLUMN,
   .name = "preset",
   .gui_name = "Presets",
-  .gui_icon = FS_ICON_SND,
+  .gui_icon = FS_ICON_PRESET,
+  .file_icon = FS_ICON_PRESET,
   .max_name_len = EFACTOR_MAX_NAME_LEN,
   .readdir = efactor_read_dir,
   .print_item = common_print_item,
@@ -408,7 +411,7 @@ static const struct fs_operations FS_EFACTOR_OPERATIONS = {
   .download = efactor_download,
   .upload = efactor_upload,
   .get_slot = efactor_get_slot,
-  .load = file_load,
+  .load = common_file_load,
   .save = file_save,
   .get_exts = common_sysex_get_extensions,
   .get_upload_path = common_slot_get_upload_path,
@@ -555,6 +558,6 @@ efactor_handshake (struct backend *backend)
 const struct connector CONNECTOR_EFACTOR = {
   .handshake = efactor_handshake,
   .name = "efactor",
-  .standard = FALSE,
+  .options = CONNECTOR_OPTION_CUSTOM_HANDSHAKE,
   .regex = ".*Factor Pedal.*"
 };
