@@ -71,7 +71,6 @@
 
 static const gchar *MICROBRUTE_EXTS[] = { "mbseq", NULL };
 
-static const guint8 ARTURIA_ID[] = { 0x0, 0x20, 0x6b };
 static const guint8 FAMILY_ID[] = { 0x4, 0x0 };
 static const guint8 MODEL_ID[] = { 0x2, 0x1 };
 
@@ -278,7 +277,7 @@ microbrute_download_seq_data (struct backend *backend, guint seqnum,
 
 static gint
 microbrute_download (struct backend *backend, const gchar *src_path,
-		     struct idata *sequence, struct job_control *control)
+		     struct idata *sequence, struct task_control *control)
 {
   gint err;
   guint seqnum;
@@ -296,7 +295,7 @@ microbrute_download (struct backend *backend, const gchar *src_path,
       return -EINVAL;
     }
 
-  job_control_reset (control, 1);
+  task_control_reset (control, 1);
 
   data = g_byte_array_new ();
   err = microbrute_download_seq_data (backend, seqnum, 0, data);
@@ -305,7 +304,7 @@ microbrute_download (struct backend *backend, const gchar *src_path,
       goto err;
     }
 
-  job_control_set_progress (control, 0.5);
+  task_control_set_progress (control, 0.5);
 
   err = microbrute_download_seq_data (backend, seqnum, 0x20, data);
   if (err)
@@ -313,8 +312,8 @@ microbrute_download (struct backend *backend, const gchar *src_path,
       goto err;
     }
 
-  job_control_set_progress (control, 1.0);
-  idata_init (sequence, data, NULL, NULL);
+  task_control_set_progress (control, 1.0);
+  idata_init (sequence, data, NULL, NULL, NULL);
   return 0;
 
 err:
@@ -338,18 +337,20 @@ microbrute_set_sequence_request_msg (struct backend *backend, guint8 id,
 
 static gint
 microbrute_send_seq_msg (struct backend *backend, guint8 seqnum,
-			 guint8 offset, gchar **tokens, gint *pos, gint total)
+			 guint8 offset, gchar **tokens, gint *pos, gint total,
+			 struct controllable *controllable)
 {
   struct sysex_transfer transfer;
+  GByteArray *msg;
   guint8 steps = 0;
   gchar *token = *tokens;
   gint err;
   guint8 *step;
 
-  transfer.raw = microbrute_set_sequence_request_msg (backend, seqnum,
-						      offset);
+  msg = microbrute_set_sequence_request_msg (backend, seqnum, offset);
 
-  step = &transfer.raw->data[MICROBRUTE_SEQ_RPLY_DATA_POS];
+
+  step = &msg->data[MICROBRUTE_SEQ_RPLY_DATA_POS];
   while (steps < 32 && *pos < total)
     {
       if (*token < 0x20)
@@ -398,11 +399,13 @@ microbrute_send_seq_msg (struct backend *backend, guint8 seqnum,
       steps++;
       step++;
     }
-  transfer.raw->data[MICROBRUTE_SEQ_RPLY_LEN_POS] = steps;
+  msg->data[MICROBRUTE_SEQ_RPLY_LEN_POS] = steps;
+
+  sysex_transfer_init_tx (&transfer, msg);
 
   //This doesn't need synchronized access as the caller provices this already.
-  err = backend_tx_sysex (backend, &transfer);
-  free_msg (transfer.raw);
+  err = backend_tx_sysex (backend, &transfer, controllable);
+  sysex_transfer_clear (&transfer);
 
   *tokens = token;
 
@@ -411,7 +414,7 @@ microbrute_send_seq_msg (struct backend *backend, guint8 seqnum,
 
 static gint
 microbrute_upload (struct backend *backend, const gchar *path,
-		   struct idata *sequence, struct job_control *control)
+		   struct idata *sequence, struct task_control *control)
 {
   GByteArray *input = sequence->content;
   gchar *token = (gchar *) & input->data[MICROBRUTE_SEQ_TXT_POS];
@@ -433,26 +436,26 @@ microbrute_upload (struct backend *backend, const gchar *path,
 
   g_mutex_lock (&backend->mutex);
 
-  job_control_reset (control, 1);
+  task_control_reset (control, 1);
 
   steps = microbrute_send_seq_msg (backend, seqnum, 0, &token, &pos,
-				   input->len);
+				   input->len, &control->controllable);
   if (steps < 0)
     {
       goto end;
     }
   else if (pos < input->len)
     {
-      job_control_set_progress (control, 0.5);
+      task_control_set_progress (control, 0.5);
       steps = microbrute_send_seq_msg (backend, seqnum, 0x20, &token, &pos,
-				       input->len);
+				       input->len, &control->controllable);
       if (steps < 0)
 	{
 	  goto end;
 	}
     }
 
-  job_control_set_progress (control, 1.0);
+  task_control_set_progress (control, 1.0);
 
 end:
   g_mutex_unlock (&backend->mutex);
@@ -460,7 +463,8 @@ end:
 }
 
 static const gchar **
-microbrute_get_extensions ()
+microbrute_get_extensions (struct backend *backend,
+			   const struct fs_operations *ops)
 {
   return MICROBRUTE_EXTS;
 }
@@ -470,12 +474,13 @@ static const struct fs_operations FS_MICROBRUTE_OPERATIONS = {
   .options = FS_OPTION_SINGLE_OP | FS_OPTION_SLOT_STORAGE,
   .name = "sequence",
   .gui_name = "Sequences",
-  .gui_icon = FS_ICON_SEQ,
+  .gui_icon = FS_ICON_SEQUENCE,
+  .file_icon = FS_ICON_SEQUENCE,
   .readdir = microbrute_read_dir,
   .print_item = common_print_item,
   .download = microbrute_download,
   .upload = microbrute_upload,
-  .load = file_load,
+  .load = common_file_load,
   .save = file_save,
   .get_exts = microbrute_get_extensions,
   .get_upload_path = common_slot_get_upload_path,
@@ -579,12 +584,14 @@ microbrute_set_parameter (struct backend *backend,
 			  guint8 channel, gboolean sysex)
 {
   gint err;
+
   if (sysex)
     {
       struct sysex_transfer transfer;
-      transfer.raw = microbrute_set_parameter_msg (backend, param, value);
-      err = backend_tx_sysex (backend, &transfer);
-      free_msg (transfer.raw);
+      GByteArray *msg = microbrute_set_parameter_msg (backend, param, value);
+      sysex_transfer_init_tx (&transfer, msg);
+      err = backend_tx_sysex (backend, &transfer, NULL);
+      sysex_transfer_clear (&transfer);
     }
   else
     {
@@ -612,6 +619,6 @@ microbrute_set_parameter (struct backend *backend,
 const struct connector CONNECTOR_MICROBRUTE = {
   .name = MICROBRUTE_NAME,
   .handshake = microbrute_handshake,
-  .standard = TRUE,
+  .options = 0,
   .regex = ".*MicroBrute.*"
 };
