@@ -79,7 +79,6 @@ static const gchar *MICROFREAK_ZWAVETABLE_EXTS[] =
 static const guint8 MICROFREAK_REQUEST_HEADER[] =
   { 0xf0, 0, 0x20, 0x6b, 7, 1 };
 
-static const guint8 ARTURIA_ID[] = { 0x0, 0x20, 0x6b };
 static const guint8 FAMILY_ID[] = { 0x6, 0x0 };
 static const guint8 MODEL_ID[] = { 0x6, 0x1 };
 
@@ -103,37 +102,13 @@ struct microfreak_iter_data
 void
 microfreak_midi_msg_to_8bit_msg (guint8 *msg_midi, guint8 *msg_8bit)
 {
-  guint8 *dst = msg_8bit;
-  guint8 *src = msg_midi;
-  for (gint i = 0; i < 4; i++)
-    {
-      guint8 bits = *src;
-      src++;
-      for (gint j = 0; j < 7; j++, src++, dst++)
-	{
-	  *dst = *src | (bits & 0x1 ? 0x80 : 0);
-	  bits >>= 1;
-	}
-    }
+  common_midi_msg_to_8bit_msg (msg_midi, msg_8bit, 32);
 }
 
 void
 microfreak_8bit_msg_to_midi_msg (guint8 *msg_8bit, guint8 *msg_midi)
 {
-  guint8 *dst = msg_midi;
-  guint8 *src = msg_8bit;
-  for (gint i = 0; i < 4; i++)
-    {
-      guint8 *bits = dst;
-      *bits = 0;
-      dst++;
-      for (gint j = 0; j < 7; j++, src++, dst++)
-	{
-	  *dst = *src & 0x7f;
-	  *bits |= *src & 0x80;
-	  *bits >>= 1;
-	}
-    }
+  common_8bit_msg_to_midi_msg (msg_8bit, msg_midi, 28);
 }
 
 static void
@@ -282,13 +257,13 @@ microfreak_next_preset_dentry (struct item_iterator *iter)
     }
 
   microfreak_preset_get_name (preset_name, rx_msg);
-  snprintf (iter->item.name, LABEL_MAX, "%s", preset_name);
+  item_set_name (&iter->item, "%s", preset_name);
 
   iter->item.id = data->next;
   iter->item.type = ITEM_TYPE_FILE;
   iter->item.size = -1;
   category = microfreak_get_category_name (rx_msg);
-  snprintf (iter->item.object_info, LABEL_MAX, "%s", category);
+  item_set_object_info (&iter->item, "category=%s", category);
   (data->next)++;
 
 end:
@@ -344,7 +319,8 @@ microfreak_serialize_preset (GByteArray *output,
 
 static gint
 microfreak_preset_download (struct backend *backend, const gchar *path,
-			    struct idata *preset, struct job_control *control)
+			    struct idata *preset,
+			    struct task_control *control)
 {
   guint id;
   gint err;
@@ -366,7 +342,7 @@ microfreak_preset_download (struct backend *backend, const gchar *path,
 
   output = g_byte_array_new ();
 
-  job_control_reset (control, 2 + MICROFREAK_PRESET_PARTS);	//Worst case
+  task_control_reset (control, 2 + MICROFREAK_PRESET_PARTS);	//Worst case
 
   tx_msg = microfreak_get_preset_op_msg (backend, 0x19, id, 0);
   err = common_data_tx_and_rx_part (backend, tx_msg, &rx_msg, control);
@@ -394,7 +370,7 @@ microfreak_preset_download (struct backend *backend, const gchar *path,
     {
       control->parts = 1;
       control->part = 0;
-      job_control_set_progress (control, 1.0);
+      task_control_set_progress (control, 1.0);
       goto end;
     }
 
@@ -418,7 +394,7 @@ microfreak_preset_download (struct backend *backend, const gchar *path,
     {
       guint8 op = i == mfp.parts - 1 ? 0x17 : 0x16;
 
-      if (!job_control_get_active_lock (control))
+      if (!controllable_is_active (&control->controllable))
 	{
 	  err = -ECANCELED;
 	  goto end;
@@ -463,7 +439,7 @@ end:
       memcpy (name, next, len);
       name[len] = 0;
 
-      idata_init (preset, output, strdup (name), NULL);
+      idata_init (preset, output, strdup (name), NULL, NULL);
     }
   usleep (MICROFREAK_REST_TIME_LONG_US);	//Additional rest
   return err;
@@ -471,7 +447,7 @@ end:
 
 static gint
 microfreak_preset_upload (struct backend *backend, const gchar *path,
-			  struct idata *preset, struct job_control *control)
+			  struct idata *preset, struct task_control *control)
 {
   struct microfreak_preset mfp;
   GByteArray *tx_msg, *rx_msg;
@@ -497,7 +473,7 @@ microfreak_preset_upload (struct backend *backend, const gchar *path,
       return err;
     }
 
-  job_control_reset (control, 3 + mfp.parts);
+  task_control_reset (control, 3 + mfp.parts);
 
   mfp.header[0] = COMMON_GET_MIDI_BANK (id);
   mfp.header[1] = COMMON_GET_MIDI_PRESET (id);
@@ -550,7 +526,7 @@ microfreak_preset_upload (struct backend *backend, const gchar *path,
 
   for (gint i = 0; i < mfp.parts; i++)
     {
-      if (!job_control_get_active_lock (control))
+      if (!controllable_is_active (&control->controllable))
 	{
 	  err = -ECANCELED;
 	  break;
@@ -664,7 +640,8 @@ microfreak_preset_rename (struct backend *backend, const gchar *src,
 }
 
 static const gchar **
-microfreak_ppreset_get_extensions ()
+microfreak_ppreset_get_extensions (struct backend *backend,
+				   const struct fs_operations *ops)
 {
   return MICROFREAK_PPRESET_EXTS;
 }
@@ -680,7 +657,7 @@ static const struct fs_operations FS_MICROFREAK_PPRESET_OPERATIONS = {
   .rename = microfreak_preset_rename,
   .download = microfreak_preset_download,
   .upload = microfreak_preset_upload,
-  .load = file_load,
+  .load = common_file_load,
   .save = file_save,
   .get_exts = microfreak_ppreset_get_extensions,
   .get_upload_path = common_slot_get_upload_path,
@@ -689,13 +666,14 @@ static const struct fs_operations FS_MICROFREAK_PPRESET_OPERATIONS = {
 
 static gint
 microfreak_zpreset_save (const gchar *path, struct idata *zpreset,
-			 struct job_control *control)
+			 struct task_control *control)
 {
   return microfreak_zobject_save (path, zpreset, control, "0_preset");
 }
 
 static const gchar **
-microfreak_zpreset_get_extensions ()
+microfreak_zpreset_get_extensions (struct backend *backend,
+				   const struct fs_operations *ops)
 {
   return MICROFREAK_ZPRESET_EXTS;
 }
@@ -719,8 +697,8 @@ static const struct fs_operations FS_MICROFREAK_ZPRESET_OPERATIONS = {
 };
 
 static gint
-microfreak_preset_load (const char *path, struct idata *preset,
-			struct job_control *control)
+microfreak_preset_load (struct backend *backend, const char *path,
+			struct idata *preset, struct task_control *control)
 {
   const gchar *ext = filename_get_ext (path);
   if (strcmp (ext, MICROFREAK_ZPRESET_EXT))
@@ -729,7 +707,7 @@ microfreak_preset_load (const char *path, struct idata *preset,
     }
   else
     {
-      return microfreak_zobject_load (path, preset, control);
+      return microfreak_zobject_load (backend, path, preset, control);
     }
 }
 
@@ -747,7 +725,8 @@ static const struct fs_operations FS_MICROFREAK_PRESET_OPERATIONS = {
     FS_OPTION_ALLOW_SEARCH,
   .name = "preset",
   .gui_name = "Presets",
-  .gui_icon = FS_ICON_SND,
+  .gui_icon = FS_ICON_PRESET,
+  .file_icon = FS_ICON_PRESET,
   .max_name_len = MICROFREAK_PRESET_NAME_LEN,
   .readdir = microfreak_preset_read_dir,
   .print_item = common_print_item,
@@ -818,10 +797,12 @@ microfreak_next_sample_dentry (struct item_iterator *iter)
 
   microfreak_midi_msg_to_8bit_msg (MICROFREAK_GET_MSG_PAYLOAD (rx_msg),
 				   (guint8 *) & header);
-  snprintf (iter->item.name, LABEL_MAX, "%s", header.name);
+  item_set_name (&iter->item, "%s", header.name);
   iter->item.id = data->next;
   iter->item.type = ITEM_TYPE_FILE;
   iter->item.size = GINT32_FROM_LE (header.size);
+  sample_info_init (&iter->item.sample_info);
+
   (data->next)++;
   free_msg (rx_msg);
 
@@ -985,7 +966,7 @@ static gint
 microfreak_sample_upload_tx_and_rx (struct backend *backend,
 				    GByteArray *tx_msg,
 				    GByteArray **rx_msg,
-				    struct job_control *control)
+				    struct task_control *control)
 {
   guint8 seq = tx_msg->data[sizeof (MICROFREAK_REQUEST_HEADER)];
   gint err = common_data_tx_and_rx_part (backend, tx_msg, rx_msg, control);
@@ -1016,7 +997,7 @@ microfreak_sample_upload_tx_and_rx (struct backend *backend,
 
 static gint
 microfreak_sample_upload (struct backend *backend, const gchar *path,
-			  struct idata *sample, struct job_control *control)
+			  struct idata *sample, struct task_control *control)
 {
   gint err;
   guint id, batches;
@@ -1050,15 +1031,15 @@ microfreak_sample_upload (struct backend *backend, const gchar *path,
       batches = MICROFREAK_SAMPLE_MAX_BATCHES;
     }
 
-  job_control_reset (control,
-		     6 + batches * (2 + MICROFREAK_SAMPLE_BATCH_PACKETS) + 1 +
-		     MICROFREAK_SAMPLE_BATCH_PACKETS);
+  task_control_reset (control,
+		      6 + batches * (2 + MICROFREAK_SAMPLE_BATCH_PACKETS) +
+		      1 + MICROFREAK_SAMPLE_BATCH_PACKETS);
 
   //This is called by Arturia MIDI Control Center before uploading a sample.
   //Perhaps this does more than just retrieving the statistics.
   microfreak_get_storage_stats (backend, 0, &statfs, NULL);
 
-  job_control_set_progress (control, 1.0);
+  task_control_set_progress (control, 1.0);
   control->part++;
 
   tx_msg = microfreak_get_wave_op_msg (backend, 0x5d, id, 0, 0);
@@ -1139,7 +1120,7 @@ microfreak_sample_upload (struct backend *backend, const gchar *path,
       goto end;
     }
 
-  job_control_set_progress (control, 1.0);
+  task_control_set_progress (control, 1.0);
   control->part++;
 
   usleep (MICROFREAK_REST_TIME_US);
@@ -1195,7 +1176,7 @@ microfreak_sample_upload (struct backend *backend, const gchar *path,
 	  gint16 blk[MICROFREAK_WAVE_BLK_SHRT];
 	  gint16 *dst = blk;
 
-	  if (!job_control_get_active_lock (control))
+	  if (!controllable_is_active (&control->controllable))
 	    {
 	      err = -ECANCELED;
 	      goto end;
@@ -1341,11 +1322,13 @@ microfreak_next_wavetable_dentry (struct item_iterator *iter)
 
   microfreak_midi_msg_to_8bit_msg (MICROFREAK_GET_MSG_PAYLOAD (rx_msg),
 				   (guint8 *) & header);
-  snprintf (iter->item.name, LABEL_MAX, "%s", header.name);
+  item_set_name (&iter->item, "%s", header.name);
   iter->item.id = data->next;
   iter->item.type = ITEM_TYPE_FILE;
   iter->item.size = header.status0 == MICROFREAK_WAVETABLE_EMPTY ?
     0 : MICROFREAK_WAVETABLE_SIZE;
+  sample_info_init (&iter->item.sample_info);
+
   (data->next)++;
 
 end:
@@ -1364,12 +1347,13 @@ microfreak_wavetable_read_dir (struct backend *backend,
 }
 
 static gint
-microfreak_wavetable_load (const gchar *path, struct idata *wavetable,
-			   struct job_control *control)
+microfreak_wavetable_load (struct backend *backend, const gchar *path,
+			   struct idata *wavetable,
+			   struct task_control *control)
 {
   struct idata aux;
-  gint err = common_sample_load (path, &aux, control, 0, 1,
-				 SF_FORMAT_PCM_16);
+  gint err = common_sample_load (path, &aux, control, 1, 0,
+				 SF_FORMAT_PCM_16, FALSE);
   if (err)
     {
       return -EINVAL;
@@ -1380,20 +1364,22 @@ microfreak_wavetable_load (const gchar *path, struct idata *wavetable,
       struct sample_info *sample_info =
 	microfreak_new_sample_info (MICROFREAK_WAVETABLE_LEN);
       gchar *name = strdup (aux.name);
-      idata_init (wavetable, idata_steal (&aux), name, sample_info);
+      idata_init (wavetable, idata_steal (&aux), name, sample_info,
+		  sample_info_free);
       return 0;
     }
   else
     {
       debug_print (1, "Resampling to get a valid wavetable...");
       struct sample_info *si = aux.info;
-      struct sample_info si_req;
+      struct sample_load_opts opts;
       GByteArray *a;
-      microfreak_init_sample_info (&si_req, MICROFREAK_WAVETABLE_LEN);
-      si_req.rate = si->rate * MICROFREAK_WAVETABLE_LEN / si->frames;
-      err = sample_reload (&aux, wavetable, NULL, &si_req,
-			   job_control_set_sample_progress_no_sync, NULL);
-      idata_free (&aux);
+      sample_load_opts_init (&opts, 1, MICROFREAK_SAMPLERATE,
+			     SF_FORMAT_PCM_16, FALSE);
+      opts.rate = si->rate * MICROFREAK_WAVETABLE_LEN / si->frames;
+      err = sample_reload (&aux, wavetable, NULL, &opts,
+			   task_control_set_sample_progress);
+      idata_clear (&aux);
       a = wavetable->content;
       debug_print (2, "Resulting size: %d", a->len);
       if (a->len < MICROFREAK_WAVETABLE_SIZE)
@@ -1411,7 +1397,7 @@ microfreak_wavetable_load (const gchar *path, struct idata *wavetable,
 static gint
 microfreak_wavetable_download_part (struct backend *backend,
 				    GByteArray *output,
-				    struct job_control *control, guint8 id,
+				    struct task_control *control, guint8 id,
 				    guint8 part)
 {
   gint err;
@@ -1440,7 +1426,7 @@ microfreak_wavetable_download_part (struct backend *backend,
       guint8 op;
       guint len;
 
-      if (!job_control_get_active_lock (control))
+      if (!controllable_is_active (&control->controllable))
 	{
 	  return -ECANCELED;
 	}
@@ -1492,7 +1478,7 @@ end:
 static gint
 microfreak_wavetable_download (struct backend *backend, const gchar *path,
 			       struct idata *wavetable,
-			       struct job_control *control)
+			       struct task_control *control)
 {
   guint id;
   guint err = 0;
@@ -1547,8 +1533,8 @@ microfreak_wavetable_download (struct backend *backend, const gchar *path,
   content = g_byte_array_sized_new (MICROFREAK_WAVETABLE_SIZE);
   content->len = MICROFREAK_WAVETABLE_SIZE;
 
-  job_control_reset (control, (MICROFREAK_SAMPLE_BATCH_PACKETS + 1) *
-		     MICROFREAK_WAVETABLE_PARTS);
+  task_control_reset (control, (MICROFREAK_SAMPLE_BATCH_PACKETS + 1) *
+		      MICROFREAK_WAVETABLE_PARTS);
 
   err = 0;
   for (guint8 part = 0; part < MICROFREAK_WAVETABLE_PARTS && !err; part++)
@@ -1564,7 +1550,8 @@ microfreak_wavetable_download (struct backend *backend, const gchar *path,
     }
   else
     {
-      idata_init (wavetable, content, strdup (name), sample_info);
+      idata_init (wavetable, content, strdup (name), sample_info,
+		  sample_info_free);
     }
 
   return err;
@@ -1773,7 +1760,7 @@ microfreak_wavetable_rename (struct backend *backend, const gchar *src,
 static gint
 microfreak_wavetable_upload_id_name (struct backend *backend,
 				     const gchar *path, GByteArray *wavetable,
-				     struct job_control *control, guint id,
+				     struct task_control *control, guint id,
 				     const gchar *name)
 {
   gint err;
@@ -1784,7 +1771,7 @@ microfreak_wavetable_upload_id_name (struct backend *backend,
       return -EINVAL;
     }
 
-  job_control_reset (control, 1 + MICROFREAK_WAVETABLE_PARTS);
+  task_control_reset (control, 1 + MICROFREAK_WAVETABLE_PARTS);
 
   err = microfreak_wavetable_set_entry (backend, id, name, 0);
   if (err)
@@ -1792,18 +1779,18 @@ microfreak_wavetable_upload_id_name (struct backend *backend,
       return err;
     }
 
-  job_control_set_progress (control, 1.0);
+  task_control_set_progress (control, 1.0);
   control->part++;
 
   for (guint8 part = 0; part < MICROFREAK_WAVETABLE_PARTS && !err; part++)
     {
-      if (!job_control_get_active_lock (control))
+      if (!controllable_is_active (&control->controllable))
 	{
 	  return -ECANCELED;
 	}
 
       err = microfreak_wavetable_upload_part (backend, wavetable, id, part);
-      job_control_set_progress (control, 1.0);
+      task_control_set_progress (control, 1.0);
       control->part++;
     }
 
@@ -1813,7 +1800,7 @@ microfreak_wavetable_upload_id_name (struct backend *backend,
 static gint
 microfreak_xwavetable_upload (struct backend *backend, const gchar *path,
 			      struct idata *idata,
-			      struct job_control *control)
+			      struct task_control *control)
 {
   gint err;
   guint id;
@@ -1831,7 +1818,8 @@ microfreak_xwavetable_upload (struct backend *backend, const gchar *path,
 
 static gint
 microfreak_wavetable_upload (struct backend *backend, const gchar *path,
-			     struct idata *idata, struct job_control *control)
+			     struct idata *idata,
+			     struct task_control *control)
 {
   gint err;
   guint id;
@@ -1888,11 +1876,11 @@ microfreak_wavetable_clear (struct backend *backend, const gchar *path)
 }
 
 static gint
-microfreak_sample_load (const gchar *path, struct idata *sample,
-			struct job_control *control)
+microfreak_sample_load (struct backend *backend, const gchar *path,
+			struct idata *sample, struct task_control *control)
 {
-  return common_sample_load (path, sample, control, MICROFREAK_SAMPLERATE,
-			     1, SF_FORMAT_PCM_16);
+  return common_sample_load (path, sample, control, 1, MICROFREAK_SAMPLERATE,
+			     SF_FORMAT_PCM_16, FALSE);
 }
 
 static const struct fs_operations FS_MICROFREAK_SAMPLE_OPERATIONS = {
@@ -1903,12 +1891,12 @@ static const struct fs_operations FS_MICROFREAK_SAMPLE_OPERATIONS = {
   .name = "sample",
   .gui_name = "Samples",
   .gui_icon = FS_ICON_WAVE,
+  .file_icon = FS_ICON_WAVE,
   .max_name_len = MICROFREAK_SAMPLE_NAME_LEN - 1,
   .readdir = microfreak_sample_read_dir,
   .print_item = common_print_item,
   .get_slot = microfreak_get_object_id_as_slot,
   .delete = microfreak_sample_clear,
-  .clear = microfreak_sample_clear,
   .upload = microfreak_sample_upload,
   .load = microfreak_sample_load,
   .get_exts = sample_get_sample_extensions,
@@ -1923,7 +1911,8 @@ microfreak_get_wavetable_id_as_slot (struct item *item,
 }
 
 static const gchar **
-microfreak_pwavetable_get_extensions ()
+microfreak_pwavetable_get_extensions (struct backend *backend,
+				      const struct fs_operations *ops)
 {
   return MICROFREAK_PWAVETABLE_EXTS;
 }
@@ -1936,7 +1925,7 @@ static const struct fs_operations FS_MICROFREAK_PWAVETABLE_OPERATIONS = {
   .readdir = microfreak_wavetable_read_dir,
   .print_item = common_print_item,
   .get_slot = microfreak_get_wavetable_id_as_slot,
-  .clear = microfreak_wavetable_clear,
+  .delete = microfreak_wavetable_clear,
   .rename = microfreak_wavetable_rename,
   .download = microfreak_wavetable_download,
   .upload = microfreak_xwavetable_upload,
@@ -1948,7 +1937,8 @@ static const struct fs_operations FS_MICROFREAK_PWAVETABLE_OPERATIONS = {
 };
 
 static const gchar **
-microfreak_zwavetable_get_extensions ()
+microfreak_zwavetable_get_extensions (struct backend *backend,
+				      const struct fs_operations *ops)
 {
   return MICROFREAK_ZWAVETABLE_EXTS;
 }
@@ -1961,7 +1951,7 @@ static const struct fs_operations FS_MICROFREAK_ZWAVETABLE_OPERATIONS = {
   .readdir = microfreak_wavetable_read_dir,
   .print_item = common_print_item,
   .get_slot = microfreak_get_wavetable_id_as_slot,
-  .clear = microfreak_wavetable_clear,
+  .delete = microfreak_wavetable_clear,
   .rename = microfreak_wavetable_rename,
   .download = microfreak_wavetable_download,
   .upload = microfreak_xwavetable_upload,
@@ -1974,7 +1964,7 @@ static const struct fs_operations FS_MICROFREAK_ZWAVETABLE_OPERATIONS = {
 
 static gint
 microfreak_wavetable_save (const gchar *path, struct idata *wavetable,
-			   struct job_control *control)
+			   struct task_control *control)
 {
   return sample_save_to_file (path, wavetable, control, SF_FORMAT_WAV |
 			      SF_FORMAT_PCM_16);
@@ -1988,12 +1978,12 @@ static const struct fs_operations FS_MICROFREAK_WAVETABLE_OPERATIONS = {
   .name = "wavetable",
   .gui_name = "Wavetables",
   .gui_icon = FS_ICON_WAVETABLE,
+  .file_icon = FS_ICON_WAVE,
   .max_name_len = MICROFREAK_WAVETABLE_NAME_LEN - 1,
   .readdir = microfreak_wavetable_read_dir,
   .print_item = common_print_item,
   .get_slot = microfreak_get_wavetable_id_as_slot,
   .delete = microfreak_wavetable_clear,
-  .clear = microfreak_wavetable_clear,
   .rename = microfreak_wavetable_rename,
   .download = microfreak_wavetable_download,
   .upload = microfreak_wavetable_upload,
@@ -2098,6 +2088,6 @@ microfreak_handshake (struct backend *backend)
 const struct connector CONNECTOR_MICROFREAK = {
   .name = MICROFREAK_NAME,
   .handshake = microfreak_handshake,
-  .standard = TRUE,
+  .options = 0,
   .regex = ".*MicroFreak.*"
 };
