@@ -391,7 +391,7 @@ package_end (struct package *pkg, struct idata *out)
   zip_source_read (pkg->zip_source, content->data, zstat.comp_size);
   zip_source_close (pkg->zip_source);
 
-  idata_init (out, content, NULL, NULL);
+  idata_init (out, content, NULL, NULL, NULL);
 
   return 0;
 }
@@ -498,7 +498,7 @@ package_get_max_sample_slots (struct backend *backend)
 gint
 package_receive_pkg_resources (struct package *pkg,
 			       const gchar *payload_path,
-			       struct job_control *control,
+			       struct task_control *control,
 			       struct backend *backend,
 			       fs_remote_file_op download_data,
 			       enum package_type type)
@@ -516,7 +516,7 @@ package_receive_pkg_resources (struct package *pkg,
 
   pkg->manifest->tags = NULL;
 
-  job_control_reset (control, 1);	//payload
+  task_control_reset (control, 1);	//payload
 
   if ((type == PKG_FILE_TYPE_DATA_PROJECT &&
        data->device_desc.id != ELEKTRON_ANALOG_RYTM_ID &&
@@ -598,7 +598,7 @@ package_receive_pkg_resources (struct package *pkg,
     }
 
   control->parts = 2 + elements;
-  job_control_set_progress (control, 0.0);
+  task_control_set_progress (control, 0.0);
   for (i = 0; i < elements; i++, control->part++)
     {
       if (!json_reader_read_element (reader, i))
@@ -672,14 +672,14 @@ package_receive_pkg_resources (struct package *pkg,
 	  continue;
 	}
 
-      idata_free (&sample_file);
+      idata_clear (&sample_file);
     }
 
 cleanup_reader:
   g_object_unref (reader);
 cleanup_parser:
   g_object_unref (parser);
-  idata_free (&metadata_file);
+  idata_clear (&metadata_file);
 get_payload:
   debug_print (1, "Getting payload from %s...", payload_path);
   ret = download_data (backend, payload_path, &payload_file, control);
@@ -705,7 +705,7 @@ get_payload:
 
 gint
 package_send_pkg_resources (struct package *pkg, const gchar *payload_path,
-			    struct job_control *control,
+			    struct task_control *control,
 			    struct backend *backend,
 			    fs_remote_file_op upload_data)
 {
@@ -773,7 +773,7 @@ package_send_pkg_resources (struct package *pkg, const gchar *payload_path,
 
   control->parts = 1 + package_get_max_sample_slots (backend);	// main and sample slots
   control->part = 0;
-  idata_init (&file, pkg_resource->data, NULL, NULL);
+  idata_init (&file, pkg_resource->data, NULL, NULL, NULL);
   ret = upload_data (backend, payload_path, &file, control);
   if (ret)
     {
@@ -854,7 +854,7 @@ package_send_pkg_resources (struct package *pkg, const gchar *payload_path,
     {
       control->parts = 1;	// Only payload and it's done.
       control->part = 0;
-      job_control_set_progress (control, 1.0);
+      task_control_set_progress (control, 1.0);
       goto cleanup_reader;
     }
 
@@ -867,7 +867,8 @@ package_send_pkg_resources (struct package *pkg, const gchar *payload_path,
     }
 
   //We are reusing the same sample_file. Let's be careful.
-  idata_init (&sample_file, g_byte_array_sized_new (zstat.size), NULL, NULL);
+  idata_init (&sample_file, g_byte_array_sized_new (zstat.size), NULL, NULL,
+	      NULL);
 
   elements = json_reader_count_elements (reader);
   control->parts = elements + 1;
@@ -875,10 +876,11 @@ package_send_pkg_resources (struct package *pkg, const gchar *payload_path,
 
   for (i = 0; i < elements; i++, control->part++)
     {
-      struct sample_info sample_info_req, sample_info_src;
-      sample_info_req.rate = ELEKTRON_SAMPLE_RATE;
-      sample_info_req.channels = 0;	//Automatic
-      sample_info_req.format = SF_FORMAT_PCM_16;
+      struct sample_info sample_info_src;
+      struct sample_load_opts sample_load_opts;
+
+      sample_load_opts_init (&sample_load_opts, 0, ELEKTRON_SAMPLE_RATE,
+			     SF_FORMAT_PCM_16, FALSE);
 
       json_reader_read_element (reader, i);
       json_reader_read_member (reader, PKG_TAG_FILE_NAME);
@@ -910,7 +912,7 @@ package_send_pkg_resources (struct package *pkg, const gchar *payload_path,
       zip_fclose (zip_file);
 
       if (sample_load_from_memfile (&sample_file, &sample, control,
-				    &sample_info_req, &sample_info_src))
+				    &sample_load_opts, &sample_info_src))
 	{
 	  error_print ("Error while loading '%s': %s",
 		       sample_path, zip_error_strerror (&zerror));
@@ -939,7 +941,7 @@ package_send_pkg_resources (struct package *pkg, const gchar *payload_path,
       sample_file.name = NULL;
     }
 
-  idata_free (&sample_file);
+  idata_clear (&sample_file);
 
 cleanup_reader:
   g_object_unref (reader);
