@@ -19,6 +19,7 @@
  */
 
 #include <glib.h>
+#include "backend.h"
 #include "utils.h"
 
 #define SCALA_DESC_MAX_LEN 1024
@@ -35,12 +36,9 @@ struct scala
   gdouble pitches[SCALA_NOTES_MAX];
 };
 
-gint scl_load_2_byte_octave_tuning_msg_from_scala_file (const char *path,
-							struct idata *idata,
-							struct job_control
-							*control);
+gint scala_load_2_byte_octave_tuning_msg (const char *path,
+					  struct idata *idata,
+					  struct task_control *control);
 
-gint scl_load_key_based_tuning_msg_from_scala_file (const char *path,
-						    struct idata *idata,
-						    struct job_control
-						    *control);
+gint scala_load_key_based_tuning_msg (const char *path, struct idata *idata,
+				      struct task_control *control);
