@@ -192,6 +192,18 @@ sds_get_download_info (GByteArray *header, struct sample_info *sample_info,
 		       guint *bytes_per_word)
 {
   *bits = header->data[6];
+  if (sds_get_bytes_per_word (*bits, word_size, bytes_per_word))
+    {
+      return -1;
+    }
+
+  sample_info->loop_start =
+    sds_get_bytes_value_right_just (&header->data[13], SDS_BYTES_PER_WORD);
+  sample_info->loop_end =
+    sds_get_bytes_value_right_just (&header->data[16], SDS_BYTES_PER_WORD);
+  sample_info->loop_type = header->data[19];
+  sample_info->rate = 1.0e9 /
+    sds_get_bytes_value_right_just (&header->data[7], SDS_BYTES_PER_WORD);
   if (*bits == 8)
     {
       sample_info->format = SF_FORMAT_WAV | SF_FORMAT_PCM_U8;
@@ -200,21 +212,12 @@ sds_get_download_info (GByteArray *header, struct sample_info *sample_info,
     {
       sample_info->format = SF_FORMAT_WAV | SF_FORMAT_PCM_16;
     }
-  if (sds_get_bytes_per_word (*bits, word_size, bytes_per_word))
-    {
-      return -1;
-    }
-  sample_info->rate = 1.0e9 /
-    sds_get_bytes_value_right_just (&header->data[7], SDS_BYTES_PER_WORD);
-  *words =
-    sds_get_bytes_value_right_just (&header->data[10], SDS_BYTES_PER_WORD);
-  sample_info->loop_start =
-    sds_get_bytes_value_right_just (&header->data[13], SDS_BYTES_PER_WORD);
-  sample_info->loop_end =
-    sds_get_bytes_value_right_just (&header->data[16], SDS_BYTES_PER_WORD);
-  sample_info->loop_type = header->data[19];
-  sample_info->midi_note = 0;
+
   sample_info->channels = 1;
+
+  *words = sds_get_bytes_value_right_just (&header->data[10],
+					   SDS_BYTES_PER_WORD);
+
   return 0;
 }
 
@@ -270,18 +273,6 @@ sds_get_dump_msg (guint id, guint frames, struct sample_info *sample_info,
   return tx_msg;
 }
 
-static GByteArray *
-sds_rx (struct backend *backend, gint timeout)
-{
-  struct sysex_transfer transfer;
-  transfer.timeout = timeout;
-  transfer.batch = FALSE;
-  g_mutex_lock (&backend->mutex);
-  backend_rx_sysex (backend, &transfer);
-  g_mutex_unlock (&backend->mutex);
-  return transfer.raw;
-}
-
 static void
 sds_download_inc_packet (gboolean *first, guint *packet)
 {
@@ -332,7 +323,7 @@ sds_download_get_header (struct backend *backend, guint id)
 
 static gint
 sds_download_try (struct backend *backend, const gchar *path,
-		  struct idata *sample, struct job_control *control)
+		  struct idata *sample, struct task_control *control)
 {
   guint id, words, word_size, read_bytes, bytes_per_word, total_words, err,
     retries, packets, packet, exp_packet, rx_packets, bits;
@@ -357,9 +348,7 @@ sds_download_try (struct backend *backend, const gchar *path,
   retries = 0;
   while (1)
     {
-      g_mutex_lock (&control->mutex);
-      active = control->active;
-      g_mutex_unlock (&control->mutex);
+      active = controllable_is_active (&control->controllable);
 
       if (!active)
 	{
@@ -384,7 +373,7 @@ sds_download_try (struct backend *backend, const gchar *path,
 	}
     }
 
-  sample_info = g_malloc (sizeof (struct sample_info));
+  sample_info = sample_info_new (FALSE);
   if (sds_get_download_info (rx_msg, sample_info, &bits, &words, &word_size,
 			     &bytes_per_word))
     {
@@ -399,15 +388,12 @@ sds_download_try (struct backend *backend, const gchar *path,
   sds_debug_print_sample_data (bits, bytes_per_word, word_size,
 			       sample_info->rate, words, packets);
 
-  g_mutex_lock (&control->mutex);
-  active = control->active;
-  g_mutex_unlock (&control->mutex);
+  active = controllable_is_active (&control->controllable);
 
-  job_control_reset (control, 1);
+  task_control_reset (control, 1);
 
   debug_print (1, "Receiving dump data...");
 
-  tx_msg = g_byte_array_new ();
   total_words = 0;
   retries = 0;
   last_packet_ack = TRUE;
@@ -424,7 +410,7 @@ sds_download_try (struct backend *backend, const gchar *path,
 	  break;
 	}
 
-      g_byte_array_set_size (tx_msg, 0);
+      tx_msg = g_byte_array_new ();
       if (last_packet_ack)
 	{
 	  g_byte_array_append (tx_msg, SDS_ACK, sizeof (SDS_ACK));
@@ -448,9 +434,12 @@ sds_download_try (struct backend *backend, const gchar *path,
 
       g_byte_array_append (tx_msg, SDS_WAIT, sizeof (SDS_WAIT));
       tx_msg->data[10] = (packet) % 0x80;
-      transfer.raw = tx_msg;
-      transfer.timeout = SDS_INCOMPLETE_PACKET_TIMEOUT;	//This is enough to detect incomplete packets.
-      err = backend_tx_and_rx_sysex_transfer (backend, &transfer, FALSE);
+
+      sysex_transfer_init_tx_and_rx (&transfer, SDS_INCOMPLETE_PACKET_TIMEOUT,
+				     tx_msg);
+
+      err = backend_tx_and_rx_sysex_transfer (backend, &transfer,
+					      &control->controllable);
       if (err == -ECANCELED)
 	{
 	  break;
@@ -480,7 +469,7 @@ sds_download_try (struct backend *backend, const gchar *path,
 	}
       else
 	{
-	  rx_msg = transfer.raw;
+	  rx_msg = sysex_transfer_steal (&transfer);
 	}
 
       if (rx_msg->len != SDS_DATA_PACKET_LEN)
@@ -521,11 +510,9 @@ sds_download_try (struct backend *backend, const gchar *path,
 	  total_words++;
 	}
 
-      job_control_set_progress (control, rx_packets / (double) packets);
+      task_control_set_progress (control, rx_packets / (double) packets);
 
-      g_mutex_lock (&control->mutex);
-      active = control->active;
-      g_mutex_unlock (&control->mutex);
+      active = controllable_is_active (&control->controllable);
 
       free_msg (rx_msg);
 
@@ -543,13 +530,11 @@ sds_download_try (struct backend *backend, const gchar *path,
       continue;
     }
 
-  free_msg (tx_msg);
-
 end:
   if (active && !err && rx_packets == packets)
     {
       debug_print (1, "%d frames received", total_words);
-      job_control_set_progress (control, 1.0);
+      task_control_set_progress (control, 1.0);
 
       if (sds_data->name_extension)
 	{
@@ -560,7 +545,7 @@ end:
 	  name = g_malloc (LABEL_MAX);
 	  snprintf (name, LABEL_MAX, "%03d", id);
 	}
-      idata_init (sample, output, name, sample_info);
+      idata_init (sample, output, name, sample_info, sample_info_free);
     }
   else
     {
@@ -576,7 +561,7 @@ end:
 
 static gint
 sds_download (struct backend *backend, const gchar *path,
-	      struct idata *sample, struct job_control *control)
+	      struct idata *sample, struct task_control *control)
 {
   gint err;
 
@@ -599,7 +584,8 @@ sds_download (struct backend *backend, const gchar *path,
 
 static gint
 sds_tx_and_wait_ack (struct backend *backend, GByteArray *tx_msg,
-		     guint packet, gint timeout, gint timeout2)
+		     guint packet, gint timeout, gint timeout2,
+		     struct controllable *controllable)
 {
   gint err;
   gint t;
@@ -651,7 +637,7 @@ sds_tx_and_wait_ack (struct backend *backend, GByteArray *tx_msg,
 	}
 
       free_msg (rx_msg);
-      rx_msg = sds_rx (backend, t);
+      rx_msg = backend_rx (backend, t, controllable);
       if (!rx_msg)
 	{
 	  return -ENOMSG;
@@ -745,7 +731,7 @@ sds_rename (struct backend *backend, const gchar *src, const gchar *dst)
 
 static gint
 sds_upload (struct backend *backend, const gchar *path, struct idata *sample,
-	    struct job_control *control, guint bits)
+	    struct task_control *control, guint bits)
 {
   GByteArray *tx_msg;
   gint16 *frame, *f;
@@ -757,7 +743,7 @@ sds_upload (struct backend *backend, const gchar *path, struct idata *sample,
   struct sample_info *sample_info = sample->info;
   GByteArray *input = sample->content;
 
-  job_control_reset (control, 1);
+  task_control_reset (control, 1);
 
   if (common_slot_get_id_from_path (path, &id))
     {
@@ -768,9 +754,7 @@ sds_upload (struct backend *backend, const gchar *path, struct idata *sample,
   backend_rx_drain (backend);
   g_mutex_unlock (&backend->mutex);
 
-  g_mutex_lock (&control->mutex);
-  active = control->active;
-  g_mutex_unlock (&control->mutex);
+  active = controllable_is_active (&control->controllable);
 
   debug_print (1, "Sending dump header...");
 
@@ -783,7 +767,7 @@ sds_upload (struct backend *backend, const gchar *path, struct idata *sample,
   tx_msg = sds_get_dump_msg (id, words, sample_info, bits);
   //The first timeout should be SDS_SPEC_TIMEOUT_HANDSHAKE (2 s) but it is not enough sometimes.
   err = sds_tx_and_wait_ack (backend, tx_msg, 0, SDS_NO_SPEC_TIMEOUT,
-			     SDS_NO_SPEC_TIMEOUT);
+			     SDS_NO_SPEC_TIMEOUT, &control->controllable);
   if (err == -ENOMSG)
     {
       debug_print (2, "No packet received after a WAIT. Continuing...");
@@ -832,7 +816,8 @@ sds_upload (struct backend *backend, const gchar *path, struct idata *sample,
 	  //SDS_SPEC_TIMEOUT is too low to be used here.
 	  err = sds_tx_and_wait_ack (backend, tx_msg, packet % 0x80,
 				     SDS_NO_SPEC_TIMEOUT,
-				     SDS_NO_SPEC_TIMEOUT);
+				     SDS_NO_SPEC_TIMEOUT,
+				     &control->controllable);
 	}
 
       if (err == -EBADMSG)
@@ -871,10 +856,9 @@ sds_upload (struct backend *backend, const gchar *path, struct idata *sample,
 	  goto end;
 	}
 
-      job_control_set_progress (control, packet / (gdouble) packets);
-      g_mutex_lock (&control->mutex);
-      active = control->active;
-      g_mutex_unlock (&control->mutex);
+      task_control_set_progress (control, packet / (gdouble) packets);
+
+      active = controllable_is_active (&control->controllable);
 
       word = w;
       frame = f;
@@ -893,7 +877,7 @@ sds_upload (struct backend *backend, const gchar *path, struct idata *sample,
 end:
   if (active && packet == packets)
     {
-      job_control_set_progress (control, 1.0);
+      task_control_set_progress (control, 1.0);
     }
   else
     {
@@ -907,28 +891,28 @@ end:
 
 static gint
 sds_upload_8b (struct backend *backend, const gchar *path,
-	       struct idata *sample, struct job_control *control)
+	       struct idata *sample, struct task_control *control)
 {
   return sds_upload (backend, path, sample, control, 8);
 }
 
 static gint
 sds_upload_12b (struct backend *backend, const gchar *path,
-		struct idata *sample, struct job_control *control)
+		struct idata *sample, struct task_control *control)
 {
   return sds_upload (backend, path, sample, control, 12);
 }
 
 static gint
 sds_upload_14b (struct backend *backend, const gchar *path,
-		struct idata *sample, struct job_control *control)
+		struct idata *sample, struct task_control *control)
 {
   return sds_upload (backend, path, sample, control, 14);
 }
 
 static gint
 sds_upload_16b (struct backend *backend, const gchar *path,
-		struct idata *sample, struct job_control *control)
+		struct idata *sample, struct task_control *control)
 {
   return sds_upload (backend, path, sample, control, 16);
 }
@@ -953,7 +937,7 @@ sds_next_sample_dentry (struct item_iterator *iter)
     {
       gchar *name = sds_get_sample_name (iterator_data->backend,
 					 iterator_data->next);
-      snprintf (iter->item.name, LABEL_MAX, "%s", name ? name : "");
+      item_set_name (&iter->item, "%s", name ? name : "");
       g_free (name);
     }
   else
@@ -986,50 +970,50 @@ sds_read_dir (struct backend *backend, struct item_iterator *iter,
 
 static gint
 sds_sample_load_common (const gchar *path, struct idata *sample,
-			struct job_control *control, gint32 rate)
+			struct task_control *control, gint32 rate)
 {
-  return common_sample_load (path, sample, control, rate, SDS_SAMPLE_CHANNELS,
-			     SF_FORMAT_PCM_16);
+  return common_sample_load (path, sample, control, SDS_SAMPLE_CHANNELS, rate,
+			     SF_FORMAT_PCM_16, FALSE);
 }
 
 static gint
-sds_sample_load (const gchar *path, struct idata *sample,
-		 struct job_control *control)
+sds_sample_load (struct backend *backend, const gchar *path,
+		 struct idata *sample, struct task_control *control)
 {
   return sds_sample_load_common (path, sample, control, 0);	// Any sample rate is valid.
 }
 
 static gint
-sds_sample_load_441 (const gchar *path, struct idata *sample,
-		     struct job_control *control)
+sds_sample_load_44k1 (struct backend *backend, const gchar *path,
+		      struct idata *sample, struct task_control *control)
 {
   return sds_sample_load_common (path, sample, control, 44100);
 }
 
 static gint
-sds_sample_load_32 (const gchar *path, struct idata *sample,
-		    struct job_control *control)
+sds_sample_load_32k_16b (struct backend *backend, const gchar *path,
+			 struct idata *sample, struct task_control *control)
 {
   return sds_sample_load_common (path, sample, control, 32000);
 }
 
 static gint
-sds_sample_load_16 (const gchar *path, struct idata *sample,
-		    struct job_control *control)
+sds_sample_load_16k_16b (struct backend *backend, const gchar *path,
+			 struct idata *sample, struct task_control *control)
 {
   return sds_sample_load_common (path, sample, control, 16000);
 }
 
 static gint
-sds_sample_load_8 (const gchar *path, struct idata *sample,
-		   struct job_control *control)
+sds_sample_load_8k_16b (struct backend *backend, const gchar *path,
+			struct idata *sample, struct task_control *control)
 {
   return sds_sample_load_common (path, sample, control, 8000);
 }
 
 static gint
 sds_sample_save (const gchar *path, struct idata *sample,
-		 struct job_control *control)
+		 struct task_control *control)
 {
   return sample_save_to_file (path, sample, control,
 			      SF_FORMAT_WAV | SF_FORMAT_PCM_16);
@@ -1037,23 +1021,24 @@ sds_sample_save (const gchar *path, struct idata *sample,
 
 enum sds_fs
 {
-  FS_SAMPLES_SDS_16_B = 1,	//SDS devices also include FS_PROGRAM_DEFAULT_OPERATIONS as the first filesystem.
-  FS_SAMPLES_SDS_14_B,
-  FS_SAMPLES_SDS_12_B,
-  FS_SAMPLES_SDS_8_B,
-  FS_SAMPLES_SDS_16_B_441,
-  FS_SAMPLES_SDS_16_B_32,
-  FS_SAMPLES_SDS_16_B_16,
-  FS_SAMPLES_SDS_16_B_8
+  FS_SDS_SAMPLES_MONO_16B = 1,	//SDS devices also include FS_PROGRAM_DEFAULT_OPERATIONS as the first filesystem.
+  FS_SDS_SAMPLES_MONO_14B,
+  FS_SDS_SAMPLES_MONO_12B,
+  FS_SDS_SAMPLES_MONO_8B,
+  FS_SDS_SAMPLES_MONO_44K1_16B,
+  FS_SDS_SAMPLES_MONO_32K_16B,
+  FS_SDS_SAMPLES_MONO_16K_16B,
+  FS_SDS_SAMPLES_MONO_8K_16B
 };
 
-static const struct fs_operations FS_SAMPLES_SDS_8B_OPERATIONS = {
-  .id = FS_SAMPLES_SDS_8_B,
+static const struct fs_operations FS_SDS_SAMPLES_MONO_8B_OPERATIONS = {
+  .id = FS_SDS_SAMPLES_MONO_8B,
   .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO | FS_OPTION_SINGLE_OP |
     FS_OPTION_SLOT_STORAGE | FS_OPTION_SHOW_ID_COLUMN,
-  .name = "8b1c",
-  .gui_name = "8 bits mono",
+  .name = "mono-8b",
+  .gui_name = "mono 8 bits",
   .gui_icon = FS_ICON_WAVE,
+  .file_icon = FS_ICON_WAVE,
   .max_name_len = SDS_SAMPLE_NAME_MAX_LEN,
   .readdir = sds_read_dir,
   .print_item = common_print_item,
@@ -1067,13 +1052,14 @@ static const struct fs_operations FS_SAMPLES_SDS_8B_OPERATIONS = {
   .get_download_path = common_system_get_download_path
 };
 
-static const struct fs_operations FS_SAMPLES_SDS_12B_OPERATIONS = {
-  .id = FS_SAMPLES_SDS_12_B,
+static const struct fs_operations FS_SDS_SAMPLES_MONO_12B_OPERATIONS = {
+  .id = FS_SDS_SAMPLES_MONO_12B,
   .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO | FS_OPTION_SINGLE_OP |
     FS_OPTION_SLOT_STORAGE | FS_OPTION_SHOW_ID_COLUMN,
-  .name = "12b1c",
-  .gui_name = "12 bits mono",
+  .name = "mono-12b",
+  .gui_name = "mono 12 bits",
   .gui_icon = FS_ICON_WAVE,
+  .file_icon = FS_ICON_WAVE,
   .max_name_len = SDS_SAMPLE_NAME_MAX_LEN,
   .readdir = sds_read_dir,
   .print_item = common_print_item,
@@ -1087,13 +1073,14 @@ static const struct fs_operations FS_SAMPLES_SDS_12B_OPERATIONS = {
   .get_download_path = common_system_get_download_path
 };
 
-static const struct fs_operations FS_SAMPLES_SDS_14B_OPERATIONS = {
-  .id = FS_SAMPLES_SDS_14_B,
+static const struct fs_operations FS_SDS_SAMPLES_MONO_14B_OPERATIONS = {
+  .id = FS_SDS_SAMPLES_MONO_14B,
   .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO | FS_OPTION_SINGLE_OP |
     FS_OPTION_SLOT_STORAGE | FS_OPTION_SHOW_ID_COLUMN,
-  .name = "14b1c",
-  .gui_name = "14 bits mono",
+  .name = "mono-14b",
+  .gui_name = "mono 14 bits",
   .gui_icon = FS_ICON_WAVE,
+  .file_icon = FS_ICON_WAVE,
   .max_name_len = SDS_SAMPLE_NAME_MAX_LEN,
   .readdir = sds_read_dir,
   .print_item = common_print_item,
@@ -1107,13 +1094,14 @@ static const struct fs_operations FS_SAMPLES_SDS_14B_OPERATIONS = {
   .get_download_path = common_system_get_download_path
 };
 
-static const struct fs_operations FS_SAMPLES_SDS_16B_OPERATIONS = {
-  .id = FS_SAMPLES_SDS_16_B,
+static const struct fs_operations FS_SDS_SAMPLES_MONO_16B_OPERATIONS = {
+  .id = FS_SDS_SAMPLES_MONO_16B,
   .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO | FS_OPTION_SINGLE_OP |
     FS_OPTION_SLOT_STORAGE | FS_OPTION_SHOW_ID_COLUMN,
-  .name = "16b1c",
-  .gui_name = "16 bits mono",
+  .name = "mono-16b",
+  .gui_name = "mono 16 bits",
   .gui_icon = FS_ICON_WAVE,
+  .file_icon = FS_ICON_WAVE,
   .max_name_len = SDS_SAMPLE_NAME_MAX_LEN,
   .readdir = sds_read_dir,
   .print_item = common_print_item,
@@ -1127,80 +1115,84 @@ static const struct fs_operations FS_SAMPLES_SDS_16B_OPERATIONS = {
   .get_download_path = common_system_get_download_path
 };
 
-static const struct fs_operations FS_SAMPLES_SDS_16B_441_OPERATIONS = {
-  .id = FS_SAMPLES_SDS_16_B_441,
+static const struct fs_operations FS_SDS_SAMPLES_MONO_44K1_16B_OPERATIONS = {
+  .id = FS_SDS_SAMPLES_MONO_44K1_16B,
   .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO | FS_OPTION_SINGLE_OP |
     FS_OPTION_SLOT_STORAGE | FS_OPTION_SHOW_ID_COLUMN,
-  .name = "44.1k16b1c",
-  .gui_name = "44.1 KHz 16 bits mono",
+  .name = "mono-44k1-16b",
+  .gui_name = "mono 44.1 kHz 16 bits",
   .gui_icon = FS_ICON_WAVE,
+  .file_icon = FS_ICON_WAVE,
   .max_name_len = SDS_SAMPLE_NAME_MAX_LEN,
   .readdir = sds_read_dir,
   .print_item = common_print_item,
   .rename = sds_rename,
   .download = sds_download,
   .upload = sds_upload_16b,
-  .load = sds_sample_load_441,
+  .load = sds_sample_load_44k1,
   .save = sds_sample_save,
   .get_exts = sample_get_sample_extensions,
   .get_upload_path = common_slot_get_upload_path,
   .get_download_path = common_system_get_download_path
 };
 
-static const struct fs_operations FS_SAMPLES_SDS_16B_32_OPERATIONS = {
-  .id = FS_SAMPLES_SDS_16_B_32,
+static const struct fs_operations FS_SDS_SAMPLES_MONO_32K_16B_OPERATIONS = {
+  .id = FS_SDS_SAMPLES_MONO_32K_16B,
   .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO | FS_OPTION_SINGLE_OP |
     FS_OPTION_SLOT_STORAGE | FS_OPTION_SHOW_ID_COLUMN,
-  .name = "32k16b1c",
-  .gui_name = "32 KHz 16 bits mono",
+  .name = "mono-32k-16b",
+  .gui_name = "mono 32 kHz 16 bits",
   .gui_icon = FS_ICON_WAVE,
+  .file_icon = FS_ICON_WAVE,
   .max_name_len = SDS_SAMPLE_NAME_MAX_LEN,
   .readdir = sds_read_dir,
   .print_item = common_print_item,
   .rename = sds_rename,
   .download = sds_download,
   .upload = sds_upload_16b,
-  .load = sds_sample_load_32,
+  .load = sds_sample_load_32k_16b,
   .save = sds_sample_save,
   .get_exts = sample_get_sample_extensions,
   .get_upload_path = common_slot_get_upload_path,
   .get_download_path = common_system_get_download_path
 };
 
-static const struct fs_operations FS_SAMPLES_SDS_16B_16_OPERATIONS = {
-  .id = FS_SAMPLES_SDS_16_B_16,
+static const struct fs_operations FS_SDS_SAMPLES_MONO_16K_16B_OPERATIONS = {
+  .id = FS_SDS_SAMPLES_MONO_16K_16B,
   .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO | FS_OPTION_SINGLE_OP |
     FS_OPTION_SLOT_STORAGE | FS_OPTION_SHOW_ID_COLUMN,
-  .name = "16k16b1c",
-  .gui_name = "16 KHz 16 bits mono",
+  .name = "mono-16k-16b",
+  .gui_name = "mono 16 kHz 16 bits",
   .gui_icon = FS_ICON_WAVE,
+  .file_icon = FS_ICON_WAVE,
   .max_name_len = SDS_SAMPLE_NAME_MAX_LEN,
   .readdir = sds_read_dir,
   .print_item = common_print_item,
   .rename = sds_rename,
   .download = sds_download,
   .upload = sds_upload_16b,
-  .load = sds_sample_load_16,
+  .load = sds_sample_load_16k_16b,
   .save = sds_sample_save,
   .get_exts = sample_get_sample_extensions,
   .get_upload_path = common_slot_get_upload_path,
   .get_download_path = common_system_get_download_path
 };
 
-static const struct fs_operations FS_SAMPLES_SDS_16B_8_OPERATIONS = {
-  .id = FS_SAMPLES_SDS_16_B_8,
+static const struct fs_operations FS_SDS_SAMPLES_MONO_8K_16B_OPERATIONS = {
+  .id = FS_SDS_SAMPLES_MONO_8K_16B,
   .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO | FS_OPTION_SINGLE_OP |
     FS_OPTION_SLOT_STORAGE | FS_OPTION_SHOW_ID_COLUMN,
-  .name = "8k16b1c",
-  .gui_name = "8 KHz 16 bits mono",
+  .name = "mono-8k-16b",
+  .gui_name = "mono 8 kHz 16 bits",
   .gui_icon = FS_ICON_WAVE,
+  .file_icon = FS_ICON_WAVE,
   .max_name_len = SDS_SAMPLE_NAME_MAX_LEN,
   .readdir = sds_read_dir,
   .print_item = common_print_item,
   .rename = sds_rename,
   .download = sds_download,
   .upload = sds_upload_16b,
-  .load = sds_sample_load_8,
+  .load = sds_sample_load_8k_16b,
   .save = sds_sample_save,
   .get_exts = sample_get_sample_extensions,
   .get_upload_path = common_slot_get_upload_path,
@@ -1278,7 +1270,7 @@ sds_handshake_esi_2000 (struct backend *backend)
   //In case we receive an ACK, NAK or CANCEL, there is a MIDI SDS device listening.
   gint err = sds_tx_and_wait_ack (backend, tx_msg, 0,
 				  SDS_SPEC_TIMEOUT_HANDSHAKE,
-				  SDS_NO_SPEC_TIMEOUT_TRY);
+				  SDS_NO_SPEC_TIMEOUT_TRY, NULL);
   if (err && err != -EBADMSG && err != -ECANCELED)
     {
       return -ENODEV;
@@ -1343,14 +1335,14 @@ end:
   sds_data->name_extension = name_extension;
 
   gslist_fill (&backend->fs_ops, &FS_PROGRAM_DEFAULT_OPERATIONS,
-	       &FS_SAMPLES_SDS_8B_OPERATIONS,
-	       &FS_SAMPLES_SDS_12B_OPERATIONS,
-	       &FS_SAMPLES_SDS_14B_OPERATIONS,
-	       &FS_SAMPLES_SDS_16B_OPERATIONS,
-	       &FS_SAMPLES_SDS_16B_441_OPERATIONS,
-	       &FS_SAMPLES_SDS_16B_32_OPERATIONS,
-	       &FS_SAMPLES_SDS_16B_16_OPERATIONS,
-	       &FS_SAMPLES_SDS_16B_8_OPERATIONS, NULL);
+	       &FS_SDS_SAMPLES_MONO_8B_OPERATIONS,
+	       &FS_SDS_SAMPLES_MONO_12B_OPERATIONS,
+	       &FS_SDS_SAMPLES_MONO_14B_OPERATIONS,
+	       &FS_SDS_SAMPLES_MONO_16B_OPERATIONS,
+	       &FS_SDS_SAMPLES_MONO_44K1_16B_OPERATIONS,
+	       &FS_SDS_SAMPLES_MONO_32K_16B_OPERATIONS,
+	       &FS_SDS_SAMPLES_MONO_16K_16B_OPERATIONS,
+	       &FS_SDS_SAMPLES_MONO_8K_16B_OPERATIONS, NULL);
   backend->destroy_data = backend_destroy_data;
   backend->data = sds_data;
 
@@ -1365,6 +1357,6 @@ end:
 const struct connector CONNECTOR_SDS = {
   .name = "sds",
   .handshake = sds_handshake,
-  .standard = FALSE,
+  .options = CONNECTOR_OPTION_CUSTOM_HANDSHAKE,
   .regex = NULL
 };
