@@ -47,7 +47,6 @@
 
 static const gchar *SUMMIT_TUNING_EXTS[] = { BE_SYSEX_EXT, SCALA_EXT, NULL };
 
-static const guint8 NOVATION_ID[] = { 0x0, 0x20, 0x29 };
 static const guint8 SUMMIT_ID[] = { 0x33, 1, 0, 0 };
 
 static const guint8 SUMMIT_GENERIC_REQ[] =
@@ -186,7 +185,7 @@ summit_patch_next_dentry (struct item_iterator *iter)
   if (data->fs == FS_SUMMIT_SINGLE_PATCH)
     {
       const gchar *category = summit_get_category_name (rx_msg);
-      snprintf (iter->item.object_info, LABEL_MAX, "%s", category);
+      item_set_object_info (&iter->item, "category=%s", category);
     }
   free_msg (rx_msg);
 
@@ -209,7 +208,7 @@ summit_patch_next_dentry_root (struct item_iterator *iter)
   if (*next < 4)
     {
       iter->item.id = 0x10000 + *next;	//Unique id
-      snprintf (iter->item.name, LABEL_MAX, "%c", 0x41 + iter->item.id);
+      item_set_name (&iter->item, "%c", 0x41 + iter->item.id);
       iter->item.type = ITEM_TYPE_DIR;
       iter->item.size = -1;
       iter->item.object_info[0] = 0;
@@ -286,7 +285,7 @@ summit_get_bank_and_id_from_path (const gchar *path, guint8 *bank, guint8 *id)
 
 static gint
 summit_patch_download (struct backend *backend, const gchar *path,
-		       struct idata *patch, struct job_control *control,
+		       struct idata *patch, struct task_control *control,
 		       enum summit_fs fs)
 {
   guint8 id, bank;
@@ -317,7 +316,7 @@ summit_patch_download (struct backend *backend, const gchar *path,
   name[SUMMIT_PATCH_NAME_LEN] = 0;
   summit_truncate_name (&name[SUMMIT_PATCH_NAME_LEN - 1]);
 
-  idata_init (patch, rx_msg, strdup (name), NULL);
+  idata_init (patch, rx_msg, strdup (name), NULL, NULL);
   goto end;
 
 cleanup:
@@ -329,7 +328,7 @@ end:
 
 static gint
 summit_single_download (struct backend *backend, const gchar *path,
-			struct idata *patch, struct job_control *control)
+			struct idata *patch, struct task_control *control)
 {
   return summit_patch_download (backend, path, patch, control,
 				FS_SUMMIT_SINGLE_PATCH);
@@ -337,7 +336,7 @@ summit_single_download (struct backend *backend, const gchar *path,
 
 static gint
 summit_multi_download (struct backend *backend, const gchar *path,
-		       struct idata *patch, struct job_control *control)
+		       struct idata *patch, struct task_control *control)
 {
   return summit_patch_download (backend, path, patch, control,
 				FS_SUMMIT_MULTI_PATCH);
@@ -345,7 +344,7 @@ summit_multi_download (struct backend *backend, const gchar *path,
 
 static gint
 summit_patch_upload (struct backend *backend, const gchar *path,
-		     GByteArray *input, struct job_control *control)
+		     GByteArray *input, struct task_control *control)
 {
   guint8 id, bank;
   gint err;
@@ -377,7 +376,7 @@ end:
 
 static gint
 summit_single_upload (struct backend *backend, const gchar *path,
-		      struct idata *patch, struct job_control *control)
+		      struct idata *patch, struct task_control *control)
 {
   if (patch->content->len != SUMMIT_SINGLE_LEN)
     {
@@ -389,7 +388,7 @@ summit_single_upload (struct backend *backend, const gchar *path,
 
 static gint
 summit_multi_upload (struct backend *backend, const gchar *path,
-		     struct idata *patch, struct job_control *control)
+		     struct idata *patch, struct task_control *control)
 {
   if (patch->content->len != SUMMIT_MULTI_LEN)
     {
@@ -408,18 +407,16 @@ summit_patch_rename (struct backend *backend, const gchar *src,
   gint err, len;
   guint8 *name;
   gchar *sanitized;
-  struct job_control control;
+  struct task_control control;
   debug_print (1, "Renaming from %s to %s...", src, dst);
 
-  //The control initialization is needed.
-  control.active = TRUE;
+  controllable_init (&control.controllable);
   control.callback = NULL;
-  g_mutex_init (&control.mutex);
 
   err = summit_patch_download (backend, src, &preset, &control, fs);
   if (err)
     {
-      return err;
+      goto end;
     }
 
   usleep (SUMMIT_REST_TIME_US);
@@ -442,6 +439,8 @@ summit_patch_rename (struct backend *backend, const gchar *src,
 
   usleep (SUMMIT_REST_TIME_US);
 
+end:
+  controllable_clear (&control.controllable);
   return err;
 }
 
@@ -528,7 +527,8 @@ static const struct fs_operations FS_SUMMIT_SINGLE_OPERATIONS = {
     FS_OPTION_SHOW_INFO_COLUMN | FS_OPTION_ALLOW_SEARCH,
   .name = "single",
   .gui_name = "Single",
-  .gui_icon = FS_ICON_SND,
+  .gui_icon = FS_ICON_PRESET,
+  .file_icon = FS_ICON_PRESET,
   .max_name_len = SUMMIT_PATCH_NAME_LEN,
   .readdir = summit_single_read_dir,
   .print_item = common_print_item,
@@ -536,7 +536,7 @@ static const struct fs_operations FS_SUMMIT_SINGLE_OPERATIONS = {
   .download = summit_single_download,
   .upload = summit_single_upload,
   .get_slot = summit_get_patch_id_as_slot,
-  .load = file_load,
+  .load = common_file_load,
   .save = file_save,
   .get_exts = common_sysex_get_extensions,
   .get_upload_path = common_slot_get_upload_path,
@@ -551,7 +551,8 @@ static const struct fs_operations FS_SUMMIT_MULTI_OPERATIONS = {
     FS_OPTION_ALLOW_SEARCH,
   .name = "multi",
   .gui_name = "Multi",
-  .gui_icon = FS_ICON_SND,
+  .gui_icon = FS_ICON_PRESET,
+  .file_icon = FS_ICON_PRESET,
   .max_name_len = SUMMIT_PATCH_NAME_LEN,
   .readdir = summit_multi_read_dir,
   .print_item = common_print_item,
@@ -559,7 +560,7 @@ static const struct fs_operations FS_SUMMIT_MULTI_OPERATIONS = {
   .download = summit_multi_download,
   .upload = summit_multi_upload,
   .get_slot = summit_get_patch_id_as_slot,
-  .load = file_load,
+  .load = common_file_load,
   .save = file_save,
   .get_exts = common_sysex_get_extensions,
   .get_upload_path = common_slot_get_upload_path,
@@ -589,7 +590,7 @@ summit_tuning_read_dir (struct backend *backend, struct item_iterator *iter,
 
 static gint
 summit_tuning_upload (struct backend *backend, const gchar *path,
-		      struct idata *tuning, struct job_control *control)
+		      struct idata *tuning, struct task_control *control)
 {
   guint id;
   GByteArray *input = tuning->content;
@@ -616,7 +617,7 @@ summit_tuning_upload (struct backend *backend, const gchar *path,
 
 static gint
 summit_tuning_download (struct backend *backend, const gchar *path,
-			struct idata *tuning, struct job_control *control)
+			struct idata *tuning, struct task_control *control)
 {
   guint32 id;
   gint err = 0;
@@ -646,7 +647,7 @@ summit_tuning_download (struct backend *backend, const gchar *path,
       goto cleanup;
     }
 
-  idata_init (tuning, rx_msg, NULL, NULL);
+  idata_init (tuning, rx_msg, NULL, NULL, NULL);
   goto end;
 
 cleanup:
@@ -657,8 +658,8 @@ end:
 }
 
 static gint
-summit_tuning_load (const gchar *path, struct idata *tuning,
-		    struct job_control *control)
+summit_tuning_load (struct backend *backend, const gchar *path,
+		    struct idata *tuning, struct task_control *control)
 {
   gint err;
   gchar *filename = g_path_get_basename (path);
@@ -668,15 +669,15 @@ summit_tuning_load (const gchar *path, struct idata *tuning,
     }
   else
     {
-      err = scl_load_key_based_tuning_msg_from_scala_file (path, tuning,
-							   control);
+      err = scala_load_key_based_tuning_msg (path, tuning, control);
     }
   g_free (filename);
   return err;
 }
 
 static const gchar **
-summit_tuning_get_extensions ()
+summit_tuning_get_extensions (struct backend *backend,
+			      const struct fs_operations *ops)
 {
   return SUMMIT_TUNING_EXTS;
 }
@@ -687,6 +688,7 @@ static const struct fs_operations FS_SUMMIT_BULK_TUNING_OPERATIONS = {
   .name = "tuning",
   .gui_name = "Tuning Tables",
   .gui_icon = FS_ICON_KEYS,
+  .file_icon = FS_ICON_KEYS,
   .readdir = summit_tuning_read_dir,
   .print_item = common_print_item,
   .download = summit_tuning_download,
@@ -777,7 +779,7 @@ summit_wavetable_read_dir (struct backend *backend,
 static gint
 summit_wavetable_download (struct backend *backend, const gchar *path,
 			   struct idata *wavetable,
-			   struct job_control *control)
+			   struct task_control *control)
 {
   guint32 id;
   gint err = 0;
@@ -793,7 +795,7 @@ summit_wavetable_download (struct backend *backend, const gchar *path,
       return -EINVAL;
     }
 
-  job_control_reset (control, 6);
+  task_control_reset (control, 6);
 
   output = g_byte_array_new ();
 
@@ -841,7 +843,7 @@ summit_wavetable_download (struct backend *backend, const gchar *path,
   name[SUMMIT_WAVETABLE_NAME_LEN] = 0;
   summit_truncate_name (&name[SUMMIT_WAVETABLE_NAME_LEN - 1]);
 
-  idata_init (wavetable, output, strdup (name), NULL);
+  idata_init (wavetable, output, strdup (name), NULL, NULL);
   goto end;
 
 err:
@@ -854,7 +856,8 @@ end:
 
 static gint
 summit_wavetable_upload (struct backend *backend, const gchar *path,
-			 struct idata *wavetable, struct job_control *control)
+			 struct idata *wavetable,
+			 struct task_control *control)
 {
   guint id;
   GByteArray *input = wavetable->content;
@@ -927,13 +930,14 @@ static const struct fs_operations FS_SUMMIT_WAVETABLE_OPERATIONS = {
   .name = "wavetable",
   .gui_name = "Wavetables",
   .gui_icon = FS_ICON_WAVETABLE,
+  .file_icon = FS_ICON_WAVETABLE,
   .max_name_len = SUMMIT_WAVETABLE_NAME_LEN,
   .readdir = summit_wavetable_read_dir,
   .print_item = common_print_item,
   .rename = summit_wavetable_rename,
   .download = summit_wavetable_download,
   .upload = summit_wavetable_upload,
-  .load = file_load,
+  .load = common_file_load,
   .save = file_save,
   .get_exts = common_sysex_get_extensions,
   .get_download_path = common_slot_get_download_path_n,
@@ -961,6 +965,6 @@ summit_handshake (struct backend *backend)
 const struct connector CONNECTOR_SUMMIT = {
   .handshake = summit_handshake,
   .name = "summit",
-  .standard = TRUE,
+  .options = 0,
   .regex = ".*(Peak|Summit).*"
 };
