@@ -18,6 +18,7 @@
  *   along with Elektroid. If not, see <http://www.gnu.org/licenses/>.
  */
 
+#define _FILE_OFFSET_BITS 64
 #include <errno.h>
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
@@ -37,17 +38,17 @@ struct system_iterator_data
 
 static gint
 system_download (struct backend *backend, const gchar *path,
-		 struct idata *idata, struct job_control *control)
+		 struct idata *idata, struct task_control *control)
 {
   gint err;
 
-  job_control_reset (control, 1);
+  task_control_reset (control, 1);
 
   err = file_load (path, idata, control);
 
   if (!err)
     {
-      job_control_set_progress (control, 1.0);
+      task_control_set_progress (control, 1.0);
     }
 
   return err;
@@ -117,11 +118,12 @@ system_free_iterator_data (void *iter_data)
 static gint
 system_next_dentry (struct item_iterator *iter, gboolean sample_info)
 {
+  gint err;
   gchar *full_path;
   const gchar *name;
-  struct stat st;
   struct system_iterator_data *data = iter->data;
 
+  err = -ENOENT;
   while ((name = g_dir_read_name (data->dir)) != NULL)
     {
       if (name[0] == '.')
@@ -145,30 +147,41 @@ system_next_dentry (struct item_iterator *iter, gboolean sample_info)
 	  continue;
 	}
 
-      if (!stat (full_path, &st))
+      GFile *file = g_file_new_for_path (full_path);
+      GError *error = NULL;
+      GFileInfo *info = g_file_query_info (file, "standard::*",
+					   G_FILE_QUERY_INFO_NONE, NULL,
+					   &error);
+      if (error == NULL)
 	{
-	  snprintf (iter->item.name, LABEL_MAX, "%s", name);
+	  item_set_name (&iter->item, "%s", name);
 	  iter->item.type = type;
-	  iter->item.size = st.st_size;
+	  iter->item.size = g_file_info_get_size (info);
 	  iter->item.id = -1;
 
-	  if (item_iterator_is_dir_or_matches_extensions (iter,
-							  data->extensions))
+	  if (item_iterator_is_dir_or_matches_exts (iter, data->extensions))
 	    {
 	      if (iter->item.type == ITEM_TYPE_FILE && sample_info)
 		{
 		  sample_load_sample_info (full_path,
 					   &iter->item.sample_info);
 		}
-	      g_free (full_path);
-	      return 0;
+	      err = 0;
 	    }
+
+	  g_object_unref (info);
 	}
 
+      g_object_unref (file);
       g_free (full_path);
+
+      if (!err)
+	{
+	  break;
+	}
     }
 
-  return -ENOENT;
+  return err;
 }
 
 static gint
@@ -183,19 +196,77 @@ system_next_dentry_with_sample_info (struct item_iterator *iter)
   return system_next_dentry (iter, TRUE);
 }
 
+#if defined(__MINGW32__) | defined(__MINGW64__)
+struct windows_drive_iterator_data
+{
+  GList *mounts;
+  GList *next;
+};
+
+static int
+windows_drive_next_dentry (struct item_iterator *iter)
+{
+  struct windows_drive_iterator_data *data = iter->data;
+
+  if (!data->next)
+    {
+      return -ENOENT;
+    }
+
+  GMount *gmount = data->next->data;
+  GFile *gfile = g_mount_get_root (gmount);
+  gchar *name = g_file_get_path (gfile);
+  g_object_unref (gfile);
+  item_set_name (&iter->item, "%s", name);
+  g_free (name);
+  iter->item.type = ITEM_TYPE_DIR;
+  iter->item.size = -1;
+  iter->item.id = -1;
+
+  data->next = data->next->next;
+
+  return 0;
+}
+
+static void
+windows_drive_free_iterator_data (void *iter_data)
+{
+  struct windows_drive_iterator_data *data = iter_data;
+  g_list_free_full (data->mounts, g_object_unref);
+  g_free (data);
+}
+#endif
+
 static gint
 system_read_dir_opts (struct backend *backend, struct item_iterator *iter,
 		      const gchar *dir, const gchar **extensions,
 		      iterator_next next)
 {
+#if defined(__MINGW32__) | defined(__MINGW64__)
+  if (!strcmp (dir, TOPMOST_DIR_WINDOWS))
+    {
+      GVolumeMonitor *monitor = g_volume_monitor_get ();
+      struct windows_drive_iterator_data *data;
+
+      data = g_malloc (sizeof (struct windows_drive_iterator_data));
+      data->mounts = g_volume_monitor_get_mounts (monitor);
+      data->next = data->mounts;
+
+      item_iterator_init (iter, dir, data, windows_drive_next_dentry,
+			  windows_drive_free_iterator_data);
+
+      return 0;
+    }
+#endif
+
   GDir *gdir;
-  struct system_iterator_data *data;
 
   if (!(gdir = g_dir_open (dir, 0, NULL)))
     {
       return -errno;
     }
 
+  struct system_iterator_data *data;
   data = g_malloc (sizeof (struct system_iterator_data));
   data->dir = gdir;
   data->extensions = extensions;
@@ -223,31 +294,31 @@ system_samples_read_dir (struct backend *backend, struct item_iterator *iter,
 
 static gint
 system_load_custom (const gchar *path, struct idata *sample,
-		    struct job_control *control,
-		    const struct sample_info *sample_info_req)
+		    struct task_control *control, guint32 channels,
+		    guint32 rate, guint32 format)
 {
   struct sample_info sample_info_src;
+  struct sample_load_opts opts;
+  sample_load_opts_init (&opts, channels, rate, format, FALSE);
   //Typically, control parts are set not here but in this case makes more sense.
   control->parts = 1;
   control->part = 0;
-  return sample_load_from_file (path, sample, control, sample_info_req,
+  return sample_load_from_file (path, sample, control, &opts,
 				&sample_info_src);
 }
 
 static gint
-system_load_48_16_stereo (const gchar *path, struct idata *sample,
-			  struct job_control *control)
+system_load_stereo_48k_16b (struct backend *backend, const gchar *path,
+			    struct idata *sample,
+			    struct task_control *control)
 {
-  struct sample_info sample_info_dst;
-  sample_info_dst.rate = 48000;
-  sample_info_dst.channels = 2;
-  sample_info_dst.format = SF_FORMAT_PCM_16;
-  return system_load_custom (path, sample, control, &sample_info_dst);
+  return system_load_custom (path, sample, control, 2, 48000,
+			     SF_FORMAT_PCM_16);
 }
 
 gint
 system_upload (struct backend *backend, const gchar *path,
-	       struct idata *sample, struct job_control *control)
+	       struct idata *sample, struct task_control *control)
 {
   //Typically, control parts are set here but in this case makes more sense do it in the load functions.
   return sample_save_to_file (path, sample, control,
@@ -255,107 +326,103 @@ system_upload (struct backend *backend, const gchar *path,
 }
 
 static gint
-system_load_48_16_mono (const gchar *path, struct idata *sample,
-			struct job_control *control)
+system_load_mono_48k_16b (struct backend *backend, const gchar *path,
+			  struct idata *sample, struct task_control *control)
 {
-  struct sample_info sample_info_dst;
-  sample_info_dst.rate = 48000;
-  sample_info_dst.channels = 1;
-  sample_info_dst.format = SF_FORMAT_PCM_16;
-  return system_load_custom (path, sample, control, &sample_info_dst);
+  return system_load_custom (path, sample, control, 1, 48000,
+			     SF_FORMAT_PCM_16);
 }
 
 static gint
-system_load_441_16_stereo (const gchar *path, struct idata *sample,
-			   struct job_control *control)
+system_load_stereo_44k1_16b (struct backend *backend, const gchar *path,
+			     struct idata *sample,
+			     struct task_control *control)
 {
-  struct sample_info sample_info_dst;
-  sample_info_dst.rate = 44100;
-  sample_info_dst.channels = 2;
-  sample_info_dst.format = SF_FORMAT_PCM_16;
-  return system_load_custom (path, sample, control, &sample_info_dst);
+  return system_load_custom (path, sample, control, 2, 44100,
+			     SF_FORMAT_PCM_16);
 }
 
 static gint
-system_load_441_16_mono (const gchar *path, struct idata *sample,
-			 struct job_control *control)
+system_load_mono_44k1_16b (struct backend *backend, const gchar *path,
+			   struct idata *sample, struct task_control *control)
 {
-  struct sample_info sample_info_dst;
-  sample_info_dst.rate = 44100;
-  sample_info_dst.channels = 1;
-  sample_info_dst.format = SF_FORMAT_PCM_16;
-  return system_load_custom (path, sample, control, &sample_info_dst);
+  return system_load_custom (path, sample, control, 1, 44100,
+			     SF_FORMAT_PCM_16);
 }
 
 static gint
-system_load_441_24_stereo (const gchar *path, struct idata *sample,
-			   struct job_control *control)
+system_load_stereo_44k1_24b (struct backend *backend, const gchar *path,
+			     struct idata *sample,
+			     struct task_control *control)
 {
-  struct sample_info sample_info_dst;
-  sample_info_dst.rate = 44100;
-  sample_info_dst.channels = 2;
-  sample_info_dst.format = SF_FORMAT_PCM_32;
-  return system_load_custom (path, sample, control, &sample_info_dst);
+  return system_load_custom (path, sample, control, 2, 44100,
+			     SF_FORMAT_PCM_32);
 }
 
 static gint
 system_upload_24_bits (struct backend *backend, const gchar *path,
-		       struct idata *sample, struct job_control *control)
+		       struct idata *sample, struct task_control *control)
 {
   return sample_save_to_file (path, sample, control,
 			      SF_FORMAT_WAV | SF_FORMAT_PCM_24);
 }
 
 static gint
-system_load_441_24_mono (const gchar *path, struct idata *sample,
-			 struct job_control *control)
+system_load_mono_44k1_24b (struct backend *backend, const gchar *path,
+			   struct idata *sample, struct task_control *control)
 {
-  struct sample_info sample_info_dst;
-  sample_info_dst.rate = 44100;
-  sample_info_dst.channels = 1;
-  sample_info_dst.format = SF_FORMAT_PCM_32;
-  return system_load_custom (path, sample, control, &sample_info_dst);
-}
-
-static gint
-system_load_441_8_stereo (const gchar *path, struct idata *sample,
-			  struct job_control *control)
-{
-  struct sample_info sample_info_dst;
-  sample_info_dst.rate = 44100;
-  sample_info_dst.channels = 2;
-  sample_info_dst.format = SF_FORMAT_PCM_16;
-  return system_load_custom (path, sample, control, &sample_info_dst);
+  return system_load_custom (path, sample, control, 1, 44100,
+			     SF_FORMAT_PCM_32);
 }
 
 static gint
 system_upload_8_bits (struct backend *backend, const gchar *path,
-		      struct idata *sample, struct job_control *control)
+		      struct idata *sample, struct task_control *control)
 {
   return sample_save_to_file (path, sample, control,
 			      SF_FORMAT_WAV | SF_FORMAT_PCM_U8);
 }
 
 static gint
-system_load_441_8_mono (const gchar *path, struct idata *sample,
-			struct job_control *control)
+system_load_mono_32k_16b (struct backend *backend, const gchar *path,
+			  struct idata *sample, struct task_control *control)
 {
-  struct sample_info sample_info_dst;
-  sample_info_dst.rate = 44100;
-  sample_info_dst.channels = 1;
-  sample_info_dst.format = SF_FORMAT_PCM_16;
-  return system_load_custom (path, sample, control, &sample_info_dst);
+  return system_load_custom (path, sample, control, 1, 32000,
+			     SF_FORMAT_PCM_16);
 }
 
 static gint
-system_load_32_16_mono (const gchar *path, struct idata *sample,
-			struct job_control *control)
+system_load_mono_31k25_16b (struct backend *backend, const gchar *path,
+			    struct idata *sample,
+			    struct task_control *control)
 {
-  struct sample_info sample_info_dst;
-  sample_info_dst.rate = 32000;
-  sample_info_dst.channels = 1;
-  sample_info_dst.format = SF_FORMAT_PCM_16;
-  return system_load_custom (path, sample, control, &sample_info_dst);
+  return system_load_custom (path, sample, control, 1, 31250,
+			     SF_FORMAT_PCM_16);
+}
+
+static gint
+system_load_mono_22k05_16b (struct backend *backend, const gchar *path,
+			    struct idata *sample,
+			    struct task_control *control)
+{
+  return system_load_custom (path, sample, control, 1, 22050,
+			     SF_FORMAT_PCM_16);
+}
+
+static gint
+system_load_mono_16k_16b (struct backend *backend, const gchar *path,
+			  struct idata *sample, struct task_control *control)
+{
+  return system_load_custom (path, sample, control, 1, 16000,
+			     SF_FORMAT_PCM_16);
+}
+
+static gint
+system_load_mono_8k_16b (struct backend *backend, const gchar *path,
+			 struct idata *sample, struct task_control *control)
+{
+  return system_load_custom (path, sample, control, 1, 8000,
+			     SF_FORMAT_PCM_16);
 }
 
 gboolean
@@ -366,25 +433,32 @@ system_file_exists (struct backend *backend, const gchar *path)
 
 enum system_fs
 {
-  FS_SAMPLES_LOCAL_48_16_STEREO,
-  FS_SAMPLES_LOCAL_48_16_MONO,
-  FS_SAMPLES_LOCAL_441_16_STEREO,
-  FS_SAMPLES_LOCAL_441_16_MONO,
-  FS_SAMPLES_LOCAL_441_24_STEREO,
-  FS_SAMPLES_LOCAL_441_24_MONO,
-  FS_SAMPLES_LOCAL_441_8_STEREO,
-  FS_SAMPLES_LOCAL_441_8_MONO,
-  FS_SAMPLES_LOCAL_32_16_MONO
+  FS_SYSTEM_SAMPLES_STEREO_48K_16B,
+  FS_SYSTEM_SAMPLES_MONO_48K_16B,
+  FS_SYSTEM_SAMPLES_STEREO_44K1_16B,
+  FS_SYSTEM_SAMPLES_MONO_44K1_16B,
+  FS_SYSTEM_SAMPLES_STEREO_44K1_24B,
+  FS_SYSTEM_SAMPLES_MONO_44K1_24B,
+  FS_SYSTEM_SAMPLES_STEREO_441K_8B,
+  FS_SYSTEM_SAMPLES_MONO_44K1_8B,
+  FS_SYSTEM_SAMPLES_MONO_32K_16B,
+  FS_SYSTEM_SAMPLES_MONO_31K25_16B,
+  FS_SYSTEM_SAMPLES_MONO_22K05_16B,
+  FS_SYSTEM_SAMPLES_MONO_16K_16B,
+  FS_SYSTEM_SAMPLES_MONO_16K_8B,
+  FS_SYSTEM_SAMPLES_MONO_8K_16B,
+  FS_SYSTEM_SAMPLES_MONO_8K_8B,
 };
 
-const struct fs_operations FS_SYSTEM_SAMPLES_48_16_STEREO_OPERATIONS = {
-  .id = FS_SAMPLES_LOCAL_48_16_STEREO,
+const struct fs_operations FS_SYSTEM_SAMPLES_STEREO_48K_16B_OPERATIONS = {
+  .id = FS_SYSTEM_SAMPLES_STEREO_48K_16B,
   .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_STEREO |
     FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SAMPLE_COLUMNS |
     FS_OPTION_ALLOW_SEARCH,
-  .name = "wav48k16b2c",
-  .gui_name = "WAV 48 KHz 16 bits stereo",
+  .name = "wav-stereo-48k-16b",
+  .gui_name = "WAV stereo 48 kHz 16 bits",
   .gui_icon = FS_ICON_WAVE,
+  .file_icon = FS_ICON_WAVE,
   .readdir = system_samples_read_dir,
   .print_item = common_print_item,
   .file_exists = system_file_exists,
@@ -394,7 +468,7 @@ const struct fs_operations FS_SYSTEM_SAMPLES_48_16_STEREO_OPERATIONS = {
   .move = system_rename,
   .download = system_download,
   .upload = system_upload,
-  .load = system_load_48_16_stereo,
+  .load = system_load_stereo_48k_16b,
   .save = file_save,
   .get_upload_path = common_system_get_upload_path,
   .get_download_path = common_system_get_download_path,
@@ -402,14 +476,15 @@ const struct fs_operations FS_SYSTEM_SAMPLES_48_16_STEREO_OPERATIONS = {
   .max_name_len = 255
 };
 
-const struct fs_operations FS_SYSTEM_SAMPLES_48_16_MONO_OPERATIONS = {
-  .id = FS_SAMPLES_LOCAL_48_16_MONO,
+const struct fs_operations FS_SYSTEM_SAMPLES_MONO_48K_16B_OPERATIONS = {
+  .id = FS_SYSTEM_SAMPLES_MONO_48K_16B,
   .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO |
     FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SAMPLE_COLUMNS |
     FS_OPTION_ALLOW_SEARCH,
-  .name = "wav48k16b1c",
-  .gui_name = "WAV 48 KHz 16 bits mono",
+  .name = "wav-mono-48k-16b",
+  .gui_name = "WAV mono 48 kHz 16 bits",
   .gui_icon = FS_ICON_WAVE,
+  .file_icon = FS_ICON_WAVE,
   .readdir = system_samples_read_dir,
   .print_item = common_print_item,
   .file_exists = system_file_exists,
@@ -419,7 +494,7 @@ const struct fs_operations FS_SYSTEM_SAMPLES_48_16_MONO_OPERATIONS = {
   .move = system_rename,
   .download = system_download,
   .upload = system_upload,
-  .load = system_load_48_16_mono,
+  .load = system_load_mono_48k_16b,
   .save = file_save,
   .get_upload_path = common_system_get_upload_path,
   .get_download_path = common_system_get_download_path,
@@ -427,14 +502,15 @@ const struct fs_operations FS_SYSTEM_SAMPLES_48_16_MONO_OPERATIONS = {
   .max_name_len = 255
 };
 
-const struct fs_operations FS_SYSTEM_SAMPLES_441_16_STEREO_OPERATIONS = {
-  .id = FS_SAMPLES_LOCAL_441_16_STEREO,
+const struct fs_operations FS_SYSTEM_SAMPLES_STEREO_44K1_16B_OPERATIONS = {
+  .id = FS_SYSTEM_SAMPLES_STEREO_44K1_16B,
   .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_STEREO |
     FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SAMPLE_COLUMNS |
     FS_OPTION_ALLOW_SEARCH,
-  .name = "wav44.1k16b2c",
-  .gui_name = "WAV 44.1 KHz 16 bits stereo",
+  .name = "wav-stereo-44k1-16b",
+  .gui_name = "WAV stereo 44.1 kHz 16 bits",
   .gui_icon = FS_ICON_WAVE,
+  .file_icon = FS_ICON_WAVE,
   .readdir = system_samples_read_dir,
   .print_item = common_print_item,
   .file_exists = system_file_exists,
@@ -444,7 +520,7 @@ const struct fs_operations FS_SYSTEM_SAMPLES_441_16_STEREO_OPERATIONS = {
   .move = system_rename,
   .download = system_download,
   .upload = system_upload,
-  .load = system_load_441_16_stereo,
+  .load = system_load_stereo_44k1_16b,
   .save = file_save,
   .get_upload_path = common_system_get_upload_path,
   .get_download_path = common_system_get_download_path,
@@ -452,14 +528,15 @@ const struct fs_operations FS_SYSTEM_SAMPLES_441_16_STEREO_OPERATIONS = {
   .max_name_len = 255
 };
 
-const struct fs_operations FS_SYSTEM_SAMPLES_441_16_MONO_OPERATIONS = {
-  .id = FS_SAMPLES_LOCAL_441_16_MONO,
+const struct fs_operations FS_SYSTEM_SAMPLES_MONO_44K1_16B_OPERATIONS = {
+  .id = FS_SYSTEM_SAMPLES_MONO_44K1_16B,
   .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO |
     FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SAMPLE_COLUMNS |
     FS_OPTION_ALLOW_SEARCH,
-  .name = "wav44.1k16b1c",
-  .gui_name = "WAV 44.1 KHz 16 bits mono",
+  .name = "wav-mono-44k1-16b",
+  .gui_name = "WAV mono 44.1 kHz 16 bits",
   .gui_icon = FS_ICON_WAVE,
+  .file_icon = FS_ICON_WAVE,
   .readdir = system_samples_read_dir,
   .print_item = common_print_item,
   .file_exists = system_file_exists,
@@ -467,12 +544,9 @@ const struct fs_operations FS_SYSTEM_SAMPLES_441_16_MONO_OPERATIONS = {
   .delete = system_delete,
   .rename = system_rename,
   .move = system_rename,
-  .copy = NULL,
-  .clear = NULL,
-  .swap = NULL,
   .download = system_download,
   .upload = system_upload,
-  .load = system_load_441_16_mono,
+  .load = system_load_mono_44k1_16b,
   .save = file_save,
   .get_upload_path = common_system_get_upload_path,
   .get_download_path = common_system_get_download_path,
@@ -480,15 +554,17 @@ const struct fs_operations FS_SYSTEM_SAMPLES_441_16_MONO_OPERATIONS = {
   .max_name_len = 255
 };
 
-const struct fs_operations FS_SYSTEM_SAMPLES_441_24_STEREO_OPERATIONS = {
-  .id = FS_SAMPLES_LOCAL_441_24_STEREO,
+const struct fs_operations FS_SYSTEM_SAMPLES_STEREO_44K1_24B_OPERATIONS = {
+  .id = FS_SYSTEM_SAMPLES_STEREO_44K1_24B,
   .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_STEREO |
     FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SAMPLE_COLUMNS |
     FS_OPTION_ALLOW_SEARCH,
-  .name = "wav44.1k24b2c",
-  .gui_name = "WAV 44.1 KHz 24 bits stereo",
+  .name = "wav-stereo-44k1-24b",
+  .gui_name = "WAV stereo 44.1 kHz 24 bits",
   .gui_icon = FS_ICON_WAVE,
+  .file_icon = FS_ICON_WAVE,
   .readdir = system_samples_read_dir,
+  .print_item = common_print_item,
   .file_exists = system_file_exists,
   .mkdir = system_mkdir,
   .delete = system_delete,
@@ -496,7 +572,7 @@ const struct fs_operations FS_SYSTEM_SAMPLES_441_24_STEREO_OPERATIONS = {
   .move = system_rename,
   .download = system_download,
   .upload = system_upload_24_bits,
-  .load = system_load_441_24_stereo,
+  .load = system_load_stereo_44k1_24b,
   .save = file_save,
   .get_upload_path = common_system_get_upload_path,
   .get_download_path = common_system_get_download_path,
@@ -504,26 +580,25 @@ const struct fs_operations FS_SYSTEM_SAMPLES_441_24_STEREO_OPERATIONS = {
   .max_name_len = 255
 };
 
-const struct fs_operations FS_SYSTEM_SAMPLES_441_24_MONO_OPERATIONS = {
-  .id = FS_SAMPLES_LOCAL_441_24_MONO,
+const struct fs_operations FS_SYSTEM_SAMPLES_MONO_44K1_24B_OPERATIONS = {
+  .id = FS_SYSTEM_SAMPLES_MONO_44K1_24B,
   .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO |
     FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SAMPLE_COLUMNS |
     FS_OPTION_ALLOW_SEARCH,
-  .name = "wav44.1k24b1c",
-  .gui_name = "WAV 44.1 KHz 24 bits mono",
+  .name = "wav-mono-44k1-24b",
+  .gui_name = "WAV mono 44.1 kHz 24 bits",
   .gui_icon = FS_ICON_WAVE,
+  .file_icon = FS_ICON_WAVE,
   .readdir = system_samples_read_dir,
+  .print_item = common_print_item,
   .file_exists = system_file_exists,
   .mkdir = system_mkdir,
   .delete = system_delete,
   .rename = system_rename,
   .move = system_rename,
-  .copy = NULL,
-  .clear = NULL,
-  .swap = NULL,
   .download = system_download,
   .upload = system_upload_24_bits,
-  .load = system_load_441_24_mono,
+  .load = system_load_mono_44k1_24b,
   .save = file_save,
   .get_upload_path = common_system_get_upload_path,
   .get_download_path = common_system_get_download_path,
@@ -531,15 +606,17 @@ const struct fs_operations FS_SYSTEM_SAMPLES_441_24_MONO_OPERATIONS = {
   .max_name_len = 255
 };
 
-const struct fs_operations FS_SYSTEM_SAMPLES_441_8_STEREO_OPERATIONS = {
-  .id = FS_SAMPLES_LOCAL_441_8_STEREO,
+const struct fs_operations FS_SYSTEM_SAMPLES_STEREO_441K_8B_OPERATIONS = {
+  .id = FS_SYSTEM_SAMPLES_STEREO_441K_8B,
   .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_STEREO |
     FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SAMPLE_COLUMNS |
     FS_OPTION_ALLOW_SEARCH,
-  .name = "wav44.1k8b2c",
-  .gui_name = "WAV 44.1 KHz 8 bits stereo",
+  .name = "wav-stereo-44k1-8b",
+  .gui_name = "WAV stereo 44.1 kHz 8 bits",
   .gui_icon = FS_ICON_WAVE,
+  .file_icon = FS_ICON_WAVE,
   .readdir = system_samples_read_dir,
+  .print_item = common_print_item,
   .file_exists = system_file_exists,
   .mkdir = system_mkdir,
   .delete = system_delete,
@@ -547,7 +624,7 @@ const struct fs_operations FS_SYSTEM_SAMPLES_441_8_STEREO_OPERATIONS = {
   .move = system_rename,
   .download = system_download,
   .upload = system_upload_8_bits,
-  .load = system_load_441_8_stereo,
+  .load = system_load_stereo_44k1_16b,
   .save = file_save,
   .get_upload_path = common_system_get_upload_path,
   .get_download_path = common_system_get_download_path,
@@ -555,26 +632,25 @@ const struct fs_operations FS_SYSTEM_SAMPLES_441_8_STEREO_OPERATIONS = {
   .max_name_len = 255
 };
 
-const struct fs_operations FS_SYSTEM_SAMPLES_441_8_MONO_OPERATIONS = {
-  .id = FS_SAMPLES_LOCAL_441_8_MONO,
+const struct fs_operations FS_SYSTEM_SAMPLES_MONO_44K1_8B_OPERATIONS = {
+  .id = FS_SYSTEM_SAMPLES_MONO_44K1_8B,
   .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO |
     FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SAMPLE_COLUMNS |
     FS_OPTION_ALLOW_SEARCH,
-  .name = "wav44.1k8b1c",
-  .gui_name = "WAV 44.1 KHz 8 bits mono",
+  .name = "wav-mono-44k1-8b",
+  .gui_name = "WAV mono 44.1 kHz 8 bit",
   .gui_icon = FS_ICON_WAVE,
+  .file_icon = FS_ICON_WAVE,
   .readdir = system_samples_read_dir,
+  .print_item = common_print_item,
   .file_exists = system_file_exists,
   .mkdir = system_mkdir,
   .delete = system_delete,
   .rename = system_rename,
   .move = system_rename,
-  .copy = NULL,
-  .clear = NULL,
-  .swap = NULL,
   .download = system_download,
   .upload = system_upload_8_bits,
-  .load = system_load_441_8_mono,
+  .load = system_load_mono_44k1_16b,
   .save = file_save,
   .get_upload_path = common_system_get_upload_path,
   .get_download_path = common_system_get_download_path,
@@ -582,26 +658,181 @@ const struct fs_operations FS_SYSTEM_SAMPLES_441_8_MONO_OPERATIONS = {
   .max_name_len = 255
 };
 
-const struct fs_operations FS_SYSTEM_SAMPLES_32_16_MONO_OPERATIONS = {
-  .id = FS_SAMPLES_LOCAL_32_16_MONO,
+const struct fs_operations FS_SYSTEM_SAMPLES_MONO_32K_16B_OPERATIONS = {
+  .id = FS_SYSTEM_SAMPLES_MONO_32K_16B,
   .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO |
     FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SAMPLE_COLUMNS |
     FS_OPTION_ALLOW_SEARCH,
-  .name = "wav32k8b1c",
-  .gui_name = "WAV 32 KHz 16 bits mono",
+  .name = "wav-mono-32k-16b",
+  .gui_name = "WAV mono 32 kHz 16 bits",
   .gui_icon = FS_ICON_WAVE,
+  .file_icon = FS_ICON_WAVE,
   .readdir = system_samples_read_dir,
+  .print_item = common_print_item,
   .file_exists = system_file_exists,
   .mkdir = system_mkdir,
   .delete = system_delete,
   .rename = system_rename,
   .move = system_rename,
-  .copy = NULL,
-  .clear = NULL,
-  .swap = NULL,
   .download = system_download,
   .upload = system_upload,
-  .load = system_load_32_16_mono,
+  .load = system_load_mono_32k_16b,
+  .save = file_save,
+  .get_upload_path = common_system_get_upload_path,
+  .get_download_path = common_system_get_download_path,
+  .get_exts = sample_get_sample_extensions,
+  .max_name_len = 255
+};
+
+const struct fs_operations FS_SYSTEM_SAMPLES_MONO_31K25_16B_OPERATIONS = {
+  .id = FS_SYSTEM_SAMPLES_MONO_31K25_16B,
+  .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO |
+    FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SAMPLE_COLUMNS |
+    FS_OPTION_ALLOW_SEARCH,
+  .name = "wav-mono-31k25-16b",
+  .gui_name = "WAV mono 31.25 KHz 16 bits",
+  .gui_icon = FS_ICON_WAVE,
+  .file_icon = FS_ICON_WAVE,
+  .readdir = system_samples_read_dir,
+  .print_item = common_print_item,
+  .file_exists = system_file_exists,
+  .mkdir = system_mkdir,
+  .delete = system_delete,
+  .rename = system_rename,
+  .move = system_rename,
+  .download = system_download,
+  .upload = system_upload,
+  .load = system_load_mono_31k25_16b,
+  .save = file_save,
+  .get_upload_path = common_system_get_upload_path,
+  .get_download_path = common_system_get_download_path,
+  .get_exts = sample_get_sample_extensions,
+  .max_name_len = 255
+};
+
+const struct fs_operations FS_SYSTEM_SAMPLES_MONO_22K05_16B_OPERATIONS = {
+  .id = FS_SYSTEM_SAMPLES_MONO_22K05_16B,
+  .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO |
+    FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SAMPLE_COLUMNS |
+    FS_OPTION_ALLOW_SEARCH,
+  .name = "wav-mono-22k05-16b",
+  .gui_name = "WAV mono 22.05 KHz 16 bits",
+  .gui_icon = FS_ICON_WAVE,
+  .file_icon = FS_ICON_WAVE,
+  .readdir = system_samples_read_dir,
+  .print_item = common_print_item,
+  .file_exists = system_file_exists,
+  .mkdir = system_mkdir,
+  .delete = system_delete,
+  .rename = system_rename,
+  .move = system_rename,
+  .download = system_download,
+  .upload = system_upload,
+  .load = system_load_mono_22k05_16b,
+  .save = file_save,
+  .get_upload_path = common_system_get_upload_path,
+  .get_download_path = common_system_get_download_path,
+  .get_exts = sample_get_sample_extensions,
+  .max_name_len = 255
+};
+
+const struct fs_operations FS_SYSTEM_SAMPLES_MONO_16K_16B_OPERATIONS = {
+  .id = FS_SYSTEM_SAMPLES_MONO_16K_16B,
+  .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO |
+    FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SAMPLE_COLUMNS |
+    FS_OPTION_ALLOW_SEARCH,
+  .name = "wav-mono-16k-16b",
+  .gui_name = "WAV mono 16 KHz 16 bits",
+  .gui_icon = FS_ICON_WAVE,
+  .file_icon = FS_ICON_WAVE,
+  .readdir = system_samples_read_dir,
+  .print_item = common_print_item,
+  .file_exists = system_file_exists,
+  .mkdir = system_mkdir,
+  .delete = system_delete,
+  .rename = system_rename,
+  .move = system_rename,
+  .download = system_download,
+  .upload = system_upload,
+  .load = system_load_mono_16k_16b,
+  .save = file_save,
+  .get_upload_path = common_system_get_upload_path,
+  .get_download_path = common_system_get_download_path,
+  .get_exts = sample_get_sample_extensions,
+  .max_name_len = 255
+};
+
+const struct fs_operations FS_SYSTEM_SAMPLES_MONO_16K_8B_OPERATIONS = {
+  .id = FS_SYSTEM_SAMPLES_MONO_16K_8B,
+  .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO |
+    FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SAMPLE_COLUMNS |
+    FS_OPTION_ALLOW_SEARCH,
+  .name = "wav-mono-16k-8b",
+  .gui_name = "WAV mono 16 KHz 8 bits",
+  .gui_icon = FS_ICON_WAVE,
+  .file_icon = FS_ICON_WAVE,
+  .readdir = system_samples_read_dir,
+  .print_item = common_print_item,
+  .file_exists = system_file_exists,
+  .mkdir = system_mkdir,
+  .delete = system_delete,
+  .rename = system_rename,
+  .move = system_rename,
+  .download = system_download,
+  .upload = system_upload_8_bits,
+  .load = system_load_mono_16k_16b,
+  .save = file_save,
+  .get_upload_path = common_system_get_upload_path,
+  .get_download_path = common_system_get_download_path,
+  .get_exts = sample_get_sample_extensions,
+  .max_name_len = 255
+};
+
+const struct fs_operations FS_SYSTEM_SAMPLES_MONO_8K_16B_OPERATIONS = {
+  .id = FS_SYSTEM_SAMPLES_MONO_8K_16B,
+  .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO |
+    FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SAMPLE_COLUMNS |
+    FS_OPTION_ALLOW_SEARCH,
+  .name = "wav-mono-8k-16b",
+  .gui_name = "WAV mono 8 KHz 16 bits",
+  .gui_icon = FS_ICON_WAVE,
+  .file_icon = FS_ICON_WAVE,
+  .readdir = system_samples_read_dir,
+  .print_item = common_print_item,
+  .file_exists = system_file_exists,
+  .mkdir = system_mkdir,
+  .delete = system_delete,
+  .rename = system_rename,
+  .move = system_rename,
+  .download = system_download,
+  .upload = system_upload,
+  .load = system_load_mono_8k_16b,
+  .save = file_save,
+  .get_upload_path = common_system_get_upload_path,
+  .get_download_path = common_system_get_download_path,
+  .get_exts = sample_get_sample_extensions,
+  .max_name_len = 255
+};
+
+const struct fs_operations FS_SYSTEM_SAMPLES_MONO_8K_8B_OPERATIONS = {
+  .id = FS_SYSTEM_SAMPLES_MONO_8K_8B,
+  .options = FS_OPTION_SAMPLE_EDITOR | FS_OPTION_MONO |
+    FS_OPTION_SHOW_SIZE_COLUMN | FS_OPTION_SHOW_SAMPLE_COLUMNS |
+    FS_OPTION_ALLOW_SEARCH,
+  .name = "wav-mono-8k-8b",
+  .gui_name = "WAV mono 8 KHz 8 bits",
+  .gui_icon = FS_ICON_WAVE,
+  .file_icon = FS_ICON_WAVE,
+  .readdir = system_samples_read_dir,
+  .print_item = common_print_item,
+  .file_exists = system_file_exists,
+  .mkdir = system_mkdir,
+  .delete = system_delete,
+  .rename = system_rename,
+  .move = system_rename,
+  .download = system_download,
+  .upload = system_upload_8_bits,
+  .load = system_load_mono_8k_16b,
   .save = file_save,
   .get_upload_path = common_system_get_upload_path,
   .get_download_path = common_system_get_download_path,
@@ -658,15 +889,21 @@ system_get_storage_stats (struct backend *backend, guint8 type,
 static gint
 system_handshake (struct backend *backend)
 {
-  gslist_fill (&backend->fs_ops, &FS_SYSTEM_SAMPLES_48_16_STEREO_OPERATIONS,
-	       &FS_SYSTEM_SAMPLES_48_16_MONO_OPERATIONS,
-	       &FS_SYSTEM_SAMPLES_441_16_STEREO_OPERATIONS,
-	       &FS_SYSTEM_SAMPLES_441_16_MONO_OPERATIONS,
-	       &FS_SYSTEM_SAMPLES_441_24_STEREO_OPERATIONS,
-	       &FS_SYSTEM_SAMPLES_441_24_MONO_OPERATIONS,
-	       &FS_SYSTEM_SAMPLES_441_8_STEREO_OPERATIONS,
-	       &FS_SYSTEM_SAMPLES_441_8_MONO_OPERATIONS,
-	       &FS_SYSTEM_SAMPLES_32_16_MONO_OPERATIONS, NULL);
+  gslist_fill (&backend->fs_ops, &FS_SYSTEM_SAMPLES_STEREO_48K_16B_OPERATIONS,
+	       &FS_SYSTEM_SAMPLES_MONO_48K_16B_OPERATIONS,
+	       &FS_SYSTEM_SAMPLES_STEREO_44K1_16B_OPERATIONS,
+	       &FS_SYSTEM_SAMPLES_MONO_44K1_16B_OPERATIONS,
+	       &FS_SYSTEM_SAMPLES_STEREO_44K1_24B_OPERATIONS,
+	       &FS_SYSTEM_SAMPLES_MONO_44K1_24B_OPERATIONS,
+	       &FS_SYSTEM_SAMPLES_STEREO_441K_8B_OPERATIONS,
+	       &FS_SYSTEM_SAMPLES_MONO_44K1_8B_OPERATIONS,
+	       &FS_SYSTEM_SAMPLES_MONO_32K_16B_OPERATIONS,
+	       &FS_SYSTEM_SAMPLES_MONO_31K25_16B_OPERATIONS,
+	       &FS_SYSTEM_SAMPLES_MONO_22K05_16B_OPERATIONS,
+	       &FS_SYSTEM_SAMPLES_MONO_16K_16B_OPERATIONS,
+	       &FS_SYSTEM_SAMPLES_MONO_16K_8B_OPERATIONS,
+	       &FS_SYSTEM_SAMPLES_MONO_8K_16B_OPERATIONS,
+	       &FS_SYSTEM_SAMPLES_MONO_8K_8B_OPERATIONS, NULL);
   snprintf (backend->name, LABEL_MAX, "%s", _("System"));
 
   backend->get_storage_stats =
@@ -682,6 +919,6 @@ system_handshake (struct backend *backend)
 const struct connector CONNECTOR_SYSTEM = {
   .handshake = system_handshake,
   .name = "system",
-  .standard = FALSE,
+  .options = CONNECTOR_OPTION_CUSTOM_HANDSHAKE,
   .regex = NULL
 };
