@@ -18,6 +18,7 @@
  *   along with Elektroid. If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include "audio.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -32,8 +33,11 @@
 #include "backend.h"
 #include "regconn.h"
 #include "regpref.h"
+#include "sample.h"
 #include "utils.h"
 
+#define CLI_SLEEP_US 200000
+
 #define COMMAND_NOT_IN_SYSTEM_FS "Command not available in system backend"
 
 #define GET_FS_OPS_OFFSET(member) offsetof(struct fs_operations, member)
@@ -41,8 +45,8 @@
 #define RETURN_IF_NULL(f) if (!(f)) {return -ENOSYS;}
 
 static struct backend backend;
-static struct job_control control;
-static struct sysex_transfer sysex_transfer;
+static struct task_control task_control;
+static struct controllable controllable;	//Used for CLI control for operations that do not use task_control or sysex_transfer.
 static gchar *connector, *fs, *op;
 
 const struct fs_operations *fs_ops;
@@ -50,18 +54,18 @@ const gchar *current_path_progress;
 gboolean same_line_progress;
 
 static void
-complete_progress ()
+complete_progress (gint err)
 {
-  if (same_line_progress)
+  if (same_line_progress && !err)
     {
       fprintf (stderr, "\n");
     }
 }
 
 static void
-print_progress (struct job_control *control)
+print_progress (struct task_control *task_control)
 {
-  gint progress = control->progress * 100;
+  gint progress = task_control->progress * 100;
   const gchar *end = same_line_progress ? "\r" : "\n";
   fprintf (stderr, "%s: %3d %%%s", current_path_progress, progress, end);
   if (same_line_progress)
@@ -166,7 +170,6 @@ cli_list (int argc, gchar *argv[], int *optind)
   const gchar *path;
   struct item_iterator iter;
   const gchar *device_path;
-  gboolean active;
 
   if (*optind == argc)
     {
@@ -187,29 +190,25 @@ cli_list (int argc, gchar *argv[], int *optind)
 
   RETURN_IF_NULL (fs_ops->readdir);
   RETURN_IF_NULL (fs_ops->print_item);
-
-  active = TRUE;
-  sysex_transfer.active = TRUE;
+  RETURN_IF_NULL (fs_ops->get_exts);
 
   path = cli_get_path (device_path);
-  err = fs_ops->readdir (&backend, &iter, path, NULL);
+  err = fs_ops->readdir (&backend, &iter, path,
+			 fs_ops->get_exts (&backend, fs_ops));
   if (err)
     {
       return err;
     }
 
-  while (!item_iterator_next (&iter) && active)
+  while (!item_iterator_next (&iter) &&
+	 controllable_is_active (&controllable))
     {
       fs_ops->print_item (&iter, &backend, fs_ops);
-
-      g_mutex_lock (&sysex_transfer.mutex);
-      active = sysex_transfer.active;
-      g_mutex_unlock (&sysex_transfer.mutex);
     }
 
   item_iterator_free (&iter);
 
-  return EXIT_SUCCESS;
+  return controllable_is_active (&controllable) ? EXIT_SUCCESS : EXIT_FAILURE;
 }
 
 static gint
@@ -359,13 +358,37 @@ cli_command_mv_rename (int argc, gchar *argv[], int *optind)
   return f (&backend, src_path, dst_path);
 }
 
+static const gchar *
+cli_get_backend_type_name ()
+{
+  switch (backend.type)
+    {
+    case BE_TYPE_SYSTEM:
+      return "SYSTEM";
+    case BE_TYPE_MIDI:
+      return "MIDI";
+    case BE_TYPE_NO_MIDI:
+      return "NO-MIDI";
+    default:
+      return "UNKNOWN";
+    }
+}
+
+static gint
+cli_fs_compare (gconstpointer a, gconstpointer b)
+{
+  const struct fs_operations *fsa = a;
+  const struct fs_operations *fsb = b;
+  return strcmp (fsa->name, fsb->name);
+}
+
 static gint
 cli_info (int argc, gchar *argv[], int *optind)
 {
   const gchar *device_path;
   gint err;
   gboolean first;
-  GSList *e;
+  GSList *e, *sorted;
 
   if (*optind == argc)
     {
@@ -384,27 +407,31 @@ cli_info (int argc, gchar *argv[], int *optind)
       return err;
     }
 
-  printf ("Type: %s\n", backend.type == BE_TYPE_SYSTEM ? "SYSTEM" : "MIDI");
+  printf ("Type: %s\n", cli_get_backend_type_name ());
   printf ("Device name: %s\n", backend.name);
   printf ("Device version: %s\n", backend.version);
   printf ("Device description: %s\n", backend.description);
   printf ("Connector name: %s\n", backend.conn_name);
   printf ("Filesystems: ");
 
-  e = backend.fs_ops;
+  sorted = g_slist_copy (backend.fs_ops);
+  sorted = g_slist_sort (sorted, cli_fs_compare);
+
+  e = sorted;
   first = TRUE;
   while (e)
     {
       const struct fs_operations *fs_ops = e->data;
-      const gchar *name = GET_CLI_NAME (fs_ops);
       gboolean cli_only = fs_ops->gui_name == NULL;
-      printf ("%s%s%s", first ? "" : ", ", name,
+      printf ("%s%s%s", first ? "" : ", ", fs_ops->name,
 	      cli_only ? " (CLI only)" : "");
       first = FALSE;
       e = e->next;
     }
   printf ("\n");
 
+  g_slist_free (g_steal_pointer (&sorted));
+
   return EXIT_SUCCESS;
 }
 
@@ -483,6 +510,7 @@ cli_upgrade_os (int argc, gchar *argv[], int *optind)
   const gchar *src_path;
   const gchar *device_path;
   struct idata idata;
+  struct sysex_transfer sysex_transfer;
 
   if (*optind == argc)
     {
@@ -526,12 +554,9 @@ cli_upgrade_os (int argc, gchar *argv[], int *optind)
     }
   else
     {
-      sysex_transfer.raw = idata.content;
-      sysex_transfer.active = TRUE;
-      sysex_transfer.timeout = BE_SYSEX_TIMEOUT_MS;
-
-      err = backend.upgrade_os (&backend, &sysex_transfer);
-      idata_free (&idata);
+      sysex_transfer_init_tx (&sysex_transfer, idata_steal (&idata));
+      err = backend.upgrade_os (&backend, &sysex_transfer, &controllable);
+      sysex_transfer_clear (&sysex_transfer);
     }
 
   return err;
@@ -548,11 +573,11 @@ cli_download_item (const gchar *src_path, const gchar *dst_path)
   RETURN_IF_NULL (fs_ops->get_download_path);
   RETURN_IF_NULL (fs_ops->save);
 
-  control.active = TRUE;
-  control.callback = print_progress;
+  controllable_set_active (&task_control.controllable, TRUE);
+  task_control.callback = print_progress;
   current_path_progress = src_path;
 
-  err = fs_ops->download (&backend, src_path, &idata, &control);
+  err = fs_ops->download (&backend, src_path, &idata, &task_control);
   if (err)
     {
       return err;
@@ -566,13 +591,13 @@ cli_download_item (const gchar *src_path, const gchar *dst_path)
       goto cleanup;
     }
 
-  err = fs_ops->save (download_path, &idata, &control);
+  err = fs_ops->save (download_path, &idata, &task_control);
   g_free (download_path);
 
 cleanup:
-  idata_free (&idata);
+  idata_clear (&idata);
 
-  complete_progress ();
+  complete_progress (err);
 
   return err;
 }
@@ -581,21 +606,18 @@ static gint
 cli_download_dir (const gchar *src_path, const gchar *dst_path)
 {
   gint err;
-  gboolean active;
   struct item_iterator iter;
 
   RETURN_IF_NULL (fs_ops->readdir);
 
-  active = TRUE;
-  sysex_transfer.active = TRUE;
-
   err = fs_ops->readdir (&backend, &iter, src_path, NULL);
   if (err)
     {
       return err;
     }
 
-  while (!item_iterator_next (&iter) && active && !err)
+  while (!item_iterator_next (&iter) && controllable_is_active (&controllable)
+	 && !err)
     {
       gchar *rsrc_path;
       gchar *filename = item_get_filename (&iter.item, fs_ops->options);
@@ -627,15 +649,11 @@ cli_download_dir (const gchar *src_path, const gchar *dst_path)
 	}
 
       g_free (rsrc_path);
-
-      g_mutex_lock (&sysex_transfer.mutex);
-      active = sysex_transfer.active;
-      g_mutex_unlock (&sysex_transfer.mutex);
     }
 
   item_iterator_free (&iter);
 
-  return active ? err : -ECANCELED;
+  return controllable_is_active (&controllable) ? err : -ECANCELED;
 }
 
 static gint
@@ -726,11 +744,11 @@ cli_upload_item (const gchar *src_path, const gchar *dst_path)
   RETURN_IF_NULL (fs_ops->get_upload_path);
   RETURN_IF_NULL (fs_ops->upload);
 
-  control.active = TRUE;
-  control.callback = print_progress;
+  controllable_set_active (&task_control.controllable, TRUE);
+  task_control.callback = print_progress;
   current_path_progress = src_path;
 
-  err = fs_ops->load (src_path, &idata, &control);
+  err = fs_ops->load (&backend, src_path, &idata, &task_control);
   if (err)
     {
       return err;
@@ -739,12 +757,12 @@ cli_upload_item (const gchar *src_path, const gchar *dst_path)
   upload_path = fs_ops->get_upload_path (&backend, fs_ops, dst_path,
 					 src_path, &idata);
 
-  err = fs_ops->upload (&backend, upload_path, &idata, &control);
-  idata_free (&idata);
+  err = fs_ops->upload (&backend, upload_path, &idata, &task_control);
+  idata_clear (&idata);
 
   g_free (upload_path);
 
-  complete_progress ();
+  complete_progress (err);
 
   return err;
 }
@@ -795,6 +813,7 @@ cli_send (int argc, gchar *argv[], int *optind)
   gint err;
   const gchar *device_dst_path, *src_file;
   struct idata idata;
+  struct sysex_transfer sysex_transfer;
 
   if (*optind == argc)
     {
@@ -831,13 +850,15 @@ cli_send (int argc, gchar *argv[], int *optind)
     }
 
   err = file_load (src_file, &idata, NULL);
-  if (!err)
+  if (err)
     {
-      sysex_transfer.active = TRUE;
-      sysex_transfer.timeout = BE_SYSEX_TIMEOUT_MS;
-      sysex_transfer.raw = idata.content;
-      err = backend_tx_sysex (&backend, &sysex_transfer);
-      idata_free (&idata);
+      error_print ("Error while loading '%s'.", src_file);
+    }
+  else
+    {
+      sysex_transfer_init_tx (&sysex_transfer, idata_steal (&idata));
+      err = backend_tx_sysex (&backend, &sysex_transfer, &controllable);
+      sysex_transfer_clear (&sysex_transfer);
     }
 
   return err;
@@ -848,6 +869,8 @@ cli_receive (int argc, gchar *argv[], int *optind)
 {
   gint err;
   const gchar *device_src_path, *dst_file;
+  struct idata idata;
+  struct sysex_transfer sysex_transfer;
 
   if (*optind == argc)
     {
@@ -883,51 +906,101 @@ cli_receive (int argc, gchar *argv[], int *optind)
       return EXIT_FAILURE;
     }
 
-  sysex_transfer.timeout = BE_SYSEX_TIMEOUT_MS;
-  sysex_transfer.batch = TRUE;
-
   backend_rx_drain (&backend);
+
   //This doesn't need to be synchronized because the CLI is not multithreaded.
-  err = backend_rx_sysex (&backend, &sysex_transfer);
-  if (!err)
+  sysex_transfer_init_rx (&sysex_transfer, BE_SYSEX_TIMEOUT_MS, TRUE);
+  err = backend_rx_sysex (&backend, &sysex_transfer, &controllable);
+  if (err)
     {
-      struct idata idata;
-      idata.content = sysex_transfer.raw;
+      error_print ("Error while downloading.");
+    }
+  else
+    {
+      idata_init (&idata, sysex_transfer_steal (&sysex_transfer), NULL, NULL,
+		  NULL);
       err = file_save (dst_file, &idata, NULL);
+      idata_clear (&idata);
     }
 
-  free_msg (sysex_transfer.raw);
   return err;
 }
 
 static gint
-set_conn_fs_op_from_command (const gchar *cmd)
+cli_play (int argc, gchar *argv[], int *optind)
 {
-  gchar *aux;
+  gint err;
+  struct idata sample;
+  const gchar *audio_file;
+  struct sample_info sample_info_src;
+  struct sample_load_opts sample_load_opts;
 
-  connector = strdup (cmd);
-  aux = strchr (connector, '-');
-  if (!aux)
+  if (*optind == argc)
     {
-      g_free (connector);
+      error_print ("Source file missing");
       return -EINVAL;
     }
-  *aux = 0;
-  aux++;
-
-  fs = strdup (aux);
-  aux = strchr (fs, '-');
-  if (!aux)
+  else
     {
-      g_free (connector);
-      g_free (fs);
-      return -EINVAL;
+      audio_file = argv[*optind];
+      (*optind)++;
     }
 
-  *aux = 0;
-  aux++;
+  controllable_set_active (&task_control.controllable, TRUE);
+  task_control.callback = print_progress;
+  current_path_progress = audio_file;
 
-  op = strdup (aux);
+  task_control_reset (&task_control, 1);
+
+  sample_load_opts_init (&sample_load_opts, 2, audio.rate,
+			 sample_get_internal_format (), FALSE);
+
+  err = sample_load_from_file (audio_file, &sample, NULL, &sample_load_opts,
+			       &sample_info_src);
+  if (err)
+    {
+      error_print ("Error while loading '%s'.", audio_file);
+    }
+  else
+    {
+      audio_set_play_and_wait (&sample, &task_control);
+      task_control.part++;
+    }
+
+  complete_progress (err);
+
+  return err;
+}
+
+static gint
+cli_record (int argc, gchar *argv[], int *optind)
+{
+  const gchar *audio_file;
+
+  if (*optind == argc)
+    {
+      error_print ("Source file missing");
+      return -EINVAL;
+    }
+  else
+    {
+      audio_file = argv[*optind];
+      (*optind)++;
+    }
+
+  controllable_set_active (&task_control.controllable, TRUE);
+  task_control.callback = print_progress;
+  current_path_progress = audio_file;
+
+  task_control_reset (&task_control, 1);
+
+  audio_record_and_wait (RECORD_STEREO, &task_control);
+
+  task_control.part++;
+  complete_progress (0);
+
+  sample_save_to_file (audio_file, &audio.sample, NULL,
+		       SF_FORMAT_WAV | sample_get_internal_format ());
 
   return 0;
 }
@@ -936,11 +1009,10 @@ set_conn_fs_op_from_command (const gchar *cmd)
 static void
 cli_end (int sig)
 {
-  job_control_set_active_lock (&control, FALSE);
-
-  g_mutex_lock (&sysex_transfer.mutex);
-  sysex_transfer.active = FALSE;
-  g_mutex_unlock (&sysex_transfer.mutex);
+  controllable_set_active (&controllable, FALSE);
+  controllable_set_active (&task_control.controllable, FALSE);
+  audio_stop_playback ();
+  audio_stop_recording ();
 }
 #endif
 
@@ -952,6 +1024,9 @@ main (int argc, gchar *argv[])
   gchar *command;
   gint vflg = 0, errflg = 0;
 
+  controllable_init (&controllable);
+  controllable_init (&task_control.controllable);
+
 #if defined(__linux__)
   struct sigaction action;
 
@@ -1004,6 +1079,9 @@ main (int argc, gchar *argv[])
   regconn_register ();
   regpref_register ();
   preferences_load ();
+  preferences_set_boolean (PREF_KEY_MIX, FALSE);	//This might be required by devices using the audio link.
+
+  audio_init_and_wait ();
 
   if (!strcmp (command, "ld") || !strcmp (command, "list-devices"))
     {
@@ -1029,9 +1107,17 @@ main (int argc, gchar *argv[])
     {
       err = cli_upgrade_os (argc, argv, &optind);
     }
+  else if (!strcmp (command, "play"))
+    {
+      err = cli_play (argc, argv, &optind);
+    }
+  else if (!strcmp (command, "record"))
+    {
+      err = cli_record (argc, argv, &optind);
+    }
   else
     {
-      err = set_conn_fs_op_from_command (command);
+      err = command_set_parts (command, &connector, &fs, &op);
       if (err)
 	{
 	  goto end;
@@ -1071,7 +1157,7 @@ main (int argc, gchar *argv[])
       else if (!strcmp (op, "cl"))
 	{
 	  err = cli_command_path (argc, argv, &optind,
-				  GET_FS_OPS_OFFSET (clear));
+				  GET_FS_OPS_OFFSET (delete));
 	}
       else if (!strcmp (op, "cp"))
 	{
@@ -1109,6 +1195,10 @@ end:
       error_print ("Error: %s", g_strerror (-err));
     }
 
+  controllable_clear (&controllable);
+
+  audio_destroy ();
+
   regconn_unregister ();
   regpref_unregister ();
 
