@@ -26,15 +26,19 @@ static gboolean
 guirecorder_set_monitor_level (gpointer data)
 {
   struct guirecorder *guirecorder = data;
-  gtk_level_bar_set_value (guirecorder->monitor_levelbar, guirecorder->level);
+  gtk_level_bar_set_value (guirecorder->monitor_levelbar_l,
+			   guirecorder->level_l);
+  gtk_level_bar_set_value (guirecorder->monitor_levelbar_r,
+			   guirecorder->level_r);
   return FALSE;
 }
 
 void
-guirecorder_monitor_notifier (gpointer recorder, gdouble value)
+guirecorder_monitor_notifier (gpointer recorder, gdouble l, gdouble r)
 {
   struct guirecorder *guirecorder = recorder;
-  guirecorder->level = value;
+  guirecorder->level_l = l;
+  guirecorder->level_r = r;
   g_idle_add (guirecorder_set_monitor_level, guirecorder);
 }
 
@@ -64,8 +68,7 @@ guirecorder_set_channels_masks (struct guirecorder *guirecorder, guint32 opts)
       gtk_list_store_insert_with_values (guirecorder->channels_list_store,
 					 NULL, -1,
 					 CHANNELS_LIST_STORE_CAPTION_FIELD,
-/* TRANSLATORS: Stereo recording */
-					 _("Stereo"),
+					 C_ ("Recording channels", "Stereo"),
 					 CHANNELS_LIST_STORE_ID_FIELD,
 					 RECORD_STEREO, -1);
     }
@@ -74,15 +77,14 @@ guirecorder_set_channels_masks (struct guirecorder *guirecorder, guint32 opts)
       gtk_list_store_insert_with_values (guirecorder->channels_list_store,
 					 NULL, -1,
 					 CHANNELS_LIST_STORE_CAPTION_FIELD,
-/* TRANSLATORS: Mono recording from left channel */
-					 _("Left"),
+					 C_ ("Recording channels", "Left"),
 					 CHANNELS_LIST_STORE_ID_FIELD,
 					 RECORD_LEFT, -1);
+
       gtk_list_store_insert_with_values (guirecorder->channels_list_store,
 					 NULL, -1,
 					 CHANNELS_LIST_STORE_CAPTION_FIELD,
-/* TRANSLATORS: Mono recording from right channel */
-					 _("Right"),
+					 C_ ("Recording channels", "Right"),
 					 CHANNELS_LIST_STORE_ID_FIELD,
 					 RECORD_RIGHT, -1);
     }
@@ -94,10 +96,9 @@ void
 guirecorder_channels_changed (GtkWidget *object, gpointer data)
 {
   struct guirecorder *guirecorder = data;
-  struct audio *audio = guirecorder->audio;
   guint options = guirecorder_get_channel_mask (guirecorder) |
     RECORD_MONITOR_ONLY;
-  g_mutex_lock (&audio->control.mutex);
-  audio->record_options = options;
-  g_mutex_unlock (&audio->control.mutex);
+  g_mutex_lock (&audio.control.controllable.mutex);
+  audio.record_options = options;
+  g_mutex_unlock (&audio.control.controllable.mutex);
 }
