@@ -20,11 +20,16 @@
 
 #include <glib/gi18n.h>
 #include "maction.h"
+#include "browser.h"
 #include "connectors/microbrute.h"
+#include "elektroid.h"
 #include "utils.h"
 
+extern GtkWindow *main_window;
+extern struct browser remote_browser;
+
 static guint8 channel;
-static GtkWidget *config_window = NULL;
+static GtkWidget *config_window;
 static GtkWidget *calibration_assistant;
 static GtkWidget *note_priority;
 static GtkWidget *vel_response;
@@ -43,108 +48,83 @@ static GtkWidget *step_length;
 static GtkWidget *persistent_changes;
 static gboolean loading;
 
-static void
-microbrute_set_combo_value (GtkWidget *combo, guint8 value)
-{
-  guint v;
-  gint index = 0;
-  GtkTreeIter iter;
-  gboolean found = FALSE;
-  GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
-  gboolean valid = gtk_tree_model_get_iter_first (model, &iter);
-
-  while (valid)
-    {
-      gtk_tree_model_get (model, &iter, 1, &v, -1);
-      if (v == value)
-	{
-	  found = TRUE;
-	  break;
-
-	}
-      valid = gtk_tree_model_iter_next (model, &iter);
-      index += 1;
-    }
-  if (found)
-    {
-      gtk_combo_box_set_active (GTK_COMBO_BOX (combo), index);
-    }
-}
-
 static void
 microbrute_configure_callback (GtkWidget *object, gpointer data)
 {
   guint8 v;
-  struct maction_context *context = data;
-  struct backend *backend = context->backend;
 
-  debug_print (2, "Configuring %s...", backend->name);
   loading = TRUE;
-  microbrute_get_parameter (backend, MICROBRUTE_NOTE_PRIORITY, &v);
-  microbrute_set_combo_value (note_priority, v);
-  microbrute_get_parameter (backend, MICROBRUTE_VEL_RESPONSE, &v);
-  microbrute_set_combo_value (vel_response, v);
+  microbrute_get_parameter (remote_browser.backend, MICROBRUTE_NOTE_PRIORITY,
+			    &v);
+  elektroid_combo_box_set_value (GTK_COMBO_BOX (note_priority), v);
+  microbrute_get_parameter (remote_browser.backend, MICROBRUTE_VEL_RESPONSE,
+			    &v);
+  elektroid_combo_box_set_value (GTK_COMBO_BOX (vel_response), v);
 
-  microbrute_get_parameter (backend, MICROBRUTE_LFO_KEY_RETRIGGER, &v);
+  microbrute_get_parameter (remote_browser.backend,
+			    MICROBRUTE_LFO_KEY_RETRIGGER, &v);
   gtk_switch_set_state (GTK_SWITCH (lfo_key_retrigger), v);
   gtk_switch_set_active (GTK_SWITCH (lfo_key_retrigger), v);
-  microbrute_get_parameter (backend, MICROBRUTE_ENVELOPE_LEGATO, &v);
+  microbrute_get_parameter (remote_browser.backend,
+			    MICROBRUTE_ENVELOPE_LEGATO, &v);
   gtk_switch_set_state (GTK_SWITCH (envelope_legato), v);
   gtk_switch_set_active (GTK_SWITCH (envelope_legato), v);
-  microbrute_get_parameter (backend, MICROBRUTE_BEND_RANGE, &v);
+  microbrute_get_parameter (remote_browser.backend, MICROBRUTE_BEND_RANGE,
+			    &v);
   gtk_spin_button_set_value (GTK_SPIN_BUTTON (bend_range), v);
-  microbrute_get_parameter (backend, MICROBRUTE_STEP_LENGTH, &v);
-  microbrute_set_combo_value (step_length, v);
-  microbrute_get_parameter (backend, MICROBRUTE_GATE_LENGTH, &v);
-  microbrute_set_combo_value (gate_length, v);
+  microbrute_get_parameter (remote_browser.backend, MICROBRUTE_STEP_LENGTH,
+			    &v);
+  elektroid_combo_box_set_value (GTK_COMBO_BOX (step_length), v);
+  microbrute_get_parameter (remote_browser.backend, MICROBRUTE_GATE_LENGTH,
+			    &v);
+  elektroid_combo_box_set_value (GTK_COMBO_BOX (gate_length), v);
 
-  microbrute_get_parameter (backend, MICROBRUTE_TX_CHANNEL, &v);
-  microbrute_set_combo_value (tx_channel, v);
-  microbrute_get_parameter (backend, MICROBRUTE_RX_CHANNEL, &channel);
-  microbrute_set_combo_value (rx_channel, channel);
+  microbrute_get_parameter (remote_browser.backend, MICROBRUTE_TX_CHANNEL,
+			    &v);
+  elektroid_combo_box_set_value (GTK_COMBO_BOX (tx_channel), v);
+  microbrute_get_parameter (remote_browser.backend, MICROBRUTE_RX_CHANNEL,
+			    &channel);
+  elektroid_combo_box_set_value (GTK_COMBO_BOX (rx_channel), channel);
 
-  microbrute_get_parameter (backend, MICROBRUTE_PLAY_ON, &v);
-  microbrute_set_combo_value (play, v);
-  microbrute_get_parameter (backend, MICROBRUTE_RETRIGGERING, &v);
-  microbrute_set_combo_value (retriggering, v);
-  microbrute_get_parameter (backend, MICROBRUTE_NEXT_SEQUENCE, &v);
-  microbrute_set_combo_value (next_sequence, v);
-  microbrute_get_parameter (backend, MICROBRUTE_STEP_ON, &v);
-  microbrute_set_combo_value (step_on, v);
-  microbrute_get_parameter (backend, MICROBRUTE_SYNC, &v);
-  microbrute_set_combo_value (synchronization, v);
+  microbrute_get_parameter (remote_browser.backend, MICROBRUTE_PLAY_ON, &v);
+  elektroid_combo_box_set_value (GTK_COMBO_BOX (play), v);
+  microbrute_get_parameter (remote_browser.backend, MICROBRUTE_RETRIGGERING,
+			    &v);
+  elektroid_combo_box_set_value (GTK_COMBO_BOX (retriggering), v);
+  microbrute_get_parameter (remote_browser.backend, MICROBRUTE_NEXT_SEQUENCE,
+			    &v);
+  elektroid_combo_box_set_value (GTK_COMBO_BOX (next_sequence), v);
+  microbrute_get_parameter (remote_browser.backend, MICROBRUTE_STEP_ON, &v);
+  elektroid_combo_box_set_value (GTK_COMBO_BOX (step_on), v);
+  microbrute_get_parameter (remote_browser.backend, MICROBRUTE_SYNC, &v);
+  elektroid_combo_box_set_value (GTK_COMBO_BOX (synchronization), v);
   loading = FALSE;
 
   gtk_widget_show (config_window);
 }
 
 static void
-microbrute_combo_changed (GtkComboBox *combo, struct backend *backend,
-			  guint8 param)
+microbrute_combo_changed (GtkComboBox *combo, guint8 param)
 {
-  guint value;
-  GtkTreeIter iter;
   gboolean sysex;
-  GtkTreeModel *model;
   if (!loading)
     {
-      model = gtk_combo_box_get_model (combo);
       sysex = gtk_switch_get_active (GTK_SWITCH (persistent_changes));
-      gtk_combo_box_get_active_iter (combo, &iter);
-      gtk_tree_model_get (model, &iter, 1, &value, -1);
-      microbrute_set_parameter (backend, param, value, channel, sysex);
+      guint value = elektroid_combo_box_get_value (combo);
+      microbrute_set_parameter (remote_browser.backend, param, value, channel,
+				sysex);
     }
 }
 
 static gboolean
-microbrute_switch_state_set (struct backend *backend, guint8 param,
-			     guint8 state)
+microbrute_switch_state_set (guint8 param, guint8 state)
 {
   gboolean sysex;
   if (!loading)
     {
       sysex = gtk_switch_get_active (GTK_SWITCH (persistent_changes));
-      microbrute_set_parameter (backend, param, state, channel, sysex);
+      microbrute_set_parameter (remote_browser.backend, param, state, channel,
+				sysex);
     }
   return FALSE;
 }
@@ -152,29 +132,27 @@ microbrute_switch_state_set (struct backend *backend, guint8 param,
 static void
 microbrute_note_priority_changed (GtkComboBox *combo, gpointer data)
 {
-  microbrute_combo_changed (combo, data, MICROBRUTE_NOTE_PRIORITY);
+  microbrute_combo_changed (combo, MICROBRUTE_NOTE_PRIORITY);
 }
 
 static void
 microbrute_vel_response_changed (GtkComboBox *combo, gpointer data)
 {
-  microbrute_combo_changed (combo, data, MICROBRUTE_VEL_RESPONSE);
+  microbrute_combo_changed (combo, MICROBRUTE_VEL_RESPONSE);
 }
 
 static gboolean
 microbrute_lfo_key_retrigger_state_set (GtkSwitch *s, gboolean state,
 					gpointer data)
 {
-  return microbrute_switch_state_set (data, MICROBRUTE_LFO_KEY_RETRIGGER,
-				      state);
+  return microbrute_switch_state_set (MICROBRUTE_LFO_KEY_RETRIGGER, state);
 }
 
 static gboolean
 microbrute_envelope_legato_state_set (GtkSwitch *s, gboolean state,
 				      gpointer data)
 {
-  return microbrute_switch_state_set (data, MICROBRUTE_ENVELOPE_LEGATO,
-				      state);
+  return microbrute_switch_state_set (MICROBRUTE_ENVELOPE_LEGATO, state);
 }
 
 static void
@@ -186,63 +164,63 @@ microbrute_bend_range_value_changed (GtkSpinButton *spin, gpointer data)
     {
       sysex = gtk_switch_get_active (GTK_SWITCH (persistent_changes));
       value = gtk_spin_button_get_value (spin);
-      microbrute_set_parameter (data, MICROBRUTE_BEND_RANGE, value, channel,
-				sysex);
+      microbrute_set_parameter (remote_browser.backend, MICROBRUTE_BEND_RANGE,
+				value, channel, sysex);
     }
 }
 
 static void
 microbrute_gate_length_changed (GtkComboBox *combo, gpointer data)
 {
-  microbrute_combo_changed (combo, data, MICROBRUTE_GATE_LENGTH);
+  microbrute_combo_changed (combo, MICROBRUTE_GATE_LENGTH);
 }
 
 static void
 microbrute_synchronization_changed (GtkComboBox *combo, gpointer data)
 {
-  microbrute_combo_changed (combo, data, MICROBRUTE_SYNC);
+  microbrute_combo_changed (combo, MICROBRUTE_SYNC);
 }
 
 static void
 microbrute_tx_channel_changed (GtkComboBox *combo, gpointer data)
 {
-  microbrute_combo_changed (combo, data, MICROBRUTE_TX_CHANNEL);
+  microbrute_combo_changed (combo, MICROBRUTE_TX_CHANNEL);
 }
 
 static void
 microbrute_rx_channel_changed (GtkComboBox *combo, gpointer data)
 {
-  microbrute_combo_changed (combo, data, MICROBRUTE_RX_CHANNEL);
+  microbrute_combo_changed (combo, MICROBRUTE_RX_CHANNEL);
 }
 
 static void
 microbrute_play_changed (GtkComboBox *combo, gpointer data)
 {
-  microbrute_combo_changed (combo, data, MICROBRUTE_PLAY_ON);
+  microbrute_combo_changed (combo, MICROBRUTE_PLAY_ON);
 }
 
 static void
 microbrute_retriggering_changed (GtkComboBox *combo, gpointer data)
 {
-  microbrute_combo_changed (combo, data, MICROBRUTE_RETRIGGERING);
+  microbrute_combo_changed (combo, MICROBRUTE_RETRIGGERING);
 }
 
 static void
 microbrute_next_sequence_changed (GtkComboBox *combo, gpointer data)
 {
-  microbrute_combo_changed (combo, data, MICROBRUTE_NEXT_SEQUENCE);
+  microbrute_combo_changed (combo, MICROBRUTE_NEXT_SEQUENCE);
 }
 
 static void
 microbrute_step_on_changed (GtkComboBox *combo, gpointer data)
 {
-  microbrute_combo_changed (combo, data, MICROBRUTE_STEP_ON);
+  microbrute_combo_changed (combo, MICROBRUTE_STEP_ON);
 }
 
 static void
 microbrute_step_length_changed (GtkComboBox *combo, gpointer data)
 {
-  microbrute_combo_changed (combo, data, MICROBRUTE_STEP_LENGTH);
+  microbrute_combo_changed (combo, MICROBRUTE_STEP_LENGTH);
 }
 
 static void
@@ -255,42 +233,38 @@ static void
 microbrute_assistant_prepare (GtkAssistant *assistant, GtkWidget *page,
 			      gpointer data)
 {
-  struct backend *backend = data;
   gint npage = gtk_assistant_get_current_page (assistant);
   switch (npage)
     {
     case 2:
-      microbrute_set_parameter (backend, MICROBRUTE_CALIB_PB_CENTER, 0,
-				channel, TRUE);
+      microbrute_set_parameter (remote_browser.backend,
+				MICROBRUTE_CALIB_PB_CENTER, 0, channel, TRUE);
       break;
     case 3:
-      microbrute_set_parameter (backend, MICROBRUTE_CALIB_BOTH_BOTTOM, 0,
-				channel, TRUE);
+      microbrute_set_parameter (remote_browser.backend,
+				MICROBRUTE_CALIB_BOTH_BOTTOM, 0, channel,
+				TRUE);
       break;
     case 4:
-      microbrute_set_parameter (backend, MICROBRUTE_CALIB_BOTH_TOP, 0,
-				channel, TRUE);
+      microbrute_set_parameter (remote_browser.backend,
+				MICROBRUTE_CALIB_BOTH_TOP, 0, channel, TRUE);
       sleep (1);
-      microbrute_set_parameter (backend, MICROBRUTE_CALIB_END, 0, channel,
-				TRUE);
+      microbrute_set_parameter (remote_browser.backend, MICROBRUTE_CALIB_END,
+				0, channel, TRUE);
       break;
     }
 }
 
-static void
-microbrute_configure_gui (struct backend *backend, GtkWindow *parent)
+void
+microbrute_init ()
 {
-  if (config_window)
-    {
-      return;
-    }
-
   GtkBuilder *builder = gtk_builder_new ();
-  gtk_builder_add_from_file (builder, DATADIR "/microbrute/gui.glade", NULL);
+  gtk_builder_add_from_file (builder, DATADIR "/microbrute/microbrute.ui",
+			     NULL);
   config_window = GTK_WIDGET (gtk_builder_get_object (builder,
 						      "config_window"));
   gtk_window_resize (GTK_WINDOW (config_window), 1, 1);
-  gtk_window_set_transient_for (GTK_WINDOW (config_window), parent);
+  gtk_window_set_transient_for (GTK_WINDOW (config_window), main_window);
 
   note_priority =
     GTK_WIDGET (gtk_builder_get_object (builder, "note_priority"));
@@ -321,39 +295,37 @@ microbrute_configure_gui (struct backend *backend, GtkWindow *parent)
     GTK_WIDGET (gtk_builder_get_object (builder, "persistent_changes"));
 
   g_signal_connect (note_priority, "changed",
-		    G_CALLBACK (microbrute_note_priority_changed), backend);
-  g_signal_connect (note_priority, "changed",
-		    G_CALLBACK (microbrute_vel_response_changed), backend);
+		    G_CALLBACK (microbrute_note_priority_changed), NULL);
+  g_signal_connect (vel_response, "changed",
+		    G_CALLBACK (microbrute_vel_response_changed), NULL);
 
   g_signal_connect (lfo_key_retrigger, "state-set",
 		    G_CALLBACK (microbrute_lfo_key_retrigger_state_set),
-		    backend);
+		    NULL);
   g_signal_connect (envelope_legato, "state-set",
-		    G_CALLBACK (microbrute_envelope_legato_state_set),
-		    backend);
+		    G_CALLBACK (microbrute_envelope_legato_state_set), NULL);
   g_signal_connect (bend_range, "value-changed",
-		    G_CALLBACK (microbrute_bend_range_value_changed),
-		    backend);
+		    G_CALLBACK (microbrute_bend_range_value_changed), NULL);
   g_signal_connect (gate_length, "changed",
-		    G_CALLBACK (microbrute_gate_length_changed), backend);
+		    G_CALLBACK (microbrute_gate_length_changed), NULL);
   g_signal_connect (synchronization, "changed",
-		    G_CALLBACK (microbrute_synchronization_changed), backend);
+		    G_CALLBACK (microbrute_synchronization_changed), NULL);
 
   g_signal_connect (tx_channel, "changed",
-		    G_CALLBACK (microbrute_tx_channel_changed), backend);
+		    G_CALLBACK (microbrute_tx_channel_changed), NULL);
   g_signal_connect (rx_channel, "changed",
-		    G_CALLBACK (microbrute_rx_channel_changed), backend);
+		    G_CALLBACK (microbrute_rx_channel_changed), NULL);
 
   g_signal_connect (play, "changed",
-		    G_CALLBACK (microbrute_play_changed), backend);
+		    G_CALLBACK (microbrute_play_changed), NULL);
   g_signal_connect (retriggering, "changed",
-		    G_CALLBACK (microbrute_retriggering_changed), backend);
+		    G_CALLBACK (microbrute_retriggering_changed), NULL);
   g_signal_connect (next_sequence, "changed",
-		    G_CALLBACK (microbrute_next_sequence_changed), backend);
+		    G_CALLBACK (microbrute_next_sequence_changed), NULL);
   g_signal_connect (step_on, "changed",
-		    G_CALLBACK (microbrute_step_on_changed), backend);
+		    G_CALLBACK (microbrute_step_on_changed), NULL);
   g_signal_connect (step_length, "changed",
-		    G_CALLBACK (microbrute_step_length_changed), backend);
+		    G_CALLBACK (microbrute_step_length_changed), NULL);
 
   g_signal_connect (config_window, "delete-event",
 		    G_CALLBACK (gtk_widget_hide_on_delete), NULL);
@@ -362,33 +334,40 @@ microbrute_configure_gui (struct backend *backend, GtkWindow *parent)
 
   calibration_assistant = GTK_WIDGET (gtk_builder_get_object (builder,
 							      "calibration_assistant"));
-  gtk_window_set_transient_for (GTK_WINDOW (calibration_assistant), parent);
+  gtk_window_set_transient_for (GTK_WINDOW (calibration_assistant),
+				main_window);
 
   g_signal_connect (calibration_assistant, "close",
-		    G_CALLBACK (microbrute_assistant_close), backend);
+		    G_CALLBACK (microbrute_assistant_close), NULL);
   g_signal_connect (calibration_assistant, "cancel",
-		    G_CALLBACK (microbrute_assistant_close), backend);
+		    G_CALLBACK (microbrute_assistant_close), NULL);
   g_signal_connect (calibration_assistant, "escape",
-		    G_CALLBACK (microbrute_assistant_close), backend);
+		    G_CALLBACK (microbrute_assistant_close), NULL);
   g_signal_connect (calibration_assistant, "prepare",
-		    G_CALLBACK (microbrute_assistant_prepare), backend);
+		    G_CALLBACK (microbrute_assistant_prepare), NULL);
 
   g_object_unref (G_OBJECT (builder));
 }
 
+void
+microbrute_destroy ()
+{
+  debug_print (1, "Destroying microbrute...");
+  gtk_widget_destroy (calibration_assistant);
+  gtk_widget_destroy (GTK_WIDGET (config_window));
+}
+
 struct maction *
 microbrute_maction_conf_builder (struct maction_context *context)
 {
   struct maction *ma;
 
-  if (!context->backend->conn_name ||
-      strcmp (context->backend->conn_name, MICROBRUTE_NAME))
+  if (!remote_browser.backend->conn_name ||
+      strcmp (remote_browser.backend->conn_name, MICROBRUTE_NAME))
     {
       return NULL;
     }
 
-  microbrute_configure_gui (context->backend, context->parent);
-
   ma = g_malloc (sizeof (struct maction));
   ma->type = MACTION_BUTTON;
   ma->name = _("_Configuration");
@@ -409,14 +388,12 @@ microbrute_maction_cal_builder (struct maction_context *context)
 {
   struct maction *ma;
 
-  if (!context->backend->conn_name ||
-      strcmp (context->backend->conn_name, MICROBRUTE_NAME))
+  if (!remote_browser.backend->conn_name ||
+      strcmp (remote_browser.backend->conn_name, MICROBRUTE_NAME))
     {
       return NULL;
     }
 
-  microbrute_configure_gui (context->backend, context->parent);
-
   ma = g_malloc (sizeof (struct maction));
   ma->type = MACTION_BUTTON;
   ma->name = _("_Calibration");
