@@ -19,54 +19,62 @@
  */
 
 #include <glib/gi18n.h>
-#include "maction.h"
-#include "../progress.h"
-#include "../browser.h"
+#include "browser.h"
 #include "connectors/microfreak.h"
+#include "elektroid.h"
+#include "maction.h"
+#include "progress_window.h"
 
-extern GtkWidget *dialog;
+extern GtkWindow *main_window;
 extern struct browser remote_browser;
 
-static gpointer
+static void
 microfreak_defragment_runner (gpointer data)
 {
-  struct backend *backend = data;
-  progress.sysex_transfer.active = TRUE;
-  microfreak_sample_defragment (backend);
-  progress_response (GTK_RESPONSE_ACCEPT);
-  return NULL;
+  microfreak_sample_defragment (remote_browser.backend);
+}
+
+static void
+microfreak_defragment_consumer (gpointer data)
+{
+  //Some operations fail unless everything is initiated again.
+  elektroid_refresh_devices ();
+}
+
+static void
+microfreak_defragment_callback_response (GtkDialog *dialog, gint response_id,
+					 gpointer user_data)
+{
+  if (response_id == GTK_RESPONSE_ACCEPT)
+    {
+      progress_window_open (microfreak_defragment_runner,
+			    microfreak_defragment_consumer, NULL, NULL,
+			    PROGRESS_TYPE_PULSE,
+			    _("Defragmenting Sample Memory"), "", FALSE);
+    }
+
+  gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
 static void
 microfreak_defragment_callback (GtkWidget *object, gpointer data)
 {
-  gint res;
-  struct maction_context *context = data;
-
-  dialog = gtk_message_dialog_new (GTK_WINDOW (context->parent),
-				   GTK_DIALOG_MODAL,
-				   GTK_MESSAGE_ERROR,
-				   GTK_BUTTONS_NONE,
-				   _
-				   ("The defragmentation process could take several minutes and could not be canceled. Are you sure you want to defragment the sample memory?"));
+  GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW (main_window),
+					      GTK_DIALOG_MODAL,
+					      GTK_MESSAGE_ERROR,
+					      GTK_BUTTONS_NONE,
+					      _
+					      ("The defragmentation process could take several minutes and could not be canceled. Are you sure you want to defragment the sample memory?"));
   gtk_dialog_add_buttons (GTK_DIALOG (dialog), _("_Cancel"),
 			  GTK_RESPONSE_CANCEL, _("_Defragment"),
 			  GTK_RESPONSE_ACCEPT, NULL);
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
-  res = gtk_dialog_run (GTK_DIALOG (dialog));
-  gtk_widget_destroy (dialog);
-  dialog = NULL;
 
-  if (res != GTK_RESPONSE_ACCEPT)
-    {
-      return;
-    }
+  g_signal_connect (dialog, "response",
+		    G_CALLBACK (microfreak_defragment_callback_response),
+		    NULL);
 
-  progress_run (microfreak_defragment_runner, PROGRESS_TYPE_PULSE,
-                context->backend, _("Defragmenting Sample Memory"), NULL,
-                FALSE, NULL);
-
-  browser_refresh (NULL, &remote_browser);
+  gtk_widget_set_visible (dialog, TRUE);
 }
 
 struct maction *
@@ -74,8 +82,8 @@ microfreak_maction_defrag_builder (struct maction_context *context)
 {
   struct maction *ma;
 
-  if (!context->backend->conn_name ||
-      strcmp (context->backend->conn_name, MICROFREAK_NAME))
+  if (!remote_browser.backend->conn_name ||
+      strcmp (remote_browser.backend->conn_name, MICROFREAK_NAME))
     {
       return NULL;
     }
