@@ -20,57 +20,56 @@
 
 #include "notifier.h"
 
-#define NOTIFIER_RATE_LIMIT 1000
+#define NOTIFIER_RATE_LIMIT_MS 1000
 
 static gboolean
-notifier_go_up (gpointer data)
+notifier_reset (gpointer data)
 {
   struct browser *browser = data;
-  browser_go_up (NULL, browser);
+  browser_cancel (browser);
+  browser->dir = get_system_startup_path (NULL);
+  browser_load_dir (browser);
   return FALSE;
 }
 
-static void
-notifier_parent_changed (GFileMonitor *self, GFile *file, GFile *other_file,
-			 GFileMonitorEvent event_type, gpointer user_data)
-{
-  struct notifier *notifier = user_data;
-  gchar *p1 = g_file_get_path (notifier->dir);
-  gchar *p2 = g_file_get_path (file);
-  gboolean itself = strcmp (p1, p2) == 0;
-
-  g_free (p1);
-  g_free (p2);
-
-  debug_print (2, "Processing notifier parent change...");
-
-  if (event_type == G_FILE_MONITOR_EVENT_DELETED && itself)
-    {
-      debug_print (1, "Processing notifier dir deletion...");
-      g_idle_add (notifier_go_up, notifier->browser);
-    }
-
-}
-
 static void
 notifier_changed (GFileMonitor *self, GFile *file, GFile *other_file,
 		  GFileMonitorEvent event_type, gpointer user_data)
 {
   struct notifier *notifier = user_data;
-  gchar *p1 = g_file_get_path (notifier->dir);
-  gchar *p2 = g_file_get_path (file);
-  gboolean itself = strcmp (p1, p2) == 0;
+  struct browser *browser = notifier->browser;
+  gchar *path_dir = g_file_get_path (notifier->dir);
+  gchar *path_file = g_file_get_path (file);
+  gboolean itself = strcmp (path_dir, path_file) == 0;
 
-  g_free (p1);
-  g_free (p2);
+  debug_print (2, "Processing notifier change (itself =?= %d)...", itself);
 
-  debug_print (2, "Processing notifier change...");
-
-  if (event_type != G_FILE_MONITOR_EVENT_DELETED || !itself)
+  if (itself)
     {
-      debug_print (1, "Processing notifier reload...");
-      g_idle_add (browser_load_dir, notifier->browser);
+      if (event_type == G_FILE_MONITOR_EVENT_DELETED)
+	{
+	  debug_print (1, "Processing notifier deleted dir...");
+	  g_idle_add (notifier_reset, browser);
+	}
     }
+  else
+    {
+      const gchar **exts = browser_get_exts (browser);
+
+      if ((event_type == G_FILE_MONITOR_EVENT_DELETED ||
+	   event_type == G_FILE_MONITOR_EVENT_CREATED ||
+	   event_type == G_FILE_MONITOR_EVENT_CHANGED ||
+	   event_type == G_FILE_MONITOR_EVENT_ATTRIBUTE_CHANGED) &&
+	  (!g_file_test (path_file, G_FILE_TEST_EXISTS) ||
+	   filename_is_dir_or_matches_exts (path_file, exts)))
+	{
+	  debug_print (1, "Processing notifier reload...");
+	  g_idle_add (browser_load_dir, browser);
+	}
+    }
+
+  g_free (path_dir);
+  g_free (path_file);
 }
 
 void
@@ -78,7 +77,6 @@ notifier_init (struct notifier **notifier, struct browser *browser)
 {
   struct notifier *n = g_malloc (sizeof (struct notifier));
   n->monitor = NULL;
-  n->parent_monitor = NULL;
   n->browser = browser;
   *notifier = n;
 }
@@ -86,8 +84,6 @@ notifier_init (struct notifier **notifier, struct browser *browser)
 void
 notifier_update_dir (struct notifier *notifier, gboolean active)
 {
-  GFile *parent;
-
   debug_print (1, "Changing %s browser path to '%s'...",
 	       notifier->browser->name, notifier->browser->dir);
 
@@ -95,44 +91,21 @@ notifier_update_dir (struct notifier *notifier, gboolean active)
     {
       g_object_unref (notifier->monitor);
       g_object_unref (notifier->dir);
+
+      notifier->monitor = NULL;
+      notifier->dir = NULL;
     }
 
-  if (notifier->parent_monitor)
-    {
-      g_object_unref (notifier->parent_monitor);
-    }
-
-  if (active)
+  if (active && strcmp (notifier->browser->dir, TOPMOST_DIR_WINDOWS))
     {
       notifier->dir = g_file_new_for_path (notifier->browser->dir);
       notifier->monitor = g_file_monitor_directory (notifier->dir,
-						    G_FILE_MONITOR_NONE, NULL,
-						    NULL);
-      g_file_monitor_set_rate_limit (notifier->monitor, NOTIFIER_RATE_LIMIT);
+						    G_FILE_MONITOR_NONE,
+						    NULL, NULL);
+      g_file_monitor_set_rate_limit (notifier->monitor,
+				     NOTIFIER_RATE_LIMIT_MS);
       g_signal_connect (notifier->monitor, "changed",
 			G_CALLBACK (notifier_changed), notifier);
-
-      parent = g_file_get_parent (notifier->dir);
-      if (parent)
-	{
-	  notifier->parent_monitor = g_file_monitor_directory (parent,
-							       G_FILE_MONITOR_NONE,
-							       NULL, NULL);
-	  g_file_monitor_set_rate_limit (notifier->parent_monitor,
-					 NOTIFIER_RATE_LIMIT);
-	  g_signal_connect (notifier->parent_monitor, "changed",
-			    G_CALLBACK (notifier_parent_changed), notifier);
-	  g_object_unref (parent);
-	}
-      else
-	{
-	  notifier->parent_monitor = NULL;
-	}
-    }
-  else
-    {
-      notifier->monitor = NULL;
-      notifier->parent_monitor = NULL;
     }
 }
 
@@ -145,10 +118,5 @@ notifier_destroy (struct notifier *notifier)
       g_object_unref (notifier->dir);
     }
 
-  if (notifier->parent_monitor)
-    {
-      g_object_unref (notifier->parent_monitor);
-    }
-
   g_free (notifier);
 }
