@@ -27,7 +27,7 @@
 #define PREFERENCES_FILE "/preferences.json"
 
 GSList *preferences;
-static GHashTable *preferences_hashtable;
+GHashTable *preferences_hashtable;
 
 static gpointer
 preferences_get_boolean_value (const gpointer in, gboolean def_value)
@@ -56,6 +56,12 @@ preferences_get_boolean_value_false (const gpointer b)
   return preferences_get_boolean_value (b, FALSE);
 }
 
+gpointer
+preferences_get_string_value_default (const gpointer s, const gchar *defaults)
+{
+  return s ? g_strdup (s) : g_strdup (defaults);
+}
+
 gpointer
 preferences_get_int_value (const gpointer in, gint max, gint min, gint def)
 {
@@ -175,8 +181,8 @@ preferences_get_value (const struct preference *p, JsonReader *reader)
 	}
       else if (p->type == PREFERENCE_TYPE_STRING)
 	{
-	  gchar *s = (gchar *) json_reader_get_string_value (reader);
-	  v = p->get_value (s);
+	  const gchar *s = json_reader_get_string_value (reader);
+	  v = p->get_value ((const gpointer) s);
 	}
       else
 	{
