@@ -21,15 +21,24 @@
 #include <samplerate.h>
 #include <math.h>
 #include <errno.h>
-#include "sample.h"
 #include "connectors/microfreak_sample.h"
+#include "preferences.h"
+#include "utils.h"
+#include "sample.h"
 
 #define LOAD_BUFFER_LEN (32 * KI)
 
-#define JUNK_CHUNK_ID "JUNK"
 #define SMPL_CHUNK_ID "smpl"
+#define JUNK_CHUNK_ID "JUNK"
+#define ACID_CHUNK_ID "acid"
+#define LIST_CHUNK_ID "LIST"
 
-#define HEADERS_SPACE 4096	//Gross estimation for the sample (WAV) headers
+#define LIST_CHUNK_INFO_SECTION_ID "INFO"
+
+#define CHUNK_SIZE 4
+#define SUBCHUNK_SIZE 4
+
+#define HEADERS_SPACE (128 * KI)	//Gross estimation for the sample (WAV) headers
 
 static const gchar *ELEKTROID_AUDIO_LOCAL_EXTS[] =
   { "wav", "ogg", "aiff", "flac", MICROFREAK_PWAVETABLE_EXT,
@@ -50,8 +59,8 @@ struct smpl_chunk_data
   guint32 midi_pitch_fraction;
   guint32 smpte_format;
   guint32 smpte_offset;
-  guint32 num_sampler_loops;
-  guint32 sampler_data;
+  guint32 num_sample_loops;
+  guint32 sample_data;
   struct sample_loop
   {
     guint32 cue_point_id;
@@ -63,10 +72,24 @@ struct smpl_chunk_data
   } sample_loop;
 };
 
-struct g_byte_array_io_data
+// Information extracted from https://github.com/libsndfile/libsndfile/blob/52b803f57a1f4d23471f5c5f77e1a21e0721ea0e/src/wav.c#L1528.
+struct acid_chunk_data
 {
-  GByteArray *array;
-  guint pos;
+  guint32 type;
+  guint16 root_note;
+  guint16 u0;
+  gfloat f0;
+  guint32 beats;
+  guint16 metre_num;
+  guint16 metre_den;
+  gfloat tempo;
+};
+
+struct list_info_chunk
+{
+  gchar chunk[CHUNK_SIZE];
+  guint32 size;
+  gchar data[];
 };
 
 static const guint8 JUNK_CHUNK_DATA[] = {
@@ -76,6 +99,23 @@ static const guint8 JUNK_CHUNK_DATA[] = {
   0, 0, 0, 0
 };
 
+struct g_byte_array_io_data
+{
+  GByteArray *array;
+  guint pos;
+};
+
+void
+task_control_set_sample_progress (struct task_control *control, gdouble p)
+{
+  task_control_set_progress_no_sync (control, p);
+
+  if (control->callback)
+    {
+      control->callback (control);
+    }
+}
+
 static sf_count_t
 get_filelen_byte_array_io (void *user_data)
 {
@@ -209,14 +249,14 @@ static SF_VIRTUAL_IO FILE_IO = {
 static gint
 sample_write_audio_file_data (struct idata *idata,
 			      struct g_byte_array_io_data *wave,
-			      struct job_control *control, guint32 format)
+			      struct task_control *control, guint32 format)
 {
   SF_INFO sf_info;
   SNDFILE *sndfile;
   sf_count_t frames, total;
-  struct SF_CHUNK_INFO junk_chunk_info;
-  struct SF_CHUNK_INFO smpl_chunk_info;
+  struct SF_CHUNK_INFO chunk_info;
   struct smpl_chunk_data smpl_chunk_data;
+  struct acid_chunk_data acid_chunk_data;
   GByteArray *sample = idata->content;
   struct sample_info *sample_info = idata->info;
 
@@ -238,24 +278,29 @@ sample_write_audio_file_data (struct idata *idata,
       return -1;
     }
 
-  strcpy (junk_chunk_info.id, JUNK_CHUNK_ID);
-  junk_chunk_info.id_size = strlen (JUNK_CHUNK_ID);
-  junk_chunk_info.datalen = sizeof (JUNK_CHUNK_DATA);
-  junk_chunk_info.data = (void *) JUNK_CHUNK_DATA;
-  if (sf_set_chunk (sndfile, &junk_chunk_info) != SF_ERR_NO_ERROR)
+  // JUNK chunk
+
+  strcpy (chunk_info.id, JUNK_CHUNK_ID);
+  chunk_info.id_size = CHUNK_SIZE;
+  chunk_info.datalen = sizeof (JUNK_CHUNK_DATA);
+  chunk_info.data = (void *) JUNK_CHUNK_DATA;
+  if (sf_set_chunk (sndfile, &chunk_info) != SF_ERR_NO_ERROR)
     {
       error_print ("%s", sf_strerror (sndfile));
     }
 
+  // smpl chunk
+
   smpl_chunk_data.manufacturer = 0;
   smpl_chunk_data.product = 0;
-  smpl_chunk_data.sample_period = 1e9 / sample_info->rate;
+  smpl_chunk_data.sample_period = GUINT32_TO_LE (1e9 / sample_info->rate);
   smpl_chunk_data.midi_unity_note = GUINT32_TO_LE (sample_info->midi_note);
-  smpl_chunk_data.midi_pitch_fraction = 0;
+  smpl_chunk_data.midi_pitch_fraction =
+    GUINT32_TO_LE (sample_info->midi_fraction);
   smpl_chunk_data.smpte_format = 0;
   smpl_chunk_data.smpte_offset = 0;
-  smpl_chunk_data.num_sampler_loops = 1;
-  smpl_chunk_data.sampler_data = 0;
+  smpl_chunk_data.num_sample_loops = GUINT32_TO_LE (1);
+  smpl_chunk_data.sample_data = 0;
   smpl_chunk_data.sample_loop.cue_point_id = 0;
   smpl_chunk_data.sample_loop.type = GUINT32_TO_LE (sample_info->loop_type);
   smpl_chunk_data.sample_loop.start = GUINT32_TO_LE (sample_info->loop_start);
@@ -263,15 +308,91 @@ sample_write_audio_file_data (struct idata *idata,
   smpl_chunk_data.sample_loop.fraction = 0;
   smpl_chunk_data.sample_loop.play_count = 0;
 
-  strcpy (smpl_chunk_info.id, SMPL_CHUNK_ID);
-  smpl_chunk_info.id_size = strlen (SMPL_CHUNK_ID);
-  smpl_chunk_info.datalen = sizeof (struct smpl_chunk_data);
-  smpl_chunk_info.data = &smpl_chunk_data;
-  if (sf_set_chunk (sndfile, &smpl_chunk_info) != SF_ERR_NO_ERROR)
+  strcpy (chunk_info.id, SMPL_CHUNK_ID);
+  chunk_info.id_size = CHUNK_SIZE;
+  chunk_info.datalen = sizeof (struct smpl_chunk_data);
+  chunk_info.data = &smpl_chunk_data;
+  if (sf_set_chunk (sndfile, &chunk_info) != SF_ERR_NO_ERROR)
     {
       error_print ("%s", sf_strerror (sndfile));
     }
 
+  // acid chunk
+
+  // If there are no beats, it is not a valid acid and we do nothing.
+  if (sample_info->beats || sample_info->metre_num ||
+      sample_info->metre_den || sample_info->tempo)
+    {
+      guint32 type = sample_info->acid_type ? sample_info->acid_type : 0x1e;
+      acid_chunk_data.type = GUINT32_TO_LE (type);
+      acid_chunk_data.root_note = sample_info->midi_note;
+      acid_chunk_data.u0 = GUINT32_TO_LE (0x8000);
+      acid_chunk_data.f0 = 0;
+      acid_chunk_data.beats = GUINT32_TO_LE (sample_info->beats);
+      acid_chunk_data.metre_num = GUINT16_TO_LE (sample_info->metre_num);
+      acid_chunk_data.metre_den = GUINT16_TO_LE (sample_info->metre_den);
+      acid_chunk_data.tempo = sample_info->tempo;
+
+      strcpy (chunk_info.id, ACID_CHUNK_ID);
+      chunk_info.id_size = CHUNK_SIZE;
+      chunk_info.datalen = sizeof (struct acid_chunk_data);
+      chunk_info.data = &acid_chunk_data;
+      if (sf_set_chunk (sndfile, &chunk_info) != SF_ERR_NO_ERROR)
+	{
+	  error_print ("%s", sf_strerror (sndfile));
+	}
+    }
+
+  // If there are no tags, it's better not having the LIST chunk
+  if (sample_info->tags && g_hash_table_size (sample_info->tags) > 0)
+    {
+      GList *keys, *e;
+      GByteArray *list_info_content = g_byte_array_sized_new (64 * KI);
+
+      g_byte_array_append (list_info_content, (guint8 *) "INFO",
+			   SUBCHUNK_SIZE);
+
+      keys = g_hash_table_get_keys (sample_info->tags);
+      e = keys;
+      while (e)
+	{
+	  guint32 size, len, sizele;
+	  gchar *buff;
+	  gchar *k = (gchar *) e->data;
+	  gchar *v = g_hash_table_lookup (sample_info->tags, k);
+	  size = strlen (k);
+	  g_byte_array_append (list_info_content, (guint8 *) k, size);
+	  len = strlen (v);
+	  size = len + 1;
+	  //The size must be a multiple of the word length (4 or sizeof (guint32))
+	  size = (size / sizeof (guint32)) +
+	    (size % sizeof (guint32) ? 1 : 0);
+	  size *= sizeof (guint32);
+	  sizele = GUINT32_TO_LE (size);
+	  g_byte_array_append (list_info_content, (guint8 *) & sizele,
+			       sizeof (guint32));
+	  buff = g_malloc (size);
+	  memset (buff, 0, size);
+	  memcpy (buff, v, len);
+	  g_byte_array_append (list_info_content, (guint8 *) buff, size);
+	  g_free (buff);
+	  e = e->next;
+	}
+
+      strcpy (chunk_info.id, LIST_CHUNK_ID);
+      chunk_info.id_size = CHUNK_SIZE;
+      chunk_info.datalen = list_info_content->len;
+      chunk_info.data = list_info_content->data;
+      if (sf_set_chunk (sndfile, &chunk_info) != SF_ERR_NO_ERROR)
+	{
+	  error_print ("%s", sf_strerror (sndfile));
+	}
+
+      g_list_free (keys);
+
+      g_byte_array_free (list_info_content, TRUE);
+    }
+
   if ((sample_info->format & SF_FORMAT_SUBMASK) == SF_FORMAT_PCM_16)
     {
       total = sf_writef_short (sndfile, (gint16 *) sample->data, frames);
@@ -304,7 +425,7 @@ sample_write_audio_file_data (struct idata *idata,
 
 gint
 sample_get_memfile_from_sample (struct idata *sample, struct idata *memfile,
-				struct job_control *control, guint32 format)
+				struct task_control *control, guint32 format)
 {
   gint err;
   GByteArray *content;
@@ -316,14 +437,14 @@ sample_get_memfile_from_sample (struct idata *sample, struct idata *memfile,
   content = g_byte_array_sized_new (sample->content->len * frame_size +
 				    HEADERS_SPACE);
   idata_init (memfile, content, sample->name ? strdup (sample->name) : NULL,
-	      NULL);
+	      NULL, NULL);
   data.pos = 0;
   data.array = content;
 
   err = sample_write_audio_file_data (sample, &data, control, format);
   if (err)
     {
-      idata_free (memfile);
+      idata_clear (memfile);
     }
 
   return err;
@@ -331,7 +452,7 @@ sample_get_memfile_from_sample (struct idata *sample, struct idata *memfile,
 
 gint
 sample_save_to_file (const gchar *path, struct idata *sample,
-		     struct job_control *control, guint32 format)
+		     struct task_control *control, guint32 format)
 {
   gint err;
   struct idata file;
@@ -343,7 +464,7 @@ sample_save_to_file (const gchar *path, struct idata *sample,
     }
 
   err = file_save (path, &file, control);
-  idata_free (&file);
+  idata_clear (&file);
 
   return err;
 }
@@ -454,28 +575,32 @@ audio_mono_to_stereo_int (gint32 *input, gint32 *output, gint size)
 
 static void
 sample_set_sample_info (struct sample_info *sample_info, SNDFILE *sndfile,
-			SF_INFO *sf_info)
+			SF_INFO *sf_info, gboolean tags)
 {
   struct SF_CHUNK_INFO chunk_info;
   SF_CHUNK_ITERATOR *chunk_iter;
   struct smpl_chunk_data smpl_chunk_data;
+  struct acid_chunk_data acid_chunk_data;
   gboolean disable_loop = FALSE;
 
+  sample_info_init (sample_info);
+
   sample_info->channels = sf_info->channels;
   sample_info->rate = sf_info->samplerate;
   sample_info->frames = sf_info->frames;
   sample_info->format = sf_info->format;
 
+  // smpl chunk
+
   strcpy (chunk_info.id, SMPL_CHUNK_ID);
-  chunk_info.id_size = strlen (SMPL_CHUNK_ID);
+  chunk_info.id_size = CHUNK_SIZE;
   chunk_iter = sf_get_chunk_iterator (sndfile, &chunk_info);
 
   if (chunk_iter)
     {
       chunk_info.datalen = sizeof (struct smpl_chunk_data);
-      memset (&smpl_chunk_data, 0, chunk_info.datalen);
-      debug_print (2, "%s chunk found (%d B)", SMPL_CHUNK_ID,
-		   chunk_info.datalen);
+      debug_print (2, "'%.*s' chunk found (%d B)", chunk_info.id_size,
+		   chunk_info.id, chunk_info.datalen);
       chunk_info.data = &smpl_chunk_data;
       sf_get_chunk_data (chunk_iter, &chunk_info);
       sample_info->loop_start =
@@ -486,6 +611,8 @@ sample_set_sample_info (struct sample_info *sample_info, SNDFILE *sndfile,
 	GUINT32_FROM_LE (smpl_chunk_data.sample_loop.type);
       sample_info->midi_note =
 	GUINT32_FROM_LE (smpl_chunk_data.midi_unity_note);
+      sample_info->midi_fraction =
+	GUINT32_FROM_LE (smpl_chunk_data.midi_pitch_fraction);
       if (sample_info->loop_start >= sample_info->frames)
 	{
 	  debug_print (2, "Bad loop start");
@@ -505,8 +632,8 @@ sample_set_sample_info (struct sample_info *sample_info, SNDFILE *sndfile,
   else
     {
       disable_loop = TRUE;
-      sample_info->midi_note = 0;
     }
+
   if (disable_loop)
     {
       sample_info->loop_start = sample_info->frames - 1;
@@ -516,10 +643,111 @@ sample_set_sample_info (struct sample_info *sample_info, SNDFILE *sndfile,
 
   debug_print (2, "Loop start at %d, loop end at %d",
 	       sample_info->loop_start, sample_info->loop_end);
+
+  // acid chunk
+
+  strcpy (chunk_info.id, ACID_CHUNK_ID);
+  chunk_info.id_size = CHUNK_SIZE;
+  chunk_iter = sf_get_chunk_iterator (sndfile, &chunk_info);
+
+  if (chunk_iter)
+    {
+      chunk_info.datalen = sizeof (struct acid_chunk_data);
+      debug_print (2, "'%.*s' chunk found (%d B)", chunk_info.id_size,
+		   chunk_info.id, chunk_info.datalen);
+
+      chunk_info.data = &acid_chunk_data;
+      sf_get_chunk_data (chunk_iter, &chunk_info);
+
+      sample_info->acid_type = GUINT32_FROM_LE (acid_chunk_data.type);
+      sample_info->beats = GUINT32_FROM_LE (acid_chunk_data.beats);
+      sample_info->metre_num = GUINT16_FROM_LE (acid_chunk_data.metre_num);
+      sample_info->metre_den = GUINT16_FROM_LE (acid_chunk_data.metre_den);
+      sample_info->tempo = acid_chunk_data.tempo;
+
+      if (acid_chunk_data.root_note != sample_info->midi_note)
+	{
+	  // Probably, the midi_note was not right or set in the smpl chunk
+	  if (sample_info->midi_note == 0 && acid_chunk_data.root_note != 0)
+	    {
+	      debug_print (2, "Fixing MIDI note to %d...",
+			   acid_chunk_data.root_note);
+	      sample_info->midi_note = acid_chunk_data.root_note;
+	    }
+	  else
+	    {
+	      error_print ("Unmatching MIDI note (%d != %d)",
+			   sample_info->midi_note, acid_chunk_data.root_note);
+	    }
+	}
+
+      debug_print (2, "Metric: %d %d; beats: %d; tempo: %.2f BPM",
+		   sample_info->metre_num, sample_info->metre_den,
+		   sample_info->beats, sample_info->tempo);
+
+      while (chunk_iter)
+	{
+	  chunk_iter = sf_next_chunk_iterator (chunk_iter);
+	}
+    }
+
+  // LIST INFO chunk
+
+  // tags are not required to be initialized when setting the sample_info
+  if (tags)
+    {
+      strcpy (chunk_info.id, LIST_CHUNK_ID);
+      chunk_info.id_size = CHUNK_SIZE;
+      chunk_iter = sf_get_chunk_iterator (sndfile, &chunk_info);
+
+      if (chunk_iter &&
+	  sf_get_chunk_size (chunk_iter, &chunk_info) == SF_ERR_NO_ERROR &&
+	  chunk_info.datalen > 0)
+	{
+	  guint8 *raw;
+	  guint32 read;
+	  struct list_info_chunk *subchunk;
+
+	  raw = g_malloc (chunk_info.datalen);
+	  chunk_info.data = raw;
+	  sf_get_chunk_data (chunk_iter, &chunk_info);
+
+	  subchunk = (struct list_info_chunk *) chunk_info.data;
+	  if (!strncmp (subchunk->chunk, LIST_CHUNK_INFO_SECTION_ID,
+			CHUNK_SIZE))
+	    {
+	      sample_info->tags = sample_info_tags_new ();
+
+	      read = CHUNK_SIZE;
+	      while (read < chunk_info.datalen)
+		{
+		  gchar *k, *v;
+		  subchunk = (struct list_info_chunk *) &raw[read];
+
+		  k = g_malloc (CHUNK_SIZE + 1);
+		  memcpy (k, subchunk->chunk, CHUNK_SIZE);
+		  k[CHUNK_SIZE] = 0;
+		  v = g_strdup (subchunk->data);
+		  debug_print (3, "Found tag '%s' with '%s' value", k, v);
+		  g_hash_table_insert (sample_info->tags, k, v);
+
+		  read += sizeof (struct list_info_chunk) +
+		    GUINT32_FROM_LE (subchunk->size);
+		}
+	    }
+
+	  g_free (raw);
+
+	  while (chunk_iter)
+	    {
+	      chunk_iter = sf_next_chunk_iterator (chunk_iter);
+	    }
+	}
+    }
 }
 
 static gint
-sample_load_sample_info_libsndfile (const gchar *path,
+sample_load_libsndfile_sample_info (const gchar *path,
 				    struct sample_info *sample_info)
 {
   SF_INFO sf_info;
@@ -541,7 +769,7 @@ sample_load_sample_info_libsndfile (const gchar *path,
       goto end;
     }
 
-  sample_set_sample_info (sample_info, sndfile, &sf_info);
+  sample_set_sample_info (sample_info, sndfile, &sf_info, TRUE);
 
 end:
   fclose (file);
@@ -549,27 +777,27 @@ end:
 }
 
 static gint
-sample_load_microfreak_sample (const gchar *path, struct idata *sample,
-			       struct job_control *control)
+sample_load_microfreak (const gchar *path, struct idata *sample,
+			struct task_control *control)
 {
   gint err;
   const gchar *ext = filename_get_ext (path);
 
   if (!strcmp (MICROFREAK_PWAVETABLE_EXT, ext))
     {
-      err = microfreak_pwavetable_load (path, sample, control);
+      err = microfreak_pwavetable_load (NULL, path, sample, control);
     }
   else if (!strcmp (MICROFREAK_ZWAVETABLE_EXT, ext))
     {
-      err = microfreak_zwavetable_load (path, sample, control);
+      err = microfreak_zwavetable_load (NULL, path, sample, control);
     }
   else if (!strcmp (MICROFREAK_PSAMPLE_EXT, ext))
     {
-      err = microfreak_psample_load (path, sample, control);
+      err = microfreak_psample_load (NULL, path, sample, control);
     }
   else if (!strcmp (MICROFREAK_ZSAMPLE_EXT, ext))
     {
-      err = microfreak_zsample_load (path, sample, control);
+      err = microfreak_zsample_load (NULL, path, sample, control);
     }
   else
     {
@@ -579,6 +807,32 @@ sample_load_microfreak_sample (const gchar *path, struct idata *sample,
   return err;
 }
 
+static gint
+sample_load_microfreak_sample_info (const gchar *path,
+				    struct sample_info *sample_info)
+{
+  gint err;
+  struct idata aux;
+  struct task_control control;
+
+  controllable_init (&control.controllable);
+  control.callback = NULL;
+
+  err = sample_load_microfreak (path, &aux, &control);
+  if (err)
+    {
+      return err;
+    }
+
+  sample_info_copy (sample_info, aux.info);
+
+  idata_clear (&aux);
+
+  controllable_clear (&control.controllable);
+
+  return 0;
+}
+
 static gboolean
 sample_microfreak_filename (const gchar *path)
 {
@@ -590,40 +844,27 @@ sample_microfreak_filename (const gchar *path)
     !strcmp (MICROFREAK_ZSAMPLE_EXT, ext);
 }
 
+// Calls to this function require sample_info to be initialized always.
+
 gint
 sample_load_sample_info (const gchar *path, struct sample_info *sample_info)
 {
   gint err;
 
-  memset (sample_info, 0, sizeof (struct sample_info));
-
   if (sample_microfreak_filename (path))
     {
-      struct idata aux;
-      struct job_control control;
-
-      control.active = TRUE;
-      g_mutex_init (&control.mutex);
-
-      err = sample_load_microfreak_sample (path, &aux, &control);
-      if (err)
-	{
-	  return err;
-	}
-
-      memcpy (sample_info, aux.info, sizeof (struct sample_info));
-      idata_free (&aux);
+      err = sample_load_microfreak_sample_info (path, sample_info);
     }
   else
     {
-      err = sample_load_sample_info_libsndfile (path, sample_info);
+      err = sample_load_libsndfile_sample_info (path, sample_info);
     }
 
   return err;
 }
 
 static void
-sample_info_fix_frame_values (struct sample_info *sample_info)
+sample_info_fix_loop_points (struct sample_info *sample_info)
 {
   if (sample_info->loop_start >= sample_info->frames)
     {
@@ -637,31 +878,33 @@ sample_info_fix_frame_values (struct sample_info *sample_info)
 
 static gint
 sample_load_libsndfile (void *data, SF_VIRTUAL_IO *sf_virtual_io,
-			struct job_control *control, struct idata *idata,
-			const struct sample_info *sample_info_req,
+			struct task_control *control, struct idata *idata,
+			const struct sample_load_opts *sample_load_opts,
 			struct sample_info *sample_info_src,
-			sample_load_cb cb, gpointer cb_data,
-			const gchar *name)
+			task_control_progress_callback cb, const gchar *name)
 {
   SF_INFO sf_info;
   SNDFILE *sndfile;
   SRC_DATA src_data;
   SRC_STATE *src_state;
   void *buffer_input;
+  void *buffer_input_float;
   void *buffer_input_multi;
   void *buffer_input_mono;
   void *buffer_input_stereo;
   void *buffer_i;		//For gint16 or gint32
   gfloat *buffer_f;
   void *buffer_output;
-  gint err = 0, resampled_buffer_len, frames;
-  gboolean active, rounding_fix;
+  gint err, resampled_buffer_len, frames;
+  gboolean active, estimation_issue;
   gdouble ratio;
   guint bytes_per_sample, bytes_per_frame;
-  guint32 read_frames, actual_frames = 0;
-  GByteArray *sample = NULL;
+  guint32 read_frames, actual_frames;
+  GByteArray *sample;
   struct sample_info *sample_info;
 
+  debug_print (1, "Loading sample...");
+
   sf_info.format = 0;
   sndfile = sf_open_virtual (sf_virtual_io, SFM_READ, &sf_info, data);
   if (!sndfile)
@@ -670,38 +913,43 @@ sample_load_libsndfile (void *data, SF_VIRTUAL_IO *sf_virtual_io,
       return -1;
     }
 
-  sample_set_sample_info (sample_info_src, sndfile, &sf_info);
+  err = 0;
+  actual_frames = 0;
+  estimation_issue = FALSE;
+  sample = NULL;
+
+  sample_set_sample_info (sample_info_src, sndfile, &sf_info,
+			  sample_load_opts->tags);
 
   sample_info = g_malloc (sizeof (struct sample_info));
+  sample_info_copy_steal_tags (sample_info, sample_info_src);
+  // tags are required to be initialized when loading a sample
+  if (!sample_info->tags)
+    {
+      sample_info->tags = sample_info_tags_new ();
+    }
 
-  sample_info->midi_note = sample_info_src->midi_note;
-  sample_info->loop_type = sample_info_src->loop_type;
-  sample_info->channels = sample_info_req->channels ?
-    sample_info_req->channels : sample_info_src->channels;
-  sample_info->rate = sample_info_req->rate ? sample_info_req->rate :
+  sample_info->rate = sample_load_opts->rate ? sample_load_opts->rate :
     sample_info_src->rate;
-  sample_info->format = sample_info_req->format ? sample_info_req->format :
-    SF_FORMAT_PCM_16;
+  //Only the sample format is needed. If the file format is provided, it must be ignored.
+  sample_info->format = sample_load_opts->format ?
+    sample_load_opts->format : (sample_info_src->format & SF_FORMAT_SUBMASK);
   if (sample_info->format != SF_FORMAT_PCM_16 &&
       sample_info->format != SF_FORMAT_PCM_32 &&
       sample_info->format != SF_FORMAT_FLOAT)
     {
-      error_print ("Invalid sample format. Using short...");
-      sample_info->format = SF_FORMAT_PCM_16;
+      debug_print (1, "Invalid sample format. Using internal format...");
+      sample_info->format = sample_get_internal_format ();
     }
+  sample_info->channels = sample_load_opts->channels ?
+    sample_load_opts->channels : sample_info_src->channels;
 
   bytes_per_frame = SAMPLE_INFO_FRAME_SIZE (sample_info);
   bytes_per_sample = SAMPLE_SIZE (sample_info->format);
 
-  //Set scale factor. See http://www.mega-nerd.com/libsndfile/api.html#note2
-  if ((sample_info_src->format & SF_FORMAT_FLOAT) == SF_FORMAT_FLOAT ||
-      (sample_info_src->format & SF_FORMAT_DOUBLE) == SF_FORMAT_DOUBLE)
-    {
-      debug_print (2,
-		   "Setting scale factor to ensure correct integer readings...");
-      sf_command (sndfile, SFC_SET_SCALE_FLOAT_INT_READ, NULL, SF_TRUE);
-    }
-
+  buffer_input_float = g_malloc (LOAD_BUFFER_LEN *
+				 FRAME_SIZE (sample_info_src->channels,
+					     SF_FORMAT_FLOAT));
   buffer_input_multi = g_malloc (LOAD_BUFFER_LEN *
 				 FRAME_SIZE (sample_info_src->channels,
 					     sample_info->format));
@@ -741,19 +989,20 @@ sample_load_libsndfile (void *data, SF_VIRTUAL_IO *sf_virtual_io,
   active = TRUE;
   if (control)
     {
-      g_mutex_lock (&control->mutex);
-      active = control->active;
+      g_mutex_lock (&control->controllable.mutex);
+      active = control->controllable.active;
     }
   sample_info->frames = ceil (sample_info_src->frames * ratio);	//Upper bound estimation. The actual amount is updated later.
   sample_info->loop_start = round (sample_info_src->loop_start * ratio);
   sample_info->loop_end = round (sample_info_src->loop_end * ratio);
-  sample_info_fix_frame_values (sample_info);
+  sample_info_fix_loop_points (sample_info);
 
   sample = g_byte_array_sized_new (sample_info->frames * bytes_per_frame);
-  idata_init (idata, sample, strdup (name), sample_info);
+  idata_init (idata, sample, name ? strdup (name) : NULL, sample_info,
+	      sample_info_free);
   if (control)
     {
-      g_mutex_unlock (&control->mutex);
+      g_mutex_unlock (&control->controllable.mutex);
     }
 
   debug_print (2, "Loading sample (%d frames)...", sample_info_src->frames);
@@ -770,13 +1019,47 @@ sample_load_libsndfile (void *data, SF_VIRTUAL_IO *sf_virtual_io,
 	}
       else if (sample_info->format == SF_FORMAT_PCM_32)
 	{
-	  frames = sf_readf_int (sndfile, (gint32 *) buffer_input_multi,
-				 LOAD_BUFFER_LEN);
+	  if ((sample_info_src->format & SF_FORMAT_SUBMASK) ==
+	      SF_FORMAT_FLOAT)
+	    {
+	      frames = sf_readf_float (sndfile, (gfloat *) buffer_input_float,
+				       LOAD_BUFFER_LEN);
+	      gint32 *v = buffer_input_multi;
+	      gfloat *f = buffer_input_float;
+	      for (int i = 0; i < frames * sample_info_src->channels; i++)
+		{
+		  *v = *f * G_MAXINT32;
+		  v++;
+		  f++;
+		}
+	    }
+	  else
+	    {
+	      frames = sf_readf_int (sndfile, (gint32 *) buffer_input_multi,
+				     LOAD_BUFFER_LEN);
+	    }
 	}
       else
 	{
-	  frames = sf_readf_short (sndfile, (gint16 *) buffer_input_multi,
-				   LOAD_BUFFER_LEN);
+	  if ((sample_info_src->format & SF_FORMAT_SUBMASK) ==
+	      SF_FORMAT_FLOAT)
+	    {
+	      frames = sf_readf_float (sndfile, (gfloat *) buffer_input_float,
+				       LOAD_BUFFER_LEN);
+	      gint16 *v = buffer_input_multi;
+	      gfloat *f = buffer_input_float;
+	      for (int i = 0; i < frames * sample_info_src->channels; i++)
+		{
+		  *v = *f * G_MAXINT16;
+		  v++;
+		  f++;
+		}
+	    }
+	  else
+	    {
+	      frames = sf_readf_short (sndfile, (gint16 *) buffer_input_multi,
+				       LOAD_BUFFER_LEN);
+	    }
 	}
       read_frames += frames;
 
@@ -837,14 +1120,14 @@ sample_load_libsndfile (void *data, SF_VIRTUAL_IO *sf_virtual_io,
 	{
 	  if (control)
 	    {
-	      g_mutex_lock (&control->mutex);
+	      g_mutex_lock (&control->controllable.mutex);
 	    }
 	  g_byte_array_append (sample, (guint8 *) buffer_input,
 			       frames * bytes_per_frame);
 	  actual_frames += frames;
 	  if (control)
 	    {
-	      g_mutex_unlock (&control->mutex);
+	      g_mutex_unlock (&control->controllable.mutex);
 	    }
 	}
       else
@@ -879,7 +1162,7 @@ sample_load_libsndfile (void *data, SF_VIRTUAL_IO *sf_virtual_io,
 
 	  if (control)
 	    {
-	      g_mutex_lock (&control->mutex);
+	      g_mutex_lock (&control->controllable.mutex);
 	    }
 
 	  if (sample_info->format == SF_FORMAT_PCM_32)
@@ -899,20 +1182,23 @@ sample_load_libsndfile (void *data, SF_VIRTUAL_IO *sf_virtual_io,
 	  actual_frames += src_data.output_frames_gen;
 	  if (control)
 	    {
-	      g_mutex_unlock (&control->mutex);
+	      g_mutex_unlock (&control->controllable.mutex);
 	    }
 	}
 
       if (control)
 	{
-	  cb (control, read_frames * 1.0 / sample_info_src->frames, cb_data);
-	  active = job_control_get_active_lock (control);
+	  g_mutex_lock (&control->controllable.mutex);
+	  cb (control, read_frames * 1.0 / sample_info_src->frames);
+	  active = control->controllable.active;
+	  g_mutex_unlock (&control->controllable.mutex);
 	}
     }
 
   src_delete (src_state);
 
 cleanup:
+  g_free (buffer_input_float);
   g_free (buffer_input_multi);
   g_free (buffer_input_mono);
   g_free (buffer_input_stereo);
@@ -934,66 +1220,103 @@ cleanup:
 
   if (control)
     {
-      g_mutex_lock (&control->mutex);
+      g_mutex_lock (&control->controllable.mutex);
     }
   if (!active || !sample_info->frames || err)
     {
-      idata_free (idata);
-      cb (control, 0, cb_data);
-      g_mutex_unlock (&control->mutex);
+      idata_clear (idata);
+      g_mutex_unlock (&control->controllable.mutex);
       return -1;
     }
-  // This removes the additional samples added by the estimation above.
-  if (sample_info->frames > actual_frames)
+  // This fixes the estimation above.
+  // In some cases, libsamplerate generates an additional frame in stereo while it does not do that in mono.
+  // We honour whatever libsamplerate generates.
+  if (sample_info->frames != actual_frames)
     {
-      rounding_fix = TRUE;
+      debug_print (2, "Applying frames estimation fix...");
+      estimation_issue = TRUE;
       sample_info->frames = actual_frames;
-      sample_info_fix_frame_values (sample_info);
-      sample->len = sample_info->frames * bytes_per_frame;
+      sample_info_fix_loop_points (sample_info);
     }
   if (control)
     {
-      //It there was a rounding fix in the previous lines, the call is needed to detect the end of the loading process.
-      if (rounding_fix)
+      //It there was an estimation issue in the previous lines, the call is needed to detect the end of the loading process.
+      if (estimation_issue)
 	{
-	  cb (control, 1.0, cb_data);
+	  debug_print (2, "Notifying because of frames estimation issue...");
+	  cb (control, 1.0);
 	}
-      g_mutex_unlock (&control->mutex);
+      g_mutex_unlock (&control->controllable.mutex);
     }
 
   return 0;
 }
 
+guint32
+sample_get_actual_frames (struct idata *sample)
+{
+  struct sample_info *sample_info = sample->info;
+  guint32 bpf = SAMPLE_INFO_FRAME_SIZE (sample_info);
+  return sample->content->len / bpf;
+}
+
+gboolean
+sample_load_completed (struct idata *sample, guint32 *actual_frames)
+{
+  gboolean completed;
+  guint32 actual;
+  struct sample_info *sample_info = sample->info;
+
+  if (!sample_info)
+    {
+      if (actual_frames)
+	{
+	  *actual_frames = 0;
+	}
+      return FALSE;
+    }
+
+  actual = sample_get_actual_frames (sample);
+  completed = actual == sample_info->frames && actual;
+  if (actual_frames)
+    {
+      *actual_frames = actual;
+    }
+  return completed;
+}
+
 gint
 sample_load_from_memfile (struct idata *memfile, struct idata *sample,
-			  struct job_control *control,
-			  const struct sample_info *sample_info_req,
+			  struct task_control *control,
+			  const struct sample_load_opts *sample_load_opts,
 			  struct sample_info *sample_info_src)
 {
   struct g_byte_array_io_data data;
   data.pos = 0;
   data.array = memfile->content;
   return sample_load_libsndfile (&data, &G_BYTE_ARRAY_IO, control, sample,
-				 sample_info_req, sample_info_src,
-				 job_control_set_sample_progress_no_sync,
-				 NULL, memfile->name);
+				 sample_load_opts, sample_info_src,
+				 task_control_set_sample_progress,
+				 memfile->name);
 }
 
-// Reloads the input into the output following all the requirements.
+// Reloads the input into the output fulfilling all the requirements.
 
 gint
 sample_reload (struct idata *input, struct idata *output,
-	       struct job_control *control,
-	       const struct sample_info *sample_info_req, sample_load_cb cb,
-	       gpointer cb_data)
+	       struct task_control *control,
+	       const struct sample_load_opts *sample_load_opts,
+	       task_control_progress_callback cb)
 {
   gint err;
   struct idata aux;
   struct sample_info sample_info_src;
   struct g_byte_array_io_data data;
 
-  err = sample_get_memfile_from_sample (input, &aux, NULL,
-					SF_FORMAT_WAV | SF_FORMAT_PCM_16);
+  sample_info_copy (&sample_info_src, input->info);
+
+  err = sample_get_memfile_from_sample (input, &aux, NULL, SF_FORMAT_WAV |
+					sample_info_src.format);
   if (err)
     {
       return err;
@@ -1002,35 +1325,34 @@ sample_reload (struct idata *input, struct idata *output,
   data.pos = 0;
   data.array = aux.content;
   err = sample_load_libsndfile (&data, &G_BYTE_ARRAY_IO, control, output,
-				sample_info_req, &sample_info_src, cb,
-				cb_data, input->name);
-  idata_free (&aux);
+				sample_load_opts, &sample_info_src, cb,
+				input->name);
+  idata_clear (&aux);
 
   return err;
 }
 
 gint
 sample_load_from_file_full (const gchar *path, struct idata *sample,
-			    struct job_control *control,
-			    const struct sample_info *sample_info_req,
+			    struct task_control *control,
+			    const struct sample_load_opts *sample_load_opts,
 			    struct sample_info *sample_info_src,
-			    sample_load_cb cb, gpointer cb_data)
+			    task_control_progress_callback cb)
 {
   gint err;
   if (sample_microfreak_filename (path))
     {
       struct idata aux;
 
-      err = sample_load_microfreak_sample (path, &aux, control);
+      err = sample_load_microfreak (path, &aux, control);
       if (err)
 	{
 	  return err;
 	}
 
-      memcpy (sample_info_src, aux.info, sizeof (struct sample_info));
-      err = sample_reload (&aux, sample, control, sample_info_req, cb,
-			   cb_data);
-      idata_free (&aux);
+      sample_info_copy (sample_info_src, aux.info);
+      err = sample_reload (&aux, sample, control, sample_load_opts, cb);
+      idata_clear (&aux);
     }
   else
     {
@@ -1043,8 +1365,8 @@ sample_load_from_file_full (const gchar *path, struct idata *sample,
       name = g_path_get_basename (path);
       filename_remove_ext (name);
       err = sample_load_libsndfile (file, &FILE_IO, control, sample,
-				    sample_info_req, sample_info_src, cb,
-				    cb_data, name);
+				    sample_load_opts, sample_info_src, cb,
+				    name);
       g_free (name);
       fclose (file);
     }
@@ -1054,18 +1376,18 @@ sample_load_from_file_full (const gchar *path, struct idata *sample,
 
 gint
 sample_load_from_file (const gchar *path, struct idata *sample,
-		       struct job_control *control,
-		       const struct sample_info *sample_info_req,
+		       struct task_control *control,
+		       const struct sample_load_opts *sample_load_opts,
 		       struct sample_info *sample_info_src)
 {
   return sample_load_from_file_full (path, sample, control,
-				     sample_info_req, sample_info_src,
-				     job_control_set_sample_progress_no_sync,
-				     NULL);
+				     sample_load_opts, sample_info_src,
+				     task_control_set_sample_progress);
 }
 
 const gchar **
-sample_get_sample_extensions ()
+sample_get_sample_extensions (struct backend *backend,
+			      const struct fs_operations *ops)
 {
   return ELEKTROID_AUDIO_LOCAL_EXTS;
 }
@@ -1073,24 +1395,38 @@ sample_get_sample_extensions ()
 const gchar *
 sample_get_format (struct sample_info *sample_info)
 {
-  switch (sample_info->format & SF_FORMAT_TYPEMASK)
+  guint64 format = sample_info->format & ELEKTROID_SAMPLE_FORMAT_MASK;
+  if (format)
     {
-    case SF_FORMAT_WAV:
-      return "WAV";
-    case SF_FORMAT_AIFF:
-      return "AIFF";
-    case SF_FORMAT_AU:
-      return "Au";
-    case SF_FORMAT_FLAC:
-      return "FLAC";
-    case SF_FORMAT_OGG:
-      return "Ogg";
+      switch (format)
+	{
+	case ELEKTROID_SAMPLE_FORMAT_MICROFREAK:
+	  return "MicroFreak";
+	default:
+	  return SF_FORMAT_UNKNOWN;
+	}
+    }
+  else
+    {
+      switch (sample_info->format & SF_FORMAT_TYPEMASK)
+	{
+	case SF_FORMAT_WAV:
+	  return "WAV";
+	case SF_FORMAT_AIFF:
+	  return "AIFF";
+	case SF_FORMAT_AU:
+	  return "Au";
+	case SF_FORMAT_FLAC:
+	  return "FLAC";
+	case SF_FORMAT_OGG:
+	  return "Ogg";
 #if !defined(__linux__) || HAVE_SNDFILE_MP3
-    case SF_FORMAT_MPEG:
-      return "MPEG";
+	case SF_FORMAT_MPEG:
+	  return "MPEG";
 #endif
-    default:
-      return "?";
+	default:
+	  return SF_FORMAT_UNKNOWN;
+	}
     }
 }
 
@@ -1100,20 +1436,88 @@ sample_get_subtype (struct sample_info *sample_info)
   switch (sample_info->format & SF_FORMAT_SUBMASK)
     {
     case SF_FORMAT_PCM_S8:
-      return "s8";
+      return SF_FORMAT_PCM_S8_STR;
     case SF_FORMAT_PCM_16:
-      return "s16";
+      return SF_FORMAT_PCM_16_STR;
     case SF_FORMAT_PCM_24:
-      return "s24";
+      return SF_FORMAT_PCM_24_STR;
     case SF_FORMAT_PCM_32:
-      return "s32";
+      return SF_FORMAT_PCM_32_STR;
     case SF_FORMAT_PCM_U8:
-      return "u8";
+      return SF_FORMAT_PCM_U8_STR;
     case SF_FORMAT_FLOAT:
-      return "f32";
+      return SF_FORMAT_FLOAT_STR;
     case SF_FORMAT_DOUBLE:
-      return "f64";
+      return SF_FORMAT_DOUBLE_STR;
     default:
-      return "?";
+      return SF_FORMAT_UNKNOWN;
+    }
+}
+
+guint32
+sample_get_internal_format ()
+{
+  return preferences_get_boolean (PREF_KEY_AUDIO_USE_FLOAT) ?
+    SF_FORMAT_FLOAT : SF_FORMAT_PCM_16;
+}
+
+void
+sample_load_opts_init (struct sample_load_opts *opts, guint32 channels,
+		       guint32 rate, guint32 format, gboolean tags)
+{
+  opts->channels = channels;
+  opts->rate = rate;
+  opts->format = format;
+  opts->tags = tags;
+}
+
+void
+sample_load_opts_init_direct (struct sample_load_opts *opts, gboolean tags)
+{
+  sample_load_opts_init (opts, 0, 0, 0, tags);
+}
+
+void
+sample_load_opts_init_from_sample_info (struct sample_load_opts *opts,
+					struct sample_info *sample_info,
+					gboolean tags)
+{
+  sample_load_opts_init (opts, sample_info->channels, sample_info->rate,
+			 sample_info->format & SF_FORMAT_SUBMASK, tags);
+}
+
+// Only saving to sample formats allowing all the features is allowed.
+// If the format does not allow saving, an export should be done and the user should be informed about this.
+
+gboolean
+sample_format_is_valid_to_save (struct sample_info *sample_info)
+{
+  // The sample is a WAV file or a record buffer
+  return ((sample_info->format & SF_FORMAT_TYPEMASK) == SF_FORMAT_WAV) ||
+    ((sample_info->format & SF_FORMAT_TYPEMASK) == 0 &&
+     (sample_info->format & ELEKTROID_SAMPLE_FORMAT_MASK) == 0);
+}
+
+// The format in sample_info_src needs to be WAV regardless of the subtype as no other saving format is allowed.
+// If the subtype is not supported, SF_FORMAT_FLOAT to provide the maximum precission.
+
+void
+sample_format_set_to_save (struct sample_info *sample_info)
+{
+  guint32 subtype = sample_info->format & SF_FORMAT_SUBMASK;
+  sample_info->format = SF_FORMAT_WAV;
+  switch (subtype)
+    {
+    case SF_FORMAT_PCM_S8:
+    case SF_FORMAT_PCM_16:
+    case SF_FORMAT_PCM_24:
+    case SF_FORMAT_PCM_32:
+    case SF_FORMAT_PCM_U8:
+    case SF_FORMAT_FLOAT:
+    case SF_FORMAT_DOUBLE:
+      sample_info->format |= subtype;
+      return;
+    default:
+      sample_info->format |= SF_FORMAT_FLOAT;
     }
 }
