@@ -3,12 +3,12 @@
 err=0
 
 echo "Cleaning up sample..."
-$ecli elektron-sample-rm $TEST_DEVICE:/auto-test/square
-$ecli elektron-sample-rmdir $TEST_DEVICE:/auto-test
+$ecli elektron:sample:rm $TEST_DEVICE:/auto-test/square
+$ecli elektron:sample:rmdir $TEST_DEVICE:/auto-test
 
 echo "Preparing tests..."
-$ecli elektron-project-ul $srcdir/res/connectors/elektron_project.data $TEST_DEVICE:/128
-$ecli elektron-project-dl $TEST_DEVICE:/128
+$ecli elektron:project:ul $srcdir/res/connectors/elektron_project.data $TEST_DEVICE:/128
+$ecli elektron:project:dl $TEST_DEVICE:/128
 
 src_content=$(unzip -l $srcdir/res/connectors/elektron_project.data | tail -n +4 | head -n -2 | awk '{print $1" "$4}')
 dst_content=$(unzip -l $srcdir/PROJECT.dtprj | tail -n +4 | head -n -2 | awk '{print $1" "$4}')
@@ -22,7 +22,7 @@ cksum2=$(echo "$dst_content" | cksum)
 
 if [ $err -eq 0 ]; then
   echo "Looking for sample..."
-  $ecli elektron-sample-dl $TEST_DEVICE:/auto-test/square
+  $ecli elektron:sample:dl $TEST_DEVICE:/auto-test/square
   [ $? -ne 0 ] && err=1
 fi
 
@@ -34,8 +34,8 @@ fi
 echo "Cleaning up..."
 rm -f $srcdir/PROJECT.dtprj
 rm -f $srcdir/square.wav
-$ecli elektron-project-rm $TEST_DEVICE:/H/256
-$ecli elektron-sample-rm $TEST_DEVICE:/auto-test/square
-$ecli elektron-sample-rmdir $TEST_DEVICE:/auto-test
+$ecli elektron:project:rm $TEST_DEVICE:/H/256
+$ecli elektron:sample:rm $TEST_DEVICE:/auto-test/square
+$ecli elektron:sample:rmdir $TEST_DEVICE:/auto-test
 
 exit $err
