@@ -1,7 +1,10 @@
 #!/usr/bin/env bash
 
 download=true
+download_ext=
 [ "$1" == "--no-download" ] && shift && download=false
+# Used to search the download file by its extension. Needed for files that don't use the connector name, such as wav files.
+[ "$1" == "--download-ext" ] && shift && download_ext=".$1" && shift
 
 CONN=$1
 FS=$2
@@ -21,23 +24,32 @@ BACKUP_PREFIX="Backup - "
 function exitWithError() {
   if [ -n "$FILE_BACKUP" ]; then
     echo "Restoring..."
-    $ecli ${CONN}-${FS}-ul "$FILE_BACKUP" $TEST_DEVICE:$FILE_PATH
+    FILE_BACKUP_ORI=$(echo "$FILE_BACKUP" | sed "s^/tmp/$BACKUP_PREFIX^^")
+    mv "$FILE_BACKUP" "$FILE_BACKUP_ORI"
+    $ecli ${CONN}:${FS}:ul "$FILE_BACKUP_ORI" $TEST_DEVICE:$FILE_PATH
   fi
-  rm -f "$FILE"
-  rm -f "$FILE_BACKUP"
+  rm -f "$FILE" "$FILE_BACKUP" "$FILE_BACKUP_ORI"
   exit $1
 }
 
+function getDownloadName() {
+  if [ -z "$download_ext" ]; then
+    echo "$srcdir/$DEVICE_NAME $FS"*
+  else
+    echo "$srcdir/"*$download_ext
+  fi
+}
+
 echo "Using device $TEST_DEVICE..."
 DEVICE_NAME=$(elektroid-cli info $TEST_DEVICE | grep "Device name:" | awk -F': ' '{print $2}')
 echo "Device name: $DEVICE_NAME"
 
 echo "Cleaning up previous executions..."
-rm -f "$srcdir/$DEVICE_NAME $FS"*
-rm -f "$srcdir/$BACKUP_PREFIX$DEVICE_NAME $FS"*
+rm -f "$(getDownloadName)"
+rm -f "$srcdir/$BACKUP_PREFIX$(getDownloadName)"
 
 echo "Testing ls..."
-files=$($ecli ${CONN}-${FS}-ls $TEST_DEVICE:$DIR_PATH)
+files=$($ecli ${CONN}:${FS}:ls $TEST_DEVICE:$DIR_PATH)
 [ $? -ne 0 ] && exit 1
 echo "$files" | head
 [ $(echo "$files" | wc -l) -ne $LS_ROWS ] && exit 1
@@ -45,45 +57,47 @@ echo "$files" | head
 if $download; then
   for p in $BAD_FILE_PATHS; do
     echo "Testing download with bad path $p..."
-    $ecli ${CONN}-${FS}-dl $TEST_DEVICE:$p
+    $ecli ${CONN}:${FS}:dl $TEST_DEVICE:$p
     [ $? -eq 0 ] && exit 1
   done
 
   echo "Testing download with path $FILE_PATH..."
-  $ecli ${CONN}-${FS}-dl $TEST_DEVICE:$FILE_PATH
+  $ecli ${CONN}:${FS}:dl $TEST_DEVICE:$FILE_PATH
   [ $? -ne 0 ] && exit 1
-  FILE=$(echo "$srcdir/$DEVICE_NAME $FS"*)
+  FILE=$(getDownloadName)
+  echo "Using file $FILE as the download file..."
   [ ! -f "$FILE" ] && exit 1
-  FILE_BACKUP=$srcdir/$BACKUP_PREFIX$(basename "$FILE")
+  FILE_BACKUP=/tmp/$BACKUP_PREFIX$(basename "$FILE")
   mv "$FILE" "$FILE_BACKUP"
 fi
 
 for p in $BAD_FILE_PATHS; do
   echo "Testing upload with bad path $p..."
-  $ecli ${CONN}-${FS}-ul $FILE_TO_UPLOAD $TEST_DEVICE:$p
+  $ecli ${CONN}:${FS}:ul $FILE_TO_UPLOAD $TEST_DEVICE:$p
   [ $? -eq 0 ] && exitWithError 1
 done
 
 echo "Testing upload with non existing file to $FILE_PATH..."
-$ecli ${CONN}-${FS}-ul foo $TEST_DEVICE:$FILE_PATH
+$ecli ${CONN}:${FS}:ul foo $TEST_DEVICE:$FILE_PATH
 [ $? -eq 0 ] && exitWithError 1
 
 echo "Testing upload with path $FILE_PATH..."
-$ecli ${CONN}-${FS}-ul $FILE_TO_UPLOAD $TEST_DEVICE:$FILE_PATH
+$ecli ${CONN}:${FS}:ul $FILE_TO_UPLOAD $TEST_DEVICE:$FILE_PATH
 [ $? -ne 0 ] && exitWithError 1
 
 if [ -n "$FILE_NEW_NAME" ]; then
   echo "Testing mv of $FILE_PATH to $FILE_NEW_NAME..."
-  $ecli ${CONN}-${FS}-mv $TEST_DEVICE:$FILE_PATH "$FILE_NEW_NAME"
+  $ecli ${CONN}:${FS}:mv $TEST_DEVICE:$FILE_PATH "$FILE_NEW_NAME"
   [ $? -ne 0 ] && exitWithError 1
 fi
 
 if $download; then
   echo "Testing data changes..."
-  $ecli ${CONN}-${FS}-dl $TEST_DEVICE:$FILE_PATH
+  $ecli ${CONN}:${FS}:dl $TEST_DEVICE:$FILE_PATH
   [ $? -ne 0 ] && exitWithError 1
-  FILE=$(echo "$srcdir/$DEVICE_NAME $FS"*)
+  FILE=$(getDownloadName)
   [ ! -f "$FILE" ] && exitWithError 1
+  echo "Comparing $FILE to $FILE_UPLOADED_BACK..."
   cksum_act=$(cksum "$FILE" | awk '{print $1}')
   cksum_exp=$(cksum "$FILE_UPLOADED_BACK" | awk '{print $1}')
   echo "Actual cksum: $cksum_act"
