@@ -3,7 +3,7 @@
 #Zip files include date information, hence generic testing is not possible.
 
 echo "Backing up wavetable..."
-$ecli microfreak-pwavetable-dl $TEST_DEVICE:/16
+$ecli microfreak:pwavetable:dl $TEST_DEVICE:/16
 [ $? -ne 0 ] && exit 1
 FILE=$(echo "$srcdir/Arturia MicroFreak pwavetable 16"*.mfw)
 BACKUP=$srcdir/backup.mfw
@@ -13,7 +13,7 @@ $srcdir/integration/generic_fs_tests.sh --no-download microfreak zwavetable / 16
 [ $? -ne 0 ] && exit 1
 
 echo "Testing download..."
-$ecli microfreak-zwavetable-dl $TEST_DEVICE:/16
+$ecli microfreak:zwavetable:dl $TEST_DEVICE:/16
 [ $? -ne 0 ] && exit 1
 FILE=$(echo "$srcdir/Arturia MicroFreak zwavetable 16"*.mfwz)
 [ ! -f "$FILE" ] && exit 1
@@ -23,7 +23,7 @@ act=$(unzip -p "$FILE" "0_wavetable" | cksum | awk '{print $1}')
 rm "$FILE"
 
 echo "Restoring wavetable..."
-$ecli microfreak-pwavetable-ul $BACKUP $TEST_DEVICE:/16
+$ecli microfreak:pwavetable:ul $BACKUP $TEST_DEVICE:/16
 rm "$BACKUP"
 
 exit $?
