@@ -84,7 +84,7 @@ test_elektron_configure_device_from_file ()
   elektron_configure_device_from_file (&backend, 12, TEST_DATA_DIR
 				       "/../../res/elektron/devices.json");
 
-  CU_ASSERT_EQUAL (elektron_data->device_desc.fs_descs_len, 4);
+  CU_ASSERT_EQUAL (elektron_data->device_desc.fs_descs_len, 7);
 
   ops = backend_get_fs_operations_by_name (&backend, "sample");
   exts = ops->get_exts (&backend, ops);
@@ -102,6 +102,18 @@ test_elektron_configure_device_from_file ()
   exts = ops->get_exts (&backend, ops);
   CU_ASSERT_EQUAL (compare_exts (exts, SND), 0);
 
+  ops = backend_get_fs_operations_by_name (&backend, "ram");
+  exts = ops->get_exts (&backend, ops);
+  CU_ASSERT_NOT_EQUAL (exts, NULL);	//Several sample extensions
+
+  ops = backend_get_fs_operations_by_name (&backend, "track");
+  exts = ops->get_exts (&backend, ops);
+  CU_ASSERT_NOT_EQUAL (exts, NULL);	//Several sample extensions
+
+  ops = backend_get_fs_operations_by_name (&backend, "track-loop");
+  exts = ops->get_exts (&backend, ops);
+  CU_ASSERT_NOT_EQUAL (exts, NULL);	//Several sample extensions
+
   CU_ASSERT_EQUAL (elektron_data->device_desc.storage, 3);
 
   elektron_destroy_data (&backend);
@@ -147,7 +159,7 @@ test_elektron_special_exts ()
   exts = ops->get_exts (&backend, ops);
   CU_ASSERT_EQUAL (compare_exts (exts, DTII_PRJ), 0);
 
-  ops = backend_get_fs_operations_by_name (&backend, "preset");
+  ops = backend_get_fs_operations_by_name (&backend, "preset-takt-ii");
   exts = ops->get_exts (&backend, ops);
   CU_ASSERT_EQUAL (compare_exts (exts, DTII_PST), 0);
 
@@ -165,7 +177,7 @@ test_elektron_special_exts ()
   exts = ops->get_exts (&backend, ops);
   CU_ASSERT_EQUAL (compare_exts (exts, DNII_PRJ), 0);
 
-  ops = backend_get_fs_operations_by_name (&backend, "preset");
+  ops = backend_get_fs_operations_by_name (&backend, "preset-takt-ii");
   exts = ops->get_exts (&backend, ops);
   CU_ASSERT_EQUAL (compare_exts (exts, DNII_PST), 0);
 
